Package: aridagri
Type: Package
Title: Comprehensive Statistical Tools for Agricultural Research
Version: 2.0.4
Authors@R: c(
    person("Lalit Kumar", "Rolaniya", 
           email = "lalitrolaniya@gmail.com", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8908-1211")),
    person("Ram Lal", "Jat", 
           email = "khokharagro1986@gmail.com", 
           role = "aut",
           comment = c(ORCID = "0009-0003-4339-0555")),
    person("Monika", "Punia", 
           email = "monikapunia02@gmail.com", 
           role = "aut",
           comment = c(ORCID = "0009-0002-0294-6767")),
    person("Raja Ram", "Choudhary", 
           email = "rajaramchoudhary26@gmail.com", 
           role = "aut"))
Description: A comprehensive suite of statistical and analytical tools for 
    agricultural research. Includes complete analysis of variance (ANOVA) 
    functions for all experimental designs: Completely Randomized Design (CRD), 
    Randomized Block Design (RBD), Pooled RBD, Split Plot with all variations, 
    Split-Split Plot, Strip Plot, Latin Square, Factorial, Augmented, and Alpha 
    Lattice, with proper error terms and comprehensive Standard Error (SE) and 
    Critical Difference (CD) calculations. Features multiple post-hoc tests: 
    Least Significant Difference (LSD), Duncan Multiple Range Test (DMRT), 
    Tukey Honestly Significant Difference (HSD), Student-Newman-Keuls (SNK), 
    Scheffe, Bonferroni, and Dunnett, along with assumption checking and 
    publication-ready output. Advanced methods include stability analysis using 
    Eberhart-Russell regression, Additive Main Effects and Multiplicative 
    Interaction (AMMI), Finlay-Wilkinson regression, Shukla stability variance, 
    Wricke ecovalence, Coefficient of Variation (CV), and Cultivar Superiority 
    Index as described in Eberhart and Russell (1966) 
    <doi:10.2135/cropsci1966.0011183X000600010011x>. Thermal indices include 
    Growing Degree Days (GDD), Heliothermal Units (HTU), Photothermal Units 
    (PTU), and Heat Use Efficiency (HUE). Crop growth analysis covers Crop 
    Growth Rate (CGR), Relative Growth Rate (RGR), Net Assimilation Rate (NAR), 
    and Leaf Area Index (LAI). Also provides harvest index, yield gap analysis, 
    economic efficiency indices (Benefit-Cost ratio), nutrient use efficiency 
    calculations, correlation matrix, Principal Component Analysis (PCA), path 
    analysis, and Structural Equation Modeling (SEM). Statistical methods follow 
    Gomez and Gomez (1984, ISBN:0471870927) and Panse and Sukhatme (1985, 
    ISBN:8170271169).
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
Imports: grDevices, graphics, stats, utils
Suggests: ggplot2, dplyr, tidyr, emmeans, multcomp, corrplot,
        factoextra, FactoMineR, lavaan, semPlot, agricolae, car,
        writexl, rmarkdown, testthat (>= 3.0.0)
URL: https://github.com/lalitrolaniya/aridagri
BugReports: https://github.com/lalitrolaniya/aridagri/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-10 17:28:37 UTC; lalitrolaniya
Author: Lalit Kumar Rolaniya [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8908-1211>),
  Ram Lal Jat [aut] (ORCID: <https://orcid.org/0009-0003-4339-0555>),
  Monika Punia [aut] (ORCID: <https://orcid.org/0009-0002-0294-6767>),
  Raja Ram Choudhary [aut]
Maintainer: Lalit Kumar Rolaniya <lalitrolaniya@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-10 18:00:02 UTC
Built: R 4.6.0; ; 2026-07-10 18:26:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(arid_plot,default)
S3method(print,aridagri_path)
S3method(print,aridagri_pca)
S3method(print,aridagri_stability)
export(anova_alpha_lattice)
export(anova_augmented)
export(anova_crd)
export(anova_factorial)
export(anova_factorial_3way)
export(anova_latin)
export(anova_rbd)
export(anova_rbd_pooled)
export(anova_spd)
export(anova_spd_ab_cd)
export(anova_spd_ab_main)
export(anova_spd_c_main_ab_sub)
export(anova_spd_pooled)
export(anova_sspd)
export(anova_sspd_pooled)
export(anova_strip)
export(arid_plot)
export(check_assumptions)
export(correlation_analysis)
export(crop_growth_analysis)
export(economic_analysis)
export(economic_indices)
export(export_results)
export(harvest_index)
export(nue_calculate)
export(nutrient_response)
export(path_analysis)
export(pca_analysis)
export(perform_posthoc)
export(sem_analysis)
export(stability_analysis)
export(thermal_indices)
export(yield_gap_analysis)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,bartlett.test)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,df.residual)
importFrom(stats,fitted)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,qtukey)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(utils,head)

# aridagri <img src="man/figures/logo.png" align="right" height="139" />

<!-- badges: start -->
[![R-CMD-check](https://img.shields.io/badge/R--CMD--check-passing-brightgreen.svg)](https://cran.r-project.org/)
[![License: GPL-3](https://img.shields.io/badge/License-GPL3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.18647782.svg)](https://doi.org/10.5281/zenodo.18647782)
<!-- badges: end -->

## Comprehensive Statistical Tools for Agricultural Research

**aridagri** is a comprehensive R package providing **35+ functions** for statistical analysis in agricultural research, with special focus on experimental design analysis and agronomic calculations.

### Key Features

- 🔬 **Complete ANOVA Suite**: All experimental designs with proper error terms
- 📊 **Multiple Post-Hoc Tests**: LSD, Duncan, Tukey, SNK, Scheffé, Bonferroni, Dunnett
- 📈 **Stability Analysis**: 7 methods (Eberhart-Russell, AMMI, Finlay-Wilkinson, Shukla, Wricke, CV, Superiority)
- 🌡️ **Thermal Indices**: GDD, HTU, PTU, Heat Use Efficiency
- 🌱 **Crop Growth Analysis**: CGR, RGR, NAR, LAI
- 📋 **Publication-Ready Output**: Formatted tables with SE, CD, CV

---

## Installation

```r
# Install devtools if needed
install.packages("devtools")

# Install aridagri from GitHub
devtools::install_github("lalitrolaniya/aridagri")
```

---

## Function Overview

### Experimental Design ANOVA (17 Functions)

| Function | Design |
|----------|--------|
| `anova_crd()` | Completely Randomized Design |
| `anova_rbd()` | Randomized Block Design |
| `anova_rbd_pooled()` | Pooled RBD (Multi-Environment) |
| `anova_latin()` | Latin Square Design |
| `anova_factorial()` | Two-Factor Factorial |
| `anova_factorial_3way()` | Three-Factor Factorial |
| `anova_spd()` | Split Plot Design |
| `anova_spd_ab_main()` | SPD with (A×B) in Main Plot |
| `anova_spd_c_main_ab_sub()` | SPD with C Main, (A×B) Sub |
| `anova_spd_ab_cd()` | SPD with (A×B) Main, (C×D) Sub |
| `anova_spd_pooled()` | Pooled Split Plot Design |
| `anova_sspd()` | Split-Split Plot Design |
| `anova_sspd_pooled()` | Pooled SSPD |
| `anova_strip()` | Strip Plot Design |
| `anova_augmented()` | Augmented Block Design |
| `anova_alpha_lattice()` | Alpha Lattice Design |

### Post-Hoc Tests (8 Methods)

All available via `perform_posthoc()`:
- Fisher's LSD
- Duncan's Multiple Range Test (DMRT)
- Tukey's HSD
- Student-Newman-Keuls (SNK)
- Scheffé's Test
- Bonferroni Correction
- Dunnett's Test (vs Control)

### Agronomic Analysis (6 Functions)

| Function | Analysis |
|----------|----------|
| `stability_analysis()` | 7-method stability analysis |
| `thermal_indices()` | GDD, HTU, PTU, HUE |
| `crop_growth_analysis()` | CGR, RGR, NAR, LAI |
| `harvest_index()` | HI and partitioning |
| `yield_gap_analysis()` | Yield gap calculations |
| `economic_indices()` | B:C ratio, net returns |

### Statistical Analysis (4 Functions)

| Function | Analysis |
|----------|----------|
| `correlation_analysis()` | Correlation matrix with significance |
| `pca_analysis()` | Principal Component Analysis |
| `path_analysis()` | Path coefficients |
| `sem_analysis()` | Structural Equation Modeling |

### Nutrient Analysis (3 Functions)

| Function | Analysis |
|----------|----------|
| `nue_calculate()` | Nutrient Use Efficiency indices |
| `nutrient_response()` | Response curve analysis |
| `economic_analysis()` | Economic optimum |

---

## Quick Examples

### Split-Split Plot Design ANOVA

```r
library(aridagri)

data <- expand.grid(
  rep = 1:3,
  irrigation = c("I1", "I2", "I3"),
  variety = c("V1", "V2"),
  nitrogen = c("N0", "N40", "N80")
)
data$yield <- rnorm(nrow(data), 1200, 150)

result <- anova_sspd(data, 
                     response = "yield",
                     main_plot = "irrigation",
                     sub_plot = "variety",
                     sub_sub_plot = "nitrogen",
                     replication = "rep")
```

### Stability Analysis (7 Methods)

```r
data <- expand.grid(
  variety = paste0("V", 1:10),
  location = paste0("L", 1:5),
  rep = 1:3
)
data$yield <- rnorm(nrow(data), 1200, 200)

stability_analysis(data, 
                   genotype = "variety",
                   environment = "location",
                   replication = "rep",
                   trait = "yield",
                   method = "all")
```

---

## Unique Features

1. **First R package** with ALL split plot design variations
2. **Complete SE/CD calculations** for every comparison type
3. **7 stability analysis methods** in single function
4. **Integrated thermal indices** (GDD, HTU, PTU, HUE)
5. **Crop growth analysis** (CGR, RGR, NAR, LAI)
6. **Publication-ready formatted output**

---

## Citation

```
Rolaniya, L.K., Jat, R.L., Punia, M., and Choudhary, R.R. (2026). aridagri: 
Comprehensive Statistical Tools for Agricultural Research. 
R package version 2.0.3. https://github.com/lalitrolaniya/aridagri
```

---

## Authors

**Lalit Kumar Rolaniya** (Maintainer)
Scientist (Agronomy)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0000-0001-8908-1211](https://orcid.org/0000-0001-8908-1211)

**Ram Lal Jat**
Senior Scientist (Agronomy)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0009-0003-4339-0555](https://orcid.org/0009-0003-4339-0555)

**Monika Punia**
Scientist (Genetics & Plant Breeding)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0009-0002-0294-6767](https://orcid.org/0009-0002-0294-6767)

**Raja Ram Choudhary**
Scientist (Agronomy)
ICAR-Indian Institute of Groundnut Research, Regional Research Station, Bikaner, Rajasthan-334006, India

---

## Acknowledgments

The authors gratefully acknowledge **ICAR-Indian Institute of Pulses Research, Kanpur** for providing necessary support and infrastructure for the development of this package.

---

## License

GPL-3

---

## Contributing

Contributions are welcome! Please feel free to submit issues or pull requests.

# aridagri <img src="man/figures/logo.png" align="right" height="139" />

<!-- badges: start -->
[![R-CMD-check](https://img.shields.io/badge/R--CMD--check-passing-brightgreen.svg)](https://cran.r-project.org/)
![License: GPL-3](https://img.shields.io/badge/License-GPL3-blue.svg)
[![CRAN](https://www.r-pkg.org/badges/version/aridagri)](https://cran.r-project.org/package=aridagri)
[![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.aridagri-blue)](https://doi.org/10.32614/CRAN.package.aridagri)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/aridagri)](https://cran.r-project.org/package=aridagri)
<!-- badges: end -->

## Comprehensive Statistical Tools for Agricultural Research

**aridagri** is an R package providing **33 functions** for statistical analysis
in agricultural research, with a focus on experimental design analysis and
agronomic calculations. It is written in base R with no hard dependencies.

### Key Features

- **ANOVA suite** covering all common field designs
- **Post-hoc tests**: LSD, Duncan (DMRT), Tukey, SNK, Scheffe, Bonferroni, Dunnett
- **Stability analysis**: 7 methods (Eberhart-Russell, AMMI, Finlay-Wilkinson, Shukla, Wricke, CV, Lin-Binns) with an integrated ranking
- **Thermal indices**: GDD, HTU, PTU, Heat Use Efficiency
- **Crop growth analysis**: CGR, RGR, NAR
- **Publication-ready output**: formatted tables with SE, CD, CV

---

## Installation

```r
# From CRAN
install.packages("aridagri")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("lalitrolaniya/aridagri")
```

---

## Function Overview

### Experimental Design ANOVA (16 functions)

| Function | Design |
|----------|--------|
| `anova_crd()` | Completely Randomized Design |
| `anova_rbd()` | Randomized Block Design |
| `anova_rbd_pooled()` | Pooled RBD (multi-environment) |
| `anova_latin()` | Latin Square Design |
| `anova_factorial()` | Two-Factor Factorial |
| `anova_factorial_3way()` | Three-Factor Factorial |
| `anova_spd()` | Split-Plot Design |
| `anova_spd_ab_main()` | SPD with (A x B) in main plot |
| `anova_spd_c_main_ab_sub()` | SPD with C main, (A x B) sub |
| `anova_spd_ab_cd()` | SPD with (A x B) main, (C x D) sub |
| `anova_spd_pooled()` | Pooled Split-Plot Design |
| `anova_sspd()` | Split-Split-Plot Design |
| `anova_sspd_pooled()` | Pooled SSPD |
| `anova_strip()` | Strip-Plot Design |
| `anova_augmented()` | Augmented Block Design |
| `anova_alpha_lattice()` | Alpha Lattice Design |

### Stability, agronomic, statistical and nutrient functions

| Function | Analysis |
|----------|----------|
| `stability_analysis()` | 7 stability methods with integrated ranking |
| `thermal_indices()` | GDD, HTU, PTU, HUE |
| `crop_growth_analysis()` | CGR, RGR, NAR |
| `harvest_index()` | Harvest index and partitioning |
| `yield_gap_analysis()` | Yield gap calculations |
| `economic_indices()` | B:C ratio, net returns |
| `correlation_analysis()` | Correlation matrix with significance |
| `pca_analysis()` | Principal Component Analysis |
| `path_analysis()` | Path coefficient analysis |
| `sem_analysis()` | Structural Equation Modeling |
| `nue_calculate()` | Nutrient Use Efficiency indices |
| `nutrient_response()` | Response curve and economic optimum |
| `economic_analysis()` | Gross/net return, B:C ratio |

### Post-hoc, diagnostics and utilities

| Function | Purpose |
|----------|---------|
| `perform_posthoc()` | Post-hoc comparisons (7 methods) |
| `check_assumptions()` | Normality, homogeneity, independence, outliers |
| `arid_plot()` | Base-graphics plots for ANOVA, correlation and stability objects |
| `export_results()` | Export results to Excel or CSV |

---

## Usage

Every function is called with the data frame first, then the column names as
character strings. Set `verbose = FALSE` to suppress the printed report and
just capture the returned object.

### Analysis of variance

```r
library(aridagri)

## Completely Randomized Design
crd <- data.frame(
  treatment = factor(rep(c("T1", "T2", "T3", "T4"), each = 4)),
  yield     = c(20, 22, 19, 21,  25, 27, 24, 26,
                30, 32, 29, 31,  28, 30, 27, 29)
)

res <- anova_crd(crd, response = "yield", treatment = "treatment")

## the returned object holds the table, means and statistics
res$anova_table
res$treatment_means
res$cv

## Randomized Block Design
anova_rbd(data, response = "yield", treatment = "variety", block = "block")

## Two-factor factorial
anova_factorial(data, response = "yield", factor1 = "nitrogen", factor2 = "variety")

## Split-plot design
anova_spd(data, response = "yield",
          main_plot = "irrigation", sub_plot = "variety", replication = "rep")
```

### Post-hoc comparisons

There are two equivalent ways to run a post-hoc test.

```r
## Option 1: inline, as part of the ANOVA call
anova_crd(crd, "yield", "treatment", posthoc = "tukey")

## Option 2: standalone, on a fitted aov() model
model <- aov(yield ~ treatment, data = crd)
perform_posthoc(model, crd, response = "yield", treatment = "treatment",
                posthoc = "tukey")
```

Valid method names are `"lsd"`, `"duncan"`, `"tukey"`, `"snk"`, `"scheffe"`,
`"dunnett"`, `"bonferroni"`, or `"all"`. Dunnett compares every level against
the **first** factor level as the control; there is no separate `control`
argument, so order the treatment factor with the control first if needed.

### Assumption checks

```r
model <- aov(yield ~ treatment, data = crd)
check_assumptions(model)   # Shapiro-Wilk, Bartlett, Durbin-Watson, outliers
```

### Split-Split-Plot Design

```r
data <- expand.grid(
  rep        = 1:3,
  irrigation = c("I1", "I2", "I3"),
  variety    = c("V1", "V2"),
  nitrogen   = c("N0", "N40", "N80")
)
data$yield <- rnorm(nrow(data), 1200, 150)

anova_sspd(data,
           response     = "yield",
           main_plot    = "irrigation",
           sub_plot     = "variety",
           sub_sub_plot = "nitrogen",
           replication  = "rep")
```

### Stability analysis (7 methods)

```r
data <- expand.grid(
  variety  = paste0("V", 1:10),
  location = paste0("L", 1:5),
  rep      = 1:3
)
data$yield <- rnorm(nrow(data), 1200, 200)

stability_analysis(data,
                   genotype    = "variety",
                   environment = "location",
                   replication = "rep",
                   trait       = "yield",
                   method      = "all")
```

### Correlation, PCA and visualization

```r
df <- data.frame(
  yield   = rnorm(30, 1200, 150),
  pods    = rnorm(30, 50, 6),
  biomass = rnorm(30, 3500, 400)
)

cor_res <- correlation_analysis(df, method = "pearson")
pca_res <- pca_analysis(df, scale = TRUE)

## arid_plot() draws factor/treatment means for any ANOVA design,
## a heatmap for a correlation object, and a ranking for a stability object
arid_plot(res)
arid_plot(cor_res)
```

### Exporting results

```r
export_results(res, "results.xlsx")                  # Excel (default)
export_results(res, "results.csv", format = "csv")   # CSV
```

---

## Unique Features

1. All split-plot design variations in one package
2. SE and CD reported for each comparison type
3. Seven stability analysis methods in a single function
4. Thermal indices (GDD, HTU, PTU, HUE) and crop growth analysis (CGR, RGR, NAR)
5. Base-R implementation with no hard dependencies

---

## Citation

```
Rolaniya, L.K., Jat, R.L., Punia, M., and Choudhary, R.R. (2026). aridagri:
Comprehensive Statistical Tools for Agricultural Research.
R package version 2.0.4. https://github.com/lalitrolaniya/aridagri
```

---

## Authors

**Lalit Kumar Rolaniya** (Maintainer)
Scientist (Agronomy)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0000-0001-8908-1211](https://orcid.org/0000-0001-8908-1211)

**Ram Lal Jat**
Senior Scientist (Agronomy)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0009-0003-4339-0555](https://orcid.org/0009-0003-4339-0555)

**Monika Punia**
Scientist (Genetics & Plant Breeding)
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner, Rajasthan-334006, India
ORCID: [0009-0002-0294-6767](https://orcid.org/0009-0002-0294-6767)

**Raja Ram Choudhary**
Scientist (Agronomy)
ICAR-Indian Institute of Groundnut Research, Regional Research Station, Bikaner, Rajasthan-334006, India

---

## License

GPL-3

---

## Contributing

Contributions are welcome. Please submit issues or pull requests on GitHub.

#' Nutrient Use Efficiency Calculations
#'
#' @description
#' Comprehensive nutrient use efficiency calculations including Agronomic Efficiency (AE),
#' Physiological Efficiency (PE), Apparent Recovery Efficiency (ARE), and Partial Factor
#' Productivity (PFP). Essential for INM research in arid regions.
#'
#' @param yield_fertilized Yield with fertilizer application (kg/ha)
#' @param yield_control Yield in control/unfertilized plot (kg/ha)
#' @param nutrient_applied Amount of nutrient applied (kg/ha)
#' @param nutrient_uptake_fert Nutrient uptake in fertilized plot (kg/ha), optional
#' @param nutrient_uptake_ctrl Nutrient uptake in control plot (kg/ha), optional
#' @param biomass_fert Total biomass in fertilized plot (kg/ha), optional
#' @param biomass_ctrl Total biomass in control plot (kg/ha), optional
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with efficiency indices
#'
#' @details
#' Efficiency calculations:
#' \itemize{
#'   \item AE (Agronomic Efficiency) = (Yield_fert - Yield_ctrl) / Nutrient_applied
#'   \item PFP (Partial Factor Productivity) = Yield_fert / Nutrient_applied
#'   \item ARE (Apparent Recovery Efficiency) = (Uptake_fert - Uptake_ctrl) / Nutrient_applied  100
#'   \item PE (Physiological Efficiency) = (Yield_fert - Yield_ctrl) / (Uptake_fert - Uptake_ctrl)
#' }
#'
#' @examples
#' # Basic NUE calculation
#' nue_calculate(yield_fertilized = 1850, yield_control = 1200, nutrient_applied = 40)
#'
#' # Complete NUE with uptake data
#' nue_calculate(
#'   yield_fertilized = 1850, 
#'   yield_control = 1200,
#'   nutrient_applied = 40,
#'   nutrient_uptake_fert = 65,
#'   nutrient_uptake_ctrl = 35
#' )
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
nue_calculate <- function(yield_fertilized, yield_control, nutrient_applied,
                          nutrient_uptake_fert = NULL, nutrient_uptake_ctrl = NULL,
                          biomass_fert = NULL, biomass_ctrl = NULL,
                            verbose = TRUE) {
  
  # Input validation
  if (nutrient_applied <= 0) {
    stop("nutrient_applied must be greater than 0")
  }
  
  # Initialize results
  results <- data.frame(
    Yield_Fertilized = yield_fertilized,
    Yield_Control = yield_control,
    Nutrient_Applied = nutrient_applied
  )
  
  # Agronomic Efficiency (AE) - kg grain per kg nutrient
  ae <- (yield_fertilized - yield_control) / nutrient_applied
  results$AE_kg_kg <- round(ae, 2)
  
  # Partial Factor Productivity (PFP) - kg grain per kg nutrient
  pfp <- yield_fertilized / nutrient_applied
  results$PFP_kg_kg <- round(pfp, 2)
  
  # Apparent Recovery Efficiency (ARE) - if uptake data available
  if (!is.null(nutrient_uptake_fert) && !is.null(nutrient_uptake_ctrl)) {
    are <- ((nutrient_uptake_fert - nutrient_uptake_ctrl) / nutrient_applied) * 100
    results$ARE_percent <- round(are, 1)
    
    # Physiological Efficiency (PE)
    uptake_diff <- nutrient_uptake_fert - nutrient_uptake_ctrl
    if (uptake_diff > 0) {
      pe <- (yield_fertilized - yield_control) / uptake_diff
      results$PE_kg_kg <- round(pe, 2)
    }
    
    # Utilization Efficiency (UE) = AE  ARE
    ue <- ae * (are / 100)
    results$UE_kg_kg <- round(ue, 2)
  }
  
  # Harvest Index if biomass available
  if (!is.null(biomass_fert)) {
    hi_fert <- (yield_fertilized / biomass_fert) * 100
    results$HI_Fertilized <- round(hi_fert, 1)
  }
  if (!is.null(biomass_ctrl)) {
    hi_ctrl <- (yield_control / biomass_ctrl) * 100
    results$HI_Control <- round(hi_ctrl, 1)
  }
  
  # Print summary
  if (verbose) {
    cat("\n=== Nutrient Use Efficiency Analysis ===\n")
    cat("Yield (Fertilized):", yield_fertilized, "kg/ha\n")
    cat("Yield (Control):", yield_control, "kg/ha\n")
    cat("Yield Response:", yield_fertilized - yield_control, "kg/ha\n")
    cat("Nutrient Applied:", nutrient_applied, "kg/ha\n")
    cat("\n--- Efficiency Indices ---\n")
    cat("Agronomic Efficiency (AE):", round(ae, 2), "kg grain/kg nutrient\n")
    cat("Partial Factor Productivity (PFP):", round(pfp, 2), "kg grain/kg nutrient\n")

  }
  if (!is.null(nutrient_uptake_fert) && !is.null(nutrient_uptake_ctrl)) {
    if (verbose) {
      cat("Apparent Recovery Efficiency (ARE):", round(are, 1), "%\n")
      if (exists("pe")) {
        cat("Physiological Efficiency (PE):", round(pe, 2), "kg grain/kg uptake\n")
      }
      cat("Utilization Efficiency (UE):", round(ue, 2), "kg grain/kg nutrient\n")
    }
  }
  
  class(results) <- c("aridagri_nue", "data.frame")
  return(invisible(results))
}


#' Nutrient Response Curve Analysis
#'
#' @description
#' Fits nutrient response curves using quadratic, linear-plateau, or 
#' Mitscherlich models to determine economic optimum dose.
#'
#' @param dose Numeric vector of nutrient doses (kg/ha)
#' @param yield Numeric vector of corresponding yields (kg/ha)
#' @param model Model type: "quadratic", "linear_plateau", or "mitscherlich"
#' @param price_output Price of output (Rs/kg)
#' @param price_nutrient Price of nutrient (Rs/kg)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List with model parameters and economic optimum
#'
#' @examples
#' dose <- c(0, 20, 40, 60, 80, 100)
#' yield <- c(1100, 1350, 1520, 1610, 1650, 1660)
#' nutrient_response(dose, yield, model = "quadratic", 
#'                   price_output = 60, price_nutrient = 15)
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
nutrient_response <- function(dose, yield, model = "quadratic",
                               price_output = 50, price_nutrient = 15,
                            verbose = TRUE) {
  
  # Input validation
  if (length(dose) != length(yield)) {
    stop("dose and yield must have same length")
  }
  
  if (model == "quadratic") {
    # Fit quadratic model: Y = a + bX + cX
    fit <- lm(yield ~ dose + I(dose^2))
    coef_vals <- coef(fit)
    a <- coef_vals[1]
    b <- coef_vals[2]
    c <- coef_vals[3]
    
    # Maximum yield dose (dy/dx = 0)
    dose_max <- -b / (2 * c)
    yield_max <- predict(fit, newdata = data.frame(dose = dose_max))
    
    # Economic optimum dose (dy/dx = price ratio)
    price_ratio <- price_nutrient / price_output
    dose_econ <- (price_ratio - b) / (2 * c)
    yield_econ <- predict(fit, newdata = data.frame(dose = dose_econ))
    
    # Model statistics
    r_squared <- summary(fit)$r.squared
    
    if (verbose) {
      cat("\n=== Nutrient Response Curve Analysis ===\n")
      cat("Model: Quadratic (Y = a + bX + cX)\n")
      cat("\n--- Model Parameters ---\n")
      cat("Intercept (a):", round(a, 2), "\n")
      cat("Linear coefficient (b):", round(b, 4), "\n")
      cat("Quadratic coefficient (c):", round(c, 6), "\n")
      cat("R-squared:", round(r_squared, 4), "\n")
      cat("\n--- Optimum Doses ---\n")
      cat("Dose for maximum yield:", round(dose_max, 1), "kg/ha\n")
      cat("Maximum yield:", round(yield_max, 0), "kg/ha\n")
      cat("Economic optimum dose:", round(dose_econ, 1), "kg/ha\n")
      cat("Yield at economic optimum:", round(yield_econ, 0), "kg/ha\n")
      cat("\n--- Price Parameters ---\n")
      cat("Output price: Rs", price_output, "/kg\n")
      cat("Nutrient price: Rs", price_nutrient, "/kg\n")
    }
    
    result <- list(
      model = "quadratic",
      coefficients = c(a = a, b = b, c = c),
      r_squared = r_squared,
      dose_max_yield = round(dose_max, 1),
      max_yield = round(yield_max, 0),
      dose_economic = round(dose_econ, 1),
      yield_economic = round(yield_econ, 0),
      fit = fit
    )
    
  } else {
    stop("Currently only 'quadratic' model is implemented")
  }
  
  class(result) <- "aridagri_response"
  return(invisible(result))
}


#' Economic Analysis for Arid Agriculture
#'
#' @description
#' Calculates economic indicators including Cost of Cultivation, Gross Returns,
#' Net Returns, B:C Ratio, and profitability indices for arid farming systems.
#'
#' @param yield Crop yield (kg/ha)
#' @param price Market price (Rs/kg)
#' @param cost_fixed Fixed costs (Rs/ha)
#' @param cost_variable Variable costs (Rs/ha)
#' @param byproduct_yield Byproduct yield (kg/ha), optional
#' @param byproduct_price Byproduct price (Rs/kg), optional
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with economic analysis
#'
#' @examples
#' economic_analysis(yield = 1200, price = 65, cost_fixed = 15000, cost_variable = 12000)
#'
#' # With byproduct
#' economic_analysis(yield = 1200, price = 65, cost_fixed = 15000, 
#'                   cost_variable = 12000, byproduct_yield = 1800, byproduct_price = 5)
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
economic_analysis <- function(yield, price, cost_fixed, cost_variable,
                               byproduct_yield = 0, byproduct_price = 0,
                            verbose = TRUE) {
  
  # Calculate costs
  total_cost <- cost_fixed + cost_variable
  
  # Calculate returns
  gross_return_main <- yield * price
  gross_return_byproduct <- byproduct_yield * byproduct_price
  gross_return <- gross_return_main + gross_return_byproduct
  
  # Net returns
  net_return <- gross_return - total_cost
  
  # Profitability ratios
  bc_ratio <- gross_return / total_cost
  return_per_rupee <- net_return / total_cost
  
  # Per kg cost of production
  cost_of_production <- total_cost / yield
  
  if (verbose) {
    cat("\n=== Economic Analysis ===\n")
    cat("\n--- Costs (Rs/ha) ---\n")
    cat("Fixed Costs:", format(cost_fixed, big.mark = ","), "\n")
    cat("Variable Costs:", format(cost_variable, big.mark = ","), "\n")
    cat("Total Cost:", format(total_cost, big.mark = ","), "\n")
    cat("\n--- Returns (Rs/ha) ---\n")
    cat("Gross Return (main):", format(gross_return_main, big.mark = ","), "\n")
    if (byproduct_yield > 0) {
      cat("Gross Return (byproduct):", format(gross_return_byproduct, big.mark = ","), "\n")
    }
    cat("Total Gross Return:", format(gross_return, big.mark = ","), "\n")
    cat("Net Return:", format(net_return, big.mark = ","), "\n")
    cat("\n--- Profitability Indices ---\n")
    cat("B:C Ratio:", round(bc_ratio, 2), "\n")
    cat("Return per Rupee Invested:", round(return_per_rupee, 2), "\n")
    cat("Cost of Production:", round(cost_of_production, 2), "Rs/kg\n")
  }
  
  result <- data.frame(
    Yield_kg_ha = yield,
    Price_Rs_kg = price,
    Cost_Fixed = cost_fixed,
    Cost_Variable = cost_variable,
    Total_Cost = total_cost,
    Gross_Return = gross_return,
    Net_Return = net_return,
    BC_Ratio = round(bc_ratio, 2),
    Cost_of_Production = round(cost_of_production, 2)
  )
  
  class(result) <- c("aridagri_economics", "data.frame")
  return(invisible(result))
}

#' aridagri: Comprehensive Statistical Tools for Agricultural Research
#'
#' @description
#' The aridagri package provides comprehensive statistical and analytical tools 
#' for agricultural research. It includes complete ANOVA functions for all 
#' experimental designs, multiple post-hoc tests, stability analysis methods,
#' thermal indices, crop growth analysis, and advanced statistical methods.
#'
#' @section Experimental Design ANOVA Functions:
#' \itemize{
#'   \item \code{\link{anova_crd}}: Completely Randomized Design
#'   \item \code{\link{anova_rbd}}: Randomized Block Design
#'   \item \code{\link{anova_rbd_pooled}}: Pooled RBD (Multi-Environment)
#'   \item \code{\link{anova_latin}}: Latin Square Design
#'   \item \code{\link{anova_factorial}}: Two-Factor Factorial
#'   \item \code{\link{anova_factorial_3way}}: Three-Factor Factorial
#'   \item \code{\link{anova_spd}}: Split Plot Design
#'   \item \code{\link{anova_sspd}}: Split-Split Plot Design
#'   \item \code{\link{anova_strip}}: Strip Plot Design
#'   \item \code{\link{anova_augmented}}: Augmented Block Design
#'   \item \code{\link{anova_alpha_lattice}}: Alpha Lattice Design
#' }
#'
#' @section Post-Hoc Tests:
#' \itemize{
#'   \item \code{\link{perform_posthoc}}: Multiple comparison tests (LSD, Duncan, Tukey, SNK, Scheffe, Bonferroni, Dunnett)
#'   \item \code{\link{check_assumptions}}: ANOVA assumption checking
#' }
#'
#' @section Agronomic Analysis Functions:
#' \itemize{
#'   \item \code{\link{stability_analysis}}: Multi-method stability analysis (Eberhart-Russell, AMMI, Finlay-Wilkinson, Shukla, Wricke, CV, Superiority)
#'   \item \code{\link{thermal_indices}}: GDD, HTU, PTU, Heat Use Efficiency
#'   \item \code{\link{crop_growth_analysis}}: CGR, RGR, NAR, LAI
#'   \item \code{\link{harvest_index}}: Harvest index and partitioning
#'   \item \code{\link{yield_gap_analysis}}: Yield gap calculations
#'   \item \code{\link{economic_indices}}: B:C ratio, net returns
#' }
#'
#' @section Statistical Functions:
#' \itemize{
#'   \item \code{\link{correlation_analysis}}: Correlation matrix with significance
#'   \item \code{\link{pca_analysis}}: Principal component analysis
#'   \item \code{\link{path_analysis}}: Path coefficient analysis
#'   \item \code{\link{sem_analysis}}: Structural equation modeling
#' }
#'
#' @section Nutrient Analysis Functions:
#' \itemize{
#'   \item \code{\link{nue_calculate}}: Nutrient use efficiency calculations
#'   \item \code{\link{nutrient_response}}: Response curve analysis
#'   \item \code{\link{economic_analysis}}: Economic viability assessment
#' }
#'
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
#' @docType package
#' @name aridagri-package
#' @aliases aridagri
#'
#' @keywords package
"_PACKAGE"


#' Visualization Functions for aridagri
#'
#' @description
#' Generate publication-quality plots for aridagri analyses.
#'
#' @param x An object from aridagri analysis functions
#' @param type Plot type: "bar", "line", "interaction", "boxplot"
#' @param ... Additional arguments passed to plotting functions
#'
#' @return No return value; called for its side effect of drawing a plot on the
#'   current graphics device. The plot produced depends on the class of \code{x}:
#'   a correlation heatmap for \code{aridagri_correlation} objects, a bar chart of
#'   treatment means for \code{aridagri_anova} objects, and a bar chart of the
#'   integrated stability ranking for \code{aridagri_stability} objects. Uses only
#'   base graphics, so no additional packages are required.
#'
#' @examples
#' df <- data.frame(
#'   yield = c(1200, 1350, 1100, 1450, 1280),
#'   wue = c(4.2, 4.8, 3.9, 5.1, 4.5),
#'   protein = c(22.1, 23.5, 21.8, 24.2, 22.9)
#' )
#' result <- correlation_analysis(df, plot = FALSE)
#' arid_plot(result)
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
arid_plot <- function(x, type = "bar", ...) {
  UseMethod("arid_plot")
}


#' @export
arid_plot.default <- function(x, type = "bar", ...) {

  # --- Correlation objects: draw a heatmap of the correlation matrix ---------
  if (inherits(x, "aridagri_correlation")) {
    cormat <- as.matrix(x$correlation)
    p <- ncol(cormat)
    vars <- colnames(cormat)
    if (is.null(vars)) vars <- paste0("V", seq_len(p))

    # Blue (negative) - white (0) - red (positive) palette
    pal <- grDevices::colorRampPalette(c("#2166AC", "#FFFFFF", "#B2182B"))(101)
    brks <- seq(-1, 1, length.out = 102)

    op <- graphics::par(mar = c(6, 6, 3, 2))
    on.exit(graphics::par(op), add = TRUE)

    # image() plots columns bottom-to-top; reverse rows so it reads top-to-bottom
    graphics::image(seq_len(p), seq_len(p), t(cormat[p:1, , drop = FALSE]),
                    col = pal, breaks = brks, axes = FALSE, xlab = "", ylab = "",
                    main = paste0("Correlation matrix (",
                                  if (!is.null(x$method)) x$method else "pearson", ")"))
    graphics::axis(1, at = seq_len(p), labels = vars, las = 2, cex.axis = 0.8)
    graphics::axis(2, at = seq_len(p), labels = rev(vars), las = 2, cex.axis = 0.8)
    graphics::box()

    # Overlay the coefficients
    for (i in seq_len(p)) {
      for (j in seq_len(p)) {
        val <- cormat[p - i + 1, j]
        graphics::text(j, i, formatC(val, format = "f", digits = 2),
                       cex = 0.7,
                       col = if (abs(val) > 0.6) "white" else "black")
      }
    }
    return(invisible(NULL))
  }

  # --- ANOVA-family objects: bar chart(s) of factor / treatment means --------
  anova_classes <- c("aridagri_anova", "aridagri_factorial", "aridagri_factorial3",
                     "aridagri_latin", "aridagri_spd", "aridagri_spd_ab",
                     "aridagri_spd_cab", "aridagri_spd_abcd", "aridagri_strip",
                     "aridagri_alpha", "aridagri_augmented", "aridagri_pooled",
                     "aridagri_spd_pooled", "aridagri_sspd_pooled")
  means_fields <- c("treatment_means", "genotype_means", "main_means", "sub_means",
                    "env_means", "horizontal_means", "vertical_means",
                    "mean_a", "mean_b", "mean_c", "factor_means")
  if (inherits(x, anova_classes) || any(means_fields %in% names(x))) {

    # normalise a means data.frame to list(labs, mean, sd)
    norm_means <- function(df) {
      if (is.null(df) || !is.data.frame(df) || nrow(df) == 0L) return(NULL)
      num <- vapply(df, is.numeric, logical(1))
      mcol <- if ("Mean" %in% names(df)) "Mean" else if (any(num)) names(df)[num][1] else NA
      if (is.na(mcol)) return(NULL)
      lcol <- setdiff(names(df), c(mcol, "SD"))
      labs <- if (length(lcol)) as.character(df[[lcol[1]]]) else as.character(seq_len(nrow(df)))
      list(labs = labs, mean = as.numeric(df[[mcol]]),
           sd = if ("SD" %in% names(df)) as.numeric(df$SD) else NULL)
    }

    panels <- list()
    push <- function(title, df) {
      m <- norm_means(df)
      if (!is.null(m)) panels[[length(panels) + 1L]] <<- c(list(title = title), m)
    }
    if (!is.null(x$treatment_means))  push("Treatment means", x$treatment_means)
    if (!is.null(x$genotype_means))   push("Genotype means",  x$genotype_means)
    if (!is.null(x$main_means))       push("Main-plot means", x$main_means)
    if (!is.null(x$sub_means))        push("Sub-plot means",  x$sub_means)
    if (!is.null(x$env_means))        push("Environment means", x$env_means)
    if (!is.null(x$horizontal_means)) push("Horizontal factor", x$horizontal_means)
    if (!is.null(x$vertical_means))   push("Vertical factor",   x$vertical_means)
    if (!is.null(x$mean_a))           push("Factor A", x$mean_a)
    if (!is.null(x$mean_b))           push("Factor B", x$mean_b)
    if (!is.null(x$mean_c))           push("Factor C", x$mean_c)
    if (is.list(x$factor_means) && !is.data.frame(x$factor_means)) {
      fm <- x$factor_means
      nms <- if (!is.null(names(fm))) names(fm) else paste0("Factor ", seq_along(fm))
      for (k in seq_along(fm)) push(paste0("Factor ", nms[k]), fm[[k]])
    }

    # fall back: derive means from the fitted model's first treatment factor
    if (length(panels) == 0L && !is.null(x$model)) {
      mf <- tryCatch(stats::model.frame(x$model), error = function(e) NULL)
      if (!is.null(mf) && ncol(mf) >= 2L) {
        is_fac <- vapply(mf, function(z) is.factor(z) || is.character(z), logical(1))
        fac <- names(mf)[is_fac]
        fac <- fac[!tolower(fac) %in% c("rep", "reps", "replication", "block",
                                        "blocks", "rep_block")]
        if (!length(fac)) fac <- names(mf)[is_fac]
        if (length(fac)) {
          ag <- aggregate(mf[[1]], by = list(mf[[fac[1]]]), FUN = mean, na.rm = TRUE)
          names(ag) <- c(fac[1], "Mean")
          push(paste0(fac[1], " means"), ag)
        }
      }
    }

    if (length(panels) == 0L) {
      message("arid_plot(): no factor means available to plot for class ",
              paste(class(x), collapse = "/"), ".")
      return(invisible(NULL))
    }

    np <- length(panels)
    nc <- if (np <= 1L) 1L else if (np <= 4L) 2L else 3L
    nr <- ceiling(np / nc)
    op <- graphics::par(mfrow = c(nr, nc), mar = c(7, 5, 3, 2))
    on.exit(graphics::par(op), add = TRUE)

    dlab <- if (!is.null(x$design)) x$design else "ANOVA"
    cols <- c("#4292C6", "#41AB5D", "#EF6548", "#8C6BB1", "#FDAE6B", "#66C2A4")
    for (k in seq_len(np)) {
      pnl  <- panels[[k]]
      ord  <- order(-pnl$mean)
      mval <- pnl$mean[ord]; labs <- pnl$labs[ord]
      sdv  <- if (!is.null(pnl$sd)) pnl$sd[ord] else NULL
      ytop <- max(mval, if (!is.null(sdv)) mval + sdv else mval, na.rm = TRUE) * 1.15
      bp <- graphics::barplot(mval, names.arg = labs, las = 2,
                              col = cols[(k - 1L) %% length(cols) + 1L], border = "grey20",
                              ylim = c(0, ytop), ylab = "Mean",
                              main = paste0(dlab, ": ", pnl$title))
      if (!is.null(sdv)) {
        ok <- !is.na(sdv) & sdv > 0
        if (any(ok))
          graphics::arrows(bp[ok], mval[ok] - sdv[ok], bp[ok], mval[ok] + sdv[ok],
                           angle = 90, code = 3, length = 0.04, col = "grey30")
      }
      graphics::box()
    }
    return(invisible(NULL))
  }

  # --- Stability objects: bar chart of the integrated stability ranking ------
  if (inherits(x, "aridagri_stability")) {
    tab <- x$integrated
    if (!is.null(tab) && all(c("Genotype", "Rank") %in% names(tab))) {
      tab <- tab[order(tab$Rank), , drop = FALSE]
      op <- graphics::par(mar = c(7, 5, 3, 2))
      on.exit(graphics::par(op), add = TRUE)
      graphics::barplot(tab$Rank, names.arg = as.character(tab$Genotype),
                        las = 2, col = "#41AB5D", border = "#006D2C",
                        ylab = "Integrated stability rank (1 = most stable)",
                        main = "Integrated stability ranking")
      graphics::box()
      return(invisible(NULL))
    }
    # Fall back to genotype means from the GE matrix
    gem <- x$ge_matrix
    if (!is.null(gem)) {
      num <- gem[, vapply(gem, is.numeric, logical(1)), drop = FALSE]
      gmean <- sort(rowMeans(num, na.rm = TRUE), decreasing = TRUE)
      op <- graphics::par(mar = c(7, 5, 3, 2))
      on.exit(graphics::par(op), add = TRUE)
      graphics::barplot(gmean, las = 2, col = "#41AB5D", border = "#006D2C",
                        ylab = "Mean", main = "Genotype means")
      graphics::box()
      return(invisible(NULL))
    }
    message("arid_plot(): no stability summary available to plot.")
    return(invisible(NULL))
  }

  message("arid_plot(): no plotting method available for an object of class ",
          paste(class(x), collapse = "/"), ".")
  invisible(NULL)
}


#' Export Results to Publication Format
#'
#' @description
#' Exports aridagri analysis results to Excel format for publication.
#'
#' @param x An object from aridagri analysis functions
#' @param file Output file path
#' @param format Output format: \code{"xlsx"} (default) or \code{"csv"}
#' @param digits Number of decimal places
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Invisibly returns the file path
#'
#' @examples
#' \donttest{
#' df <- data.frame(
#'   yield = c(1200, 1350, 1100, 1450, 1280),
#'   wue = c(4.2, 4.8, 3.9, 5.1, 4.5)
#' )
#' result <- correlation_analysis(df, plot = FALSE)
#' export_results(result, tempfile(fileext = ".xlsx"))
#' }
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
export_results <- function(x, file, format = "xlsx", digits = 3,
                            verbose = TRUE) {
  
  # Extract data based on object class
  if (inherits(x, "aridagri_correlation")) {
    export_data <- round(x$correlation, digits)
  } else if (inherits(x, "data.frame")) {
    export_data <- x
  } else if (is.list(x) && !is.null(x$anova_table)) {
    export_data <- x$anova_table
  } else {
    stop("Object type not supported for export")
  }
  
  format <- match.arg(tolower(format), c("xlsx", "csv"))

  if (format == "xlsx") {
    if (!requireNamespace("writexl", quietly = TRUE)) {
      stop("Package 'writexl' required for Excel export. Install with: install.packages('writexl')")
    }
    writexl::write_xlsx(as.data.frame(export_data), file)
  } else if (format == "csv") {
    utils::write.csv(as.data.frame(export_data), file, row.names = TRUE)
  }

  if (verbose) message("Results exported to: ", file)

  invisible(file)
}


#' Print Method for PCA Results
#'
#' @description
#' Prints a formatted summary of Principal Component Analysis (PCA) results.
#'
#' @param x An object of class 'aridagri_pca' from \code{\link{pca_analysis}}
#' @param ... Additional arguments (currently unused)
#'
#' @return No return value, called for side effects. Prints the number of
#'   components retained by Kaiser criterion and cumulative variance explained
#'   to the console. The input object is returned invisibly.
#'
#' @export
print.aridagri_pca <- function(x, ...) {
  cat("\n=== PCA Summary ===\n")
  cat("Components retained (Kaiser):", x$n_components_kaiser, "\n")
  cat("Variance explained:", round(x$cumulative_variance[x$n_components_kaiser], 1), "%\n")
  invisible(x)
}

#' @export
print.aridagri_path <- function(x, ...) {
  cat("\n=== Path Analysis ===\n")
  cat("R-squared:", round(x$r_squared, 4), "\n")
  cat("Residual:", round(x$residual, 4), "\n\n")
  cat("Direct Effects:\n")
  print(round(x$direct_effects, 4))
  invisible(x)
}

#' @export
print.aridagri_stability <- function(x, ...) {
  cat("\n=== Stability Analysis ===\n")
  cat("Grand Mean:", round(x$grand_mean, 2), "\n")
  if (!is.null(x$integrated)) {
    cat("\nIntegrated Ranking:\n")
    print(head(x$integrated, 10))
  }
  invisible(x)
}

#' ============================================================================
#' COMPREHENSIVE EXPERIMENTAL DESIGN ANALYSIS FUNCTIONS
#' Package: aridagri
#' Author: Lalit Kumar Rolaniya
#' ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
#' ============================================================================

#' Completely Randomized Design (CRD) ANOVA
#'
#' @description
#' Performs complete ANOVA for Completely Randomized Design with post-hoc tests,
#' assumptions checking, and publication-ready output.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable (as string)
#' @param treatment Name of treatment factor
#' @param posthoc Post-hoc test: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"
#' @param alpha Significance level (default 0.05)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table, means, post-hoc results, and diagnostics
#'
#' @examples
#' data <- data.frame(
#'   treatment = rep(c("T1", "T2", "T3", "T4"), each = 5),
#'   yield = c(rnorm(5, 1200, 50), rnorm(5, 1350, 60), 
#'             rnorm(5, 1100, 55), rnorm(5, 1450, 65))
#' )
#' anova_crd(data, response = "yield", treatment = "treatment", posthoc = "all")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_crd <- function(data, response, treatment, posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Input validation
  if (!response %in% names(data)) stop(paste("Response variable", response, "not found"))
  if (!treatment %in% names(data)) stop(paste("Treatment variable", treatment, "not found"))
  
  # Convert to factor
  data[[treatment]] <- as.factor(data[[treatment]])
  
  # Get dimensions
  t <- nlevels(data[[treatment]])
  N <- nrow(data)
  
  # Check for balanced design
  trt_counts <- table(data[[treatment]])
  if (length(unique(trt_counts)) > 1) {
    warning("Unbalanced design detected: unequal replications across treatments (",
            paste(names(trt_counts), "=", trt_counts, collapse = ", "),
            "). Results assume balanced data and may be unreliable. ",
            "Consider using lme4::lmer() for unbalanced data.", call. = FALSE)
  }
  
  # Fit model
  formula_crd <- as.formula(paste(response, "~", treatment))
  model <- aov(formula_crd, data = data)
  anova_table <- anova(model)
  
  # Extract values
  SS_treatment <- anova_table$`Sum Sq`[1]
  SS_error <- anova_table$`Sum Sq`[2]
  SS_total <- SS_treatment + SS_error
  
  df_treatment <- anova_table$Df[1]
  df_error <- anova_table$Df[2]
  df_total <- df_treatment + df_error
  
  MS_treatment <- anova_table$`Mean Sq`[1]
  MS_error <- anova_table$`Mean Sq`[2]
  
  F_value <- anova_table$`F value`[1]
  p_value <- anova_table$`Pr(>F)`[1]
  
  # Calculate statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  cv <- sqrt(MS_error) / grand_mean * 100
  se_mean <- sqrt(MS_error / (N / t))
  cd_5 <- qt(0.975, df_error) * sqrt(2 * MS_error / (N / t))
  cd_1 <- qt(0.995, df_error) * sqrt(2 * MS_error / (N / t))
  
  # Print ANOVA table
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("         COMPLETELY RANDOMIZED DESIGN (CRD) - ANOVA                   \n")
    cat("\n")
    cat(" Response Variable:", sprintf("%-50s", response), "\n")
    cat(" Treatment Factor:", sprintf("%-51s", treatment), "\n")
    cat(" Number of Treatments:", sprintf("%-47d", t), "\n")
    cat(" Total Observations:", sprintf("%-49d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")
    cat(" Source             df        SS           MS         F    Pr>F  \n")
    cat("\n")
    cat(sprintf(" Treatment        %6d  %11.2f  %11.2f  %5.2f  %5.4f\n",
                df_treatment, SS_treatment, MS_treatment, F_value, p_value))
    cat(sprintf(" Error            %6d  %11.2f  %11.2f               \n",
                df_error, SS_error, MS_error))
    cat("\n")
    cat(sprintf(" Total            %6d  %11.2f                            \n",
                df_total, SS_total))
    cat("\n")
  }
  
  # Significance
  sig <- ifelse(p_value < 0.001, "***", ifelse(p_value < 0.01, "**", 
         ifelse(p_value < 0.05, "*", "NS")))
  
  if (verbose) {
    cat("\n\n")
    cat("                        SUMMARY STATISTICS                            \n")
    cat("\n")
    cat(sprintf(" Grand Mean          : %10.2f                                     \n", grand_mean))
    cat(sprintf(" CV (%%)              : %10.2f                                     \n", cv))
    cat(sprintf(" SE(m)               : %10.3f                                     \n", se_mean))
    cat(sprintf(" CD (5%%)             : %10.3f                                     \n", cd_5))
    cat(sprintf(" CD (1%%)             : %10.3f                                     \n", cd_1))
    cat(sprintf(" Significance        : %10s                                     \n", sig))
    cat("\n")
  }
  
  # Treatment means
  means <- aggregate(data[[response]], by = list(data[[treatment]]), 
                     FUN = function(x) c(mean = mean(x, na.rm = TRUE), 
                                         sd = sd(x, na.rm = TRUE),
                                         n = length(x)))
  means_df <- data.frame(
    Treatment = means$Group.1,
    Mean = means$x[, "mean"],
    SD = means$x[, "sd"],
    N = means$x[, "n"]
  )
  means_df <- means_df[order(-means_df$Mean), ]
  
  if (verbose) {
    cat("\n\n")
    cat("                        TREATMENT MEANS                               \n")
    cat("\n")
    cat(" Treatment           Mean          SD            N         Rank   \n")
    cat("\n")
    for (i in 1:nrow(means_df)) {
      cat(sprintf(" %-15s  %11.2f  %11.2f  %11.0f  %8d \n",
                  as.character(means_df$Treatment[i]), means_df$Mean[i], 
                  means_df$SD[i], means_df$N[i], i))
    }
    cat("\n")
  }
  
  # Post-hoc tests
  posthoc_results <- perform_posthoc(model, data, response, treatment, 
                                      MS_error, df_error, posthoc, alpha,
                                      verbose = verbose)
  
  # Model diagnostics
  diagnostics <- check_assumptions(model, verbose = verbose)
  
  # Return results
  result <- list(
    design = "CRD",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    se_mean = se_mean,
    cd_5 = cd_5,
    cd_1 = cd_1,
    treatment_means = means_df,
    posthoc = posthoc_results,
    diagnostics = diagnostics,
    significance = sig
  )
  
  class(result) <- c("aridagri_anova", "list")
  return(invisible(result))
}


#' Randomized Block Design (RBD) ANOVA
#'
#' @description
#' Performs complete ANOVA for Randomized Block Design (RCBD) with post-hoc tests,
#' assumptions checking, and publication-ready output.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param treatment Name of treatment factor
#' @param block Name of block/replication factor
#' @param posthoc Post-hoc test: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"
#' @param alpha Significance level (default 0.05)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table, means, post-hoc results
#'
#' @examples
#' data <- data.frame(
#'   rep = rep(1:4, each = 5),
#'   treatment = rep(c("T1", "T2", "T3", "T4", "T5"), 4),
#'   yield = c(rnorm(5, 1200, 50), rnorm(5, 1250, 55),
#'             rnorm(5, 1180, 45), rnorm(5, 1270, 60))
#' )
#' anova_rbd(data, response = "yield", treatment = "treatment", 
#'           block = "rep", posthoc = "duncan")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_rbd <- function(data, response, treatment, block, posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Input validation
  if (!response %in% names(data)) stop(paste("Response variable", response, "not found"))
  if (!treatment %in% names(data)) stop(paste("Treatment variable", treatment, "not found"))
  if (!block %in% names(data)) stop(paste("Block variable", block, "not found"))
  
  # Convert to factors
  data[[treatment]] <- as.factor(data[[treatment]])
  data[[block]] <- as.factor(data[[block]])
  
  # Get dimensions
  t <- nlevels(data[[treatment]])
  r <- nlevels(data[[block]])
  N <- nrow(data)
  
  # Check for balanced design
  if (N != t * r) {
    warning("Unbalanced design detected: expected ", t * r, " observations (", 
            t, " treatments x ", r, " blocks) but found ", N, ". ",
            "Results assume balanced data and may be unreliable. ",
            "Consider using lme4::lmer() for unbalanced data.", call. = FALSE)
  }
  
  # Fit model
  formula_rbd <- as.formula(paste(response, "~", block, "+", treatment))
  model <- aov(formula_rbd, data = data)
  anova_table <- anova(model)
  
  # Extract values
  SS_block <- anova_table$`Sum Sq`[1]
  SS_treatment <- anova_table$`Sum Sq`[2]
  SS_error <- anova_table$`Sum Sq`[3]
  SS_total <- SS_block + SS_treatment + SS_error
  
  df_block <- anova_table$Df[1]
  df_treatment <- anova_table$Df[2]
  df_error <- anova_table$Df[3]
  df_total <- df_block + df_treatment + df_error
  
  MS_block <- anova_table$`Mean Sq`[1]
  MS_treatment <- anova_table$`Mean Sq`[2]
  MS_error <- anova_table$`Mean Sq`[3]
  
  F_block <- anova_table$`F value`[1]
  F_treatment <- anova_table$`F value`[2]
  p_block <- anova_table$`Pr(>F)`[1]
  p_treatment <- anova_table$`Pr(>F)`[2]
  
  # Calculate statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  cv <- sqrt(MS_error) / grand_mean * 100
  se_mean <- sqrt(MS_error / r)
  cd_5 <- qt(0.975, df_error) * sqrt(2 * MS_error / r)
  cd_1 <- qt(0.995, df_error) * sqrt(2 * MS_error / r)
  
  # Relative efficiency of RBD over CRD
  MS_block_adj <- max(MS_block, MS_error)
  re_crd <- ((df_block * MS_block_adj + (df_treatment + df_block) * MS_error) /
              ((df_block + df_treatment + df_error) * MS_error)) * 100
  
  # Print ANOVA table
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("       RANDOMIZED BLOCK DESIGN (RBD/RCBD) - ANOVA                     \n")
    cat("\n")
    cat(" Response Variable:", sprintf("%-50s", response), "\n")
    cat(" Treatment Factor:", sprintf("%-51s", treatment), "\n")
    cat(" Block Factor:", sprintf("%-55s", block), "\n")
    cat(" Number of Treatments:", sprintf("%-47d", t), "\n")
    cat(" Number of Blocks:", sprintf("%-51d", r), "\n")
    cat(" Total Observations:", sprintf("%-49d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")
    cat(" Source             df        SS           MS         F    Pr>F  \n")
    cat("\n")
    cat(sprintf(" Block            %6d  %11.2f  %11.2f  %5.2f  %5.4f\n",
                df_block, SS_block, MS_block, F_block, p_block))
    cat(sprintf(" Treatment        %6d  %11.2f  %11.2f  %5.2f  %5.4f\n",
                df_treatment, SS_treatment, MS_treatment, F_treatment, p_treatment))
    cat(sprintf(" Error            %6d  %11.2f  %11.2f               \n",
                df_error, SS_error, MS_error))
    cat("\n")
    cat(sprintf(" Total            %6d  %11.2f                            \n",
                df_total, SS_total))
    cat("\n")
  }
  
  # Significance
  sig <- ifelse(p_treatment < 0.001, "***", ifelse(p_treatment < 0.01, "**", 
         ifelse(p_treatment < 0.05, "*", "NS")))
  
  if (verbose) {
    cat("\n\n")
    cat("                        SUMMARY STATISTICS                            \n")
    cat("\n")
    cat(sprintf(" Grand Mean                    : %10.2f                         \n", grand_mean))
    cat(sprintf(" CV (%%)                        : %10.2f                         \n", cv))
    cat(sprintf(" SE(m)                         : %10.3f                         \n", se_mean))
    cat(sprintf(" SE(d)                         : %10.3f                         \n", se_mean * sqrt(2)))
    cat(sprintf(" CD (5%%)                       : %10.3f                         \n", cd_5))
    cat(sprintf(" CD (1%%)                       : %10.3f                         \n", cd_1))
    cat(sprintf(" Treatment Significance        : %10s                         \n", sig))
    cat(sprintf(" Relative Efficiency over CRD  : %10.1f%%                        \n", re_crd))
    cat("\n")
  }
  
  # Treatment means
  means <- aggregate(data[[response]], by = list(data[[treatment]]), 
                     FUN = function(x) c(mean = mean(x, na.rm = TRUE), 
                                         sd = sd(x, na.rm = TRUE),
                                         n = length(x)))
  means_df <- data.frame(
    Treatment = means$Group.1,
    Mean = means$x[, "mean"],
    SD = means$x[, "sd"],
    N = means$x[, "n"]
  )
  means_df <- means_df[order(-means_df$Mean), ]
  
  if (verbose) {
    cat("\n\n")
    cat("                        TREATMENT MEANS                               \n")
    cat("\n")
    cat(" Treatment           Mean          SD            N         Rank   \n")
    cat("\n")
    for (i in 1:nrow(means_df)) {
      cat(sprintf(" %-15s  %11.2f  %11.2f  %11.0f  %8d \n",
                  as.character(means_df$Treatment[i]), means_df$Mean[i], 
                  means_df$SD[i], means_df$N[i], i))
    }
    cat("\n")
  }
  
  # Post-hoc tests
  posthoc_results <- perform_posthoc(model, data, response, treatment, 
                                      MS_error, df_error, posthoc, alpha,
                                      verbose = verbose)
  
  # Return results
  result <- list(
    design = "RBD",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    se_mean = se_mean,
    se_diff = se_mean * sqrt(2),
    cd_5 = cd_5,
    cd_1 = cd_1,
    relative_efficiency = re_crd,
    treatment_means = means_df,
    posthoc = posthoc_results,
    significance = sig
  )
  
  class(result) <- c("aridagri_anova", "list")
  return(invisible(result))
}


#' Pooled Analysis of RBD Experiments (Multi-Environment/Year)
#'
#' @description
#' Performs pooled ANOVA for RBD experiments conducted across multiple 
#' environments, years, or locations. Tests homogeneity of error variances
#' using Bartlett's test before pooling.
#'
#' @param data Data frame containing combined data from all environments
#' @param response Name of the response variable
#' @param treatment Name of treatment factor
#' @param environment Name of environment/year/location factor
#' @param block Name of block factor (nested within environment)
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing pooled ANOVA, individual ANOVAs, and interaction analysis
#'
#' @examples
#' # Data from 3 years
#' data <- data.frame(
#'   year = rep(c("Y1", "Y2", "Y3"), each = 20),
#'   rep = rep(rep(1:4, each = 5), 3),
#'   treatment = rep(c("T1", "T2", "T3", "T4", "T5"), 12),
#'   yield = rnorm(60, 1200, 150)
#' )
#' anova_rbd_pooled(data, response = "yield", treatment = "treatment",
#'                  environment = "year", block = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_rbd_pooled <- function(data, response, treatment, environment, block, 
                              posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[treatment]] <- as.factor(data[[treatment]])
  data[[environment]] <- as.factor(data[[environment]])
  data[[block]] <- as.factor(data[[block]])
  
  # Get dimensions
  t <- nlevels(data[[treatment]])
  e <- nlevels(data[[environment]])
  r <- nlevels(data[[block]])
  N <- nrow(data)
  
  # Check for balanced design
  if (N != t * e * r) {
    warning("Unbalanced design detected: expected ", t * e * r, " observations (", 
            t, " treatments x ", e, " environments x ", r, " blocks) but found ", N, ". ",
            "Results assume balanced data and may be unreliable. ",
            "Consider using lme4::lmer() for unbalanced data.", call. = FALSE)
  }
  
  # Individual ANOVAs for each environment
  env_levels <- levels(data[[environment]])
  individual_anovas <- list()
  mse_values <- numeric(e)
  df_error_values <- numeric(e)
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("              POOLED RBD ANALYSIS (Multi-Environment)                 \n")
    cat("\n")
    cat(" Response Variable:", sprintf("%-50s", response), "\n")
    cat(" Treatment Factor:", sprintf("%-51s", treatment), "\n")
    cat(" Environment Factor:", sprintf("%-49s", environment), "\n")
    cat(" Block Factor:", sprintf("%-55s", block), "\n")
    cat(" Number of Treatments:", sprintf("%-47d", t), "\n")
    cat(" Number of Environments:", sprintf("%-45d", e), "\n")
    cat(" Replications per Environment:", sprintf("%-39d", r), "\n")
    cat(" Total Observations:", sprintf("%-49d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("              INDIVIDUAL ENVIRONMENT ANALYSES                         \n")
    cat("\n")

    for (i in 1:e) {
      env_data <- data[data[[environment]] == env_levels[i], ]
      formula_rbd <- as.formula(paste(response, "~", block, "+", treatment))
      model_i <- aov(formula_rbd, data = env_data)
      anova_i <- anova(model_i)

      individual_anovas[[env_levels[i]]] <- anova_i
      mse_values[i] <- anova_i$`Mean Sq`[3]
      df_error_values[i] <- anova_i$Df[3]

      mean_i <- mean(env_data[[response]], na.rm = TRUE)
      cv_i <- sqrt(mse_values[i]) / mean_i * 100

      cat(sprintf("\n--- %s ---\n", env_levels[i]))
      cat(sprintf("Mean: %.2f | MSE: %.2f | CV: %.2f%%\n", mean_i, mse_values[i], cv_i))
    }
  }
  
  # Bartlett's test for homogeneity of error variances
  if (verbose) {
    cat("\n\n")
    cat("           HOMOGENEITY OF ERROR VARIANCES (Bartlett's Test)           \n")
    cat("\n")
  }
  
  # Calculate Bartlett's test manually
  mse_pooled <- sum(df_error_values * mse_values) / sum(df_error_values)
  
  numerator <- sum(df_error_values) * log(mse_pooled) - sum(df_error_values * log(mse_values))
  c_factor <- 1 + (1 / (3 * (e - 1))) * (sum(1 / df_error_values) - 1 / sum(df_error_values))
  chi_sq <- numerator / c_factor
  p_bartlett <- 1 - pchisq(chi_sq, e - 1)
  
  if (verbose) {
    cat(sprintf("\nBartlett's Chi-square: %.4f\n", chi_sq))
    cat(sprintf("Degrees of freedom: %d\n", e - 1))
    cat(sprintf("P-value: %.4f\n", p_bartlett))

    if (p_bartlett < 0.05) {
      cat("WARNING: Error variances are heterogeneous (p < 0.05)\n")
      cat("Consider data transformation or weighted analysis.\n")
    } else {
      cat("Error variances are homogeneous. Pooling is appropriate.\n")
    }
  }
  
  # Pooled ANOVA
  if (verbose) {
    cat("\n\n")
    cat("                        POOLED ANALYSIS OF VARIANCE                   \n")
    cat("\n")
  }
  
  # Create nested block factor
  data$block_nested <- interaction(data[[environment]], data[[block]])
  
  # Fit pooled model
  formula_pooled <- as.formula(paste(response, "~", environment, "+", 
                                      environment, ":", block, "+",
                                      treatment, "+", environment, ":", treatment))
  model_pooled <- aov(formula_pooled, data = data)
  anova_pooled <- anova(model_pooled)
  
  if (verbose) {
    print(anova_pooled)
  }
  
  # Calculate pooled statistics
  pooled_error_df <- anova_pooled$Df[length(anova_pooled$Df)]
  pooled_error_ms <- anova_pooled$`Mean Sq`[length(anova_pooled$`Mean Sq`)]
  
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  cv_pooled <- sqrt(pooled_error_ms) / grand_mean * 100
  
  # SE and CD for different comparisons
  se_treatment <- sqrt(pooled_error_ms / (e * r))
  cd_treatment_5 <- qt(0.975, pooled_error_df) * sqrt(2 * pooled_error_ms / (e * r))
  
  se_env <- sqrt(pooled_error_ms / (t * r))
  cd_env_5 <- qt(0.975, pooled_error_df) * sqrt(2 * pooled_error_ms / (t * r))
  
  se_interaction <- sqrt(pooled_error_ms / r)
  cd_interaction_5 <- qt(0.975, pooled_error_df) * sqrt(2 * pooled_error_ms / r)
  
  if (verbose) {
    cat("\n\n")
    cat("                        POOLED SUMMARY STATISTICS                     \n")
    cat("\n")
    cat(sprintf(" Grand Mean                    : %10.2f                         \n", grand_mean))
    cat(sprintf(" Pooled CV (%%)                 : %10.2f                         \n", cv_pooled))
    cat("\n")
    cat(" For comparing TREATMENT means:                                       \n")
    cat(sprintf("   SE(m)  : %10.3f    CD(5%%) : %10.3f                        \n", se_treatment, cd_treatment_5))
    cat("\n")
    cat(" For comparing ENVIRONMENT means:                                     \n")
    cat(sprintf("   SE(m)  : %10.3f    CD(5%%) : %10.3f                        \n", se_env, cd_env_5))
    cat("\n")
    cat(" For comparing INTERACTION means:                                     \n")
    cat(sprintf("   SE(m)  : %10.3f    CD(5%%) : %10.3f                        \n", se_interaction, cd_interaction_5))
    cat("\n")
  }
  
  # Treatment means across environments
  treatment_means <- aggregate(data[[response]], by = list(data[[treatment]]), 
                                FUN = mean, na.rm = TRUE)
  names(treatment_means) <- c("Treatment", "Mean")
  treatment_means <- treatment_means[order(-treatment_means$Mean), ]
  
  if (verbose) {
    cat("\n\n")
    cat("                   TREATMENT MEANS (Across Environments)              \n")
    cat("\n")
    for (i in 1:nrow(treatment_means)) {
      cat(sprintf(" %-31s  %34.2f \n", 
                  as.character(treatment_means$Treatment[i]), treatment_means$Mean[i]))
    }
    cat("\n")
  }
  
  # Interaction means table
  interaction_means <- aggregate(data[[response]], 
                                  by = list(data[[treatment]], data[[environment]]), 
                                  FUN = mean, na.rm = TRUE)
  names(interaction_means) <- c("Treatment", "Environment", "Mean")
  interaction_table <- reshape(interaction_means, idvar = "Treatment", 
                                timevar = "Environment", direction = "wide")
  
  if (verbose) {
    cat("\n\n")
    cat("                   TREATMENT  ENVIRONMENT MEANS                      \n")
    cat("\n")
    print(interaction_table, row.names = FALSE)
  }
  
  # Return results
  result <- list(
    design = "Pooled RBD",
    pooled_anova = anova_pooled,
    individual_anovas = individual_anovas,
    model = model_pooled,
    grand_mean = grand_mean,
    cv = cv_pooled,
    bartlett_test = list(chi_sq = chi_sq, df = e - 1, p_value = p_bartlett),
    se_treatment = se_treatment,
    cd_treatment = cd_treatment_5,
    se_environment = se_env,
    cd_environment = cd_env_5,
    se_interaction = se_interaction,
    cd_interaction = cd_interaction_5,
    treatment_means = treatment_means,
    interaction_means = interaction_table,
    mse_values = mse_values
  )
  
  class(result) <- c("aridagri_pooled", "list")
  return(invisible(result))
}

test_that("anova_crd produces correct output", {
  set.seed(123)
  data <- data.frame(treatment = rep(c("T1","T2","T3","T4"), each=5),
                     yield = c(rnorm(5,1200,50), rnorm(5,1350,60), rnorm(5,1100,55), rnorm(5,1450,65)))
  result <- anova_crd(data, response="yield", treatment="treatment", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
  expect_true("grand_mean" %in% names(result))
  expect_true("cv" %in% names(result))
  expect_true("se_mean" %in% names(result))
})
test_that("anova_crd returns correct df", {
  set.seed(1)
  data <- data.frame(treatment = rep(c("T1","T2","T3"), each=4), yield = rnorm(12,100,10))
  result <- anova_crd(data, response="yield", treatment="treatment", verbose=FALSE)
  expect_equal(result$anova_table[["Df"]][1], 2)
  expect_equal(result$anova_table[["Df"]][2], 9)
})
test_that("anova_crd stops with invalid columns", {
  data <- data.frame(x=1:10, y=rnorm(10))
  expect_error(anova_crd(data, response="yield", treatment="trt"))
})
test_that("anova_rbd produces correct output", {
  set.seed(456)
  data <- data.frame(variety = rep(paste0("V",1:5), times=4), block = rep(1:4, each=5), yield = rnorm(20,2000,200))
  result <- anova_rbd(data, response="yield", treatment="variety", block="block", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
  expect_true("cv" %in% names(result))
})
test_that("anova_rbd returns correct df", {
  set.seed(1)
  data <- data.frame(trt = rep(c("A","B","C","D"), times=3), block = rep(1:3, each=4), yield = rnorm(12,50,5))
  result <- anova_rbd(data, response="yield", treatment="trt", block="block", verbose=FALSE)
  expect_equal(result$anova_table[["Df"]][1], 2)
  expect_equal(result$anova_table[["Df"]][2], 3)
  expect_equal(result$anova_table[["Df"]][3], 6)
})
test_that("anova_rbd stops with invalid columns", {
  data <- data.frame(x=1:10, y=rnorm(10))
  expect_error(anova_rbd(data, response="yield", treatment="trt", block="blk"))
})
test_that("anova_rbd_pooled produces correct output", {
  set.seed(789)
  data <- expand.grid(trt = paste0("T",1:4), env = paste0("E",1:3), block = 1:2)
  data$yield <- rnorm(nrow(data), 1500, 200)
  result <- anova_rbd_pooled(data, response="yield", treatment="trt", environment="env", block="block", verbose=FALSE)
  expect_type(result, "list")
  expect_true("pooled_anova" %in% names(result))
})
test_that("anova_latin produces correct output", {
  set.seed(404)
  data <- data.frame(
    trt = c("A","B","C","D","E","B","C","D","E","A","C","D","E","A","B","D","E","A","B","C","E","A","B","C","D"),
    row = rep(1:5, each=5), col = rep(1:5, times=5), yield = rnorm(25,500,50))
  result <- anova_latin(data, response="yield", treatment="trt", row="row", column="col", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})
test_that("anova_factorial produces correct output", {
  set.seed(303)
  data <- expand.grid(fert = c("F1","F2","F3"), variety = c("V1","V2"), rep = 1:4)
  data$yield <- rnorm(nrow(data), 1000, 100)
  result <- anova_factorial(data, response="yield", factor1="fert", factor2="variety", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})
test_that("anova_factorial_3way produces correct output", {
  set.seed(505)
  data <- expand.grid(f1 = c("A","B"), f2 = c("X","Y"), f3 = c("P","Q"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 100, 15)
  result <- anova_factorial_3way(data, response="yield", factor1="f1", factor2="f2", factor3="f3", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

test_that("anova_crd with posthoc lsd works", {
  set.seed(1)
  data <- data.frame(trt = rep(c("A","B","C","D"), each=5), yield = c(rnorm(5,20,2), rnorm(5,25,2), rnorm(5,22,2), rnorm(5,30,2)))
  result <- anova_crd(data, response="yield", treatment="trt", posthoc="lsd", verbose=FALSE)
  expect_type(result, "list")
  expect_true("posthoc" %in% names(result))
})

test_that("anova_rbd with posthoc duncan works", {
  set.seed(2)
  data <- data.frame(trt = rep(c("A","B","C"), times=4), block = rep(1:4, each=3), yield = rnorm(12,50,5))
  result <- anova_rbd(data, response="yield", treatment="trt", block="block", posthoc="duncan", verbose=FALSE)
  expect_type(result, "list")
  expect_true("posthoc" %in% names(result))
})

test_that("anova_rbd with posthoc tukey works", {
  set.seed(3)
  data <- data.frame(trt = rep(c("A","B","C"), times=4), block = rep(1:4, each=3), yield = rnorm(12,50,5))
  result <- anova_rbd(data, response="yield", treatment="trt", block="block", posthoc="tukey", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_crd with posthoc snk works", {
  set.seed(4)
  data <- data.frame(trt = rep(c("A","B","C"), each=5), yield = rnorm(15,100,10))
  result <- anova_crd(data, response="yield", treatment="trt", posthoc="snk", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_crd with posthoc scheffe works", {
  set.seed(5)
  data <- data.frame(trt = rep(c("A","B","C"), each=5), yield = rnorm(15,100,10))
  result <- anova_crd(data, response="yield", treatment="trt", posthoc="scheffe", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_crd with posthoc bonferroni works", {
  set.seed(6)
  data <- data.frame(trt = rep(c("A","B","C"), each=5), yield = rnorm(15,100,10))
  result <- anova_crd(data, response="yield", treatment="trt", posthoc="bonferroni", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_crd with posthoc dunnett works", {
  set.seed(7)
  data <- data.frame(treatment = rep(c("A","B","C"), each=5), yield = rnorm(15,100,10))
  result <- anova_crd(data, response="yield", treatment="treatment", posthoc="dunnett", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_crd warns on unbalanced data", {
  data <- data.frame(trt = c(rep("A",5), rep("B",3), rep("C",4)), yield = rnorm(12,50,5))
  expect_warning(anova_crd(data, response="yield", treatment="trt", verbose=FALSE), "Unbalanced design detected")
})

test_that("anova_crd no warning on balanced data", {
  data <- data.frame(trt = rep(c("A","B","C"), each=4), yield = rnorm(12,50,5))
  expect_no_warning(anova_crd(data, response="yield", treatment="trt", verbose=FALSE))
})

test_that("anova_rbd warns on unbalanced data", {
  set.seed(1)
  data <- data.frame(trt = rep(c("A","B","C"), times=4), block = rep(1:4, each=3), yield = rnorm(12,50,5))
  data_unbal <- data[-5, ]
  expect_warning(anova_rbd(data_unbal, response="yield", treatment="trt", block="block", verbose=FALSE), "Unbalanced design detected")
})

test_that("anova_rbd no warning on balanced data", {
  set.seed(1)
  data <- data.frame(trt = rep(c("A","B","C"), times=4), block = rep(1:4, each=3), yield = rnorm(12,50,5))
  expect_no_warning(anova_rbd(data, response="yield", treatment="trt", block="block", verbose=FALSE))
})

test_that("anova_spd warns on unbalanced data", {
  set.seed(1)
  data <- expand.grid(main = c("M1","M2"), sub = c("S1","S2","S3"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 100, 10)
  data_unbal <- data[-3, ]
  expect_warning(anova_spd(data_unbal, response="yield", main_plot="main", sub_plot="sub", replication="rep", verbose=FALSE), "Unbalanced design detected")
})

test_that("anova_spd no warning on balanced data", {
  set.seed(1)
  data <- expand.grid(main = c("M1","M2"), sub = c("S1","S2","S3"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 100, 10)
  expect_no_warning(anova_spd(data, response="yield", main_plot="main", sub_plot="sub", replication="rep", verbose=FALSE))
})

#' Split-Split Plot Design ANOVA
#'
#' @description
#' Performs complete ANOVA for Split-Split Plot Design with proper error terms
#' for main plot, sub-plot, and sub-sub-plot factors. Generates publication-ready
#' ANOVA table with significance levels.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable (as string)
#' @param main_plot Name of main plot factor
#' @param sub_plot Name of sub-plot factor
#' @param sub_sub_plot Name of sub-sub-plot factor
#' @param replication Name of replication/block factor
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table, means, and significance tests
#'
#' @examples
#' # Example with sample data
#' data <- expand.grid(rep=1:3, A=c('A1','A2'), B=c('B1','B2'), C=c('C1','C2'))
#' data$yield <- rnorm(24, 1200, 150)
#' anova_sspd(data, response='yield', main_plot='A', sub_plot='B',
#'            sub_sub_plot='C', replication='rep')
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_sspd <- function(data, response, main_plot, sub_plot, sub_sub_plot, replication,
                            verbose = TRUE) {
  groups <- list(main = main_plot, sub = sub_plot, subsub = sub_sub_plot)
  core <- .aridagri_spd_anova(data, response, replication, groups, 0.05)
  if (verbose)
    .aridagri_print_spd(core, "Split-Split-Plot Design")
  result <- list(
    design       = "Split-Split-Plot Design",
    anova_table  = core$anova_table,
    grand_mean   = core$grand_mean,
    cv           = core$cv,
    ms_error_a   = core$error_ms[["main"]],   df_error_a = core$error_df[["main"]],
    ms_error_b   = core$error_ms[["sub"]],    df_error_b = core$error_df[["sub"]],
    ms_error_c   = core$error_ms[["subsub"]], df_error_c = core$error_df[["subsub"]],
    factor_means = core$factor_means,
    comparisons  = core$comparisons,
    model        = core$model
  )
  class(result) <- "aridagri_anova"
  return(invisible(result))
}


# NOTE: anova_spd is defined in design_split_plot.R with full features


#' Correlation Analysis with Significance
#'
#' @description
#' Computes correlation matrix with significance levels and generates
#' publication-ready correlation table and plot.
#'
#' @param data Data frame with numeric variables
#' @param method Correlation method: "pearson", "spearman", or "kendall"
#' @param plot Logical, whether to generate correlation plot
#' @param digits Number of decimal places
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List with correlation matrix and significance matrix
#'
#' @examples
#' data <- data.frame(
#'   yield = c(1200, 1350, 1100, 1450, 1280),
#'   wue = c(4.2, 4.8, 3.9, 5.1, 4.5),
#'   protein = c(22.1, 23.5, 21.8, 24.2, 22.9)
#' )
#' correlation_analysis(data)
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
correlation_analysis <- function(data, method = "pearson", plot = TRUE, digits = 3,
                            verbose = TRUE) {
  
  # Select only numeric columns
  numeric_data <- data[sapply(data, is.numeric)]
  
  if (ncol(numeric_data) < 2) {
    stop("Need at least 2 numeric variables for correlation analysis")
  }
  
  n_vars <- ncol(numeric_data)
  n_obs <- nrow(numeric_data)
  var_names <- names(numeric_data)
  
  # Initialize matrices
  cor_matrix <- matrix(NA, n_vars, n_vars, dimnames = list(var_names, var_names))
  p_matrix <- matrix(NA, n_vars, n_vars, dimnames = list(var_names, var_names))
  
  # Calculate correlations and p-values
  for (i in 1:n_vars) {
    for (j in 1:n_vars) {
      test <- cor.test(numeric_data[[i]], numeric_data[[j]], method = method)
      cor_matrix[i, j] <- test$estimate
      p_matrix[i, j] <- test$p.value
    }
  }
  
  # Create significance symbols
  sig_matrix <- matrix("", n_vars, n_vars, dimnames = list(var_names, var_names))
  sig_matrix[p_matrix < 0.001] <- "***"
  sig_matrix[p_matrix >= 0.001 & p_matrix < 0.01] <- "**"
  sig_matrix[p_matrix >= 0.01 & p_matrix < 0.05] <- "*"
  sig_matrix[p_matrix >= 0.05 & p_matrix < 0.1] <- "."
  
  # Print results
  if (verbose) {
    cat("\n==========================================================\n")
    cat("          CORRELATION ANALYSIS\n")
    cat("==========================================================\n")
    cat("Method:", method, "\n")
    cat("Number of observations:", n_obs, "\n")
    cat("Number of variables:", n_vars, "\n")
    cat("\n--- Correlation Matrix ---\n\n")

    print(round(cor_matrix, digits))

    cat("\n--- Significance Levels ---\n")
    cat("*** p < 0.001, ** p < 0.01, * p < 0.05, . p < 0.1\n")
  }
  
  # Plot if requested
  if (plot && requireNamespace("corrplot", quietly = TRUE)) {
    corrplot::corrplot(cor_matrix, method = "color", type = "upper",
                       addCoef.col = "black", tl.col = "black",
                       tl.srt = 45, diag = FALSE,
                       p.mat = p_matrix, sig.level = 0.05, insig = "blank",
                       title = "Correlation Matrix", mar = c(0, 0, 2, 0))
  }
  
  result <- list(
    correlation = round(cor_matrix, digits),
    p_values = round(p_matrix, 4),
    significance = sig_matrix,
    method = method,
    n = n_obs
  )
  
  class(result) <- "aridagri_correlation"
  return(invisible(result))
}


#' Principal Component Analysis
#'
#' @description
#' Performs PCA with visualization suitable for agricultural research data.
#' Includes scree plot, biplot, and variable contributions.
#'
#' @param data Data frame with numeric variables
#' @param scale Logical, whether to scale variables (default TRUE)
#' @param ncp Number of components to retain (default 5)
#' @param plot Logical, whether to generate plots
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return PCA results with eigenvalues, loadings, and scores
#'
#' @examples
#' data <- data.frame(
#'   yield = rnorm(30, 1200, 200),
#'   wue = rnorm(30, 4.5, 0.5),
#'   protein = rnorm(30, 22, 2),
#'   biomass = rnorm(30, 3500, 500)
#' )
#' pca_analysis(data)
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
pca_analysis <- function(data, scale = TRUE, ncp = 5, plot = TRUE,
                            verbose = TRUE) {
  
  # Select numeric columns
  numeric_data <- data[sapply(data, is.numeric)]
  
  if (ncol(numeric_data) < 2) {
    stop("Need at least 2 numeric variables for PCA")
  }
  
  # Remove rows with NA
  numeric_data <- na.omit(numeric_data)
  
  # Perform PCA
  pca_result <- prcomp(numeric_data, scale. = scale, center = TRUE)
  
  # Extract results
  eigenvalues <- pca_result$sdev^2
  var_explained <- eigenvalues / sum(eigenvalues) * 100
  cum_var <- cumsum(var_explained)
  
  # Loadings
  loadings <- pca_result$rotation
  
  # Print results
  if (verbose) {
    cat("\n==========================================================\n")
    cat("          PRINCIPAL COMPONENT ANALYSIS\n")
    cat("==========================================================\n")
    cat("Variables:", ncol(numeric_data), "\n")
    cat("Observations:", nrow(numeric_data), "\n")
    cat("Scaling:", ifelse(scale, "Yes", "No"), "\n")

    cat("\n--- Eigenvalues and Variance Explained ---\n\n")
  }
  
  eigen_table <- data.frame(
    PC = paste0("PC", 1:length(eigenvalues)),
    Eigenvalue = round(eigenvalues, 3),
    Variance_Percent = round(var_explained, 2),
    Cumulative_Percent = round(cum_var, 2)
  )
  if (verbose) {
    print(eigen_table)

    cat("\n--- Variable Loadings (First 3 PCs) ---\n\n")
    print(round(loadings[, 1:min(3, ncol(loadings))], 3))
  }
  
  # Kaiser criterion - components with eigenvalue > 1
  n_retain <- sum(eigenvalues > 1)
  if (verbose) {
    cat("\n--- Recommendation ---\n")
    cat("Components with eigenvalue > 1:", n_retain, "\n")
    cat("Variance explained by", n_retain, "components:", round(cum_var[n_retain], 1), "%\n")
  }
  
  # Plots
  if (plot && requireNamespace("factoextra", quietly = TRUE)) {
    # Scree plot
    if (verbose) {
      print(factoextra::fviz_screeplot(pca_result, addlabels = TRUE, 
                                        title = "Scree Plot"))
    }
    
    # Biplot
    if (verbose) {
      print(factoextra::fviz_pca_biplot(pca_result, 
                                         repel = TRUE,
                                         title = "PCA Biplot"))
    }
  }
  
  result <- list(
    pca = pca_result,
    eigenvalues = eigenvalues,
    variance_explained = var_explained,
    cumulative_variance = cum_var,
    loadings = loadings,
    scores = pca_result$x,
    n_components_kaiser = n_retain
  )
  
  class(result) <- "aridagri_pca"
  return(invisible(result))
}


#' Path Coefficient Analysis
#'
#' @description
#' Performs path analysis to determine direct and indirect effects of
#' independent variables on a dependent variable. Essential for 
#' understanding yield contributing factors.
#'
#' @param data Data frame with numeric variables
#' @param dependent Name of dependent variable (e.g., "yield")
#' @param independent Character vector of independent variable names
#' @param digits Number of decimal places
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List with direct effects, indirect effects, and correlation breakdown
#'
#' @examples
#' data <- data.frame(
#'   yield = c(1200, 1350, 1100, 1450, 1280, 1380, 1220, 1400),
#'   pods = c(45, 52, 42, 58, 48, 54, 46, 56),
#'   seeds = c(8.2, 9.1, 7.8, 9.5, 8.5, 9.0, 8.3, 9.3),
#'   weight = c(32, 35, 30, 38, 33, 36, 31, 37)
#' )
#' path_analysis(data, dependent = "yield", 
#'               independent = c("pods", "seeds", "weight"))
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
path_analysis <- function(data, dependent, independent, digits = 4,
                            verbose = TRUE) {
  
  # Validate inputs
  if (!dependent %in% names(data)) {
    stop(paste("Dependent variable", dependent, "not found in data"))
  }
  
  missing_vars <- setdiff(independent, names(data))
  if (length(missing_vars) > 0) {
    stop(paste("Variables not found:", paste(missing_vars, collapse = ", ")))
  }
  
  # Subset data
  vars <- c(dependent, independent)
  analysis_data <- data[, vars]
  analysis_data <- na.omit(analysis_data)
  
  n <- nrow(analysis_data)
  p <- length(independent)
  
  # Calculate correlation matrix
  cor_matrix <- cor(analysis_data)
  
  # Correlation of independent variables with dependent
  r_y <- cor_matrix[dependent, independent]
  
  # Correlation among independent variables
  R_xx <- cor_matrix[independent, independent]
  
  # Path coefficients (direct effects) = inverse of R_xx multiplied by r_y
  R_xx_inv <- solve(R_xx)
  path_coef <- R_xx_inv %*% r_y
  direct_effects <- as.vector(path_coef)
  names(direct_effects) <- independent
  
  # Calculate indirect effects
  indirect_matrix <- matrix(0, p, p, dimnames = list(independent, independent))
  
  for (i in 1:p) {
    for (j in 1:p) {
      if (i != j) {
        indirect_matrix[i, j] <- direct_effects[j] * R_xx[i, j]
      }
    }
  }
  
  # Total indirect effects
  indirect_total <- rowSums(indirect_matrix)
  
  # Residual effect
  r_squared <- sum(direct_effects * r_y)
  residual <- sqrt(1 - r_squared)
  
  # Print results
  if (verbose) {
    cat("\n==========================================================\n")
    cat("          PATH COEFFICIENT ANALYSIS\n")
    cat("==========================================================\n")
    cat("Dependent variable:", dependent, "\n")
    cat("Independent variables:", paste(independent, collapse = ", "), "\n")
    cat("Number of observations:", n, "\n")

    cat("\n--- Direct Effects (Path Coefficients) ---\n\n")
  }
  direct_df <- data.frame(
    Variable = independent,
    Direct_Effect = round(direct_effects, digits)
  )
  if (verbose) {
    print(direct_df, row.names = FALSE)

    cat("\n--- Indirect Effects ---\n\n")
    print(round(indirect_matrix, digits))

    cat("\n--- Correlation Partitioning ---\n\n")
  }
  partition_df <- data.frame(
    Variable = independent,
    Correlation_with_Y = round(r_y, digits),
    Direct_Effect = round(direct_effects, digits),
    Indirect_Effect = round(indirect_total, digits)
  )
  if (verbose) {
    print(partition_df, row.names = FALSE)

    cat("\n--- Model Summary ---\n")
    cat("R-squared:", round(r_squared, 4), "\n")
    cat("Residual effect:", round(residual, 4), "\n")
    cat("==========================================================\n")
  }
  
  result <- list(
    direct_effects = direct_effects,
    indirect_effects = indirect_matrix,
    indirect_total = indirect_total,
    correlation_with_y = r_y,
    correlation_matrix = cor_matrix,
    r_squared = r_squared,
    residual = residual
  )
  
  class(result) <- "aridagri_path"
  return(invisible(result))
}


#' Structural Equation Modeling for Field Experiments
#'
#' @description
#' Performs SEM analysis for agricultural field experiments. Allows testing
#' of hypothesized causal relationships among variables.
#'
#' @param data Data frame with variables
#' @param model Model specification in lavaan syntax
#' @param plot Logical, whether to generate path diagram
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return SEM results including fit indices and parameter estimates
#'
#' @examples
#' \donttest{
#' if (requireNamespace("lavaan", quietly = TRUE)) {
#'   set.seed(42)
#'   n <- 100
#'   nitrogen <- rnorm(n, 60, 10)
#'   phosphorus <- rnorm(n, 30, 5)
#'   yield <- 0.5 * nitrogen + 0.3 * phosphorus + rnorm(n, 0, 5)
#'   df <- data.frame(yield = yield, nitrogen = nitrogen, phosphorus = phosphorus)
#'   model <- 'yield ~ nitrogen + phosphorus'
#'   result <- sem_analysis(df, model, plot = FALSE)
#' }
#' }
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
sem_analysis <- function(data, model, plot = TRUE,
                            verbose = TRUE) {
  
  if (!requireNamespace("lavaan", quietly = TRUE)) {
    stop("Package 'lavaan' is required for SEM analysis")
  }
  
  # Fit SEM model
  fit <- lavaan::sem(model, data = data)
  
  # Extract results
  fit_measures <- lavaan::fitMeasures(fit, c("chisq", "df", "pvalue", 
                                              "cfi", "tli", "rmsea", 
                                              "srmr", "aic", "bic"))
  
  if (verbose) {
    cat("\n==========================================================\n")
    cat("          STRUCTURAL EQUATION MODEL ANALYSIS\n")
    cat("==========================================================\n")

    cat("\n--- Model Fit Indices ---\n")
    cat("Chi-square:", round(fit_measures["chisq"], 3), 
        "(df =", fit_measures["df"], ", p =", round(fit_measures["pvalue"], 4), ")\n")
    cat("CFI:", round(fit_measures["cfi"], 3), "\n")
    cat("TLI:", round(fit_measures["tli"], 3), "\n")
    cat("RMSEA:", round(fit_measures["rmsea"], 3), "\n")
    cat("SRMR:", round(fit_measures["srmr"], 3), "\n")

    cat("\n--- Fit Assessment ---\n")
    if (fit_measures["cfi"] >= 0.95 && fit_measures["rmsea"] <= 0.06) {
      cat("Model fit: GOOD\n")
    } else if (fit_measures["cfi"] >= 0.90 && fit_measures["rmsea"] <= 0.08) {
      cat("Model fit: ACCEPTABLE\n")
    } else {
      cat("Model fit: POOR - Consider model modification\n")
    }

    cat("\n--- Parameter Estimates ---\n\n")
    print(lavaan::parameterEstimates(fit, standardized = TRUE))
  }
  
  # Plot path diagram
  if (plot && requireNamespace("semPlot", quietly = TRUE)) {
    semPlot::semPaths(fit, what = "std", layout = "tree",
                      edge.label.cex = 0.8, curvePivot = TRUE,
                      title = TRUE, title.cex = 1.2)
  }
  
  result <- list(
    fit = fit,
    fit_measures = fit_measures,
    parameters = lavaan::parameterEstimates(fit, standardized = TRUE)
  )
  
  class(result) <- "aridagri_sem"
  return(invisible(result))
}

#' ============================================================================
#' ADVANCED AGRONOMIC STATISTICAL METHODS
#' Package: aridagri
#' Authors: Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
#' ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
#' ICAR-Indian Institute of Groundnut Research, Regional Research Station, Bikaner
#' ============================================================================

#' Stability Analysis for Agronomic Traits (Multiple Methods)
#'
#' @description
#' Performs comprehensive stability analysis using multiple established methods for 
#' evaluating genotype/treatment performance across environments.
#'
#' Methods included:
#' \itemize{
#'   \item Eberhart & Russell (1966): Regression approach
#'   \item AMMI: Additive Main effects and Multiplicative Interaction
#'   \item Finlay & Wilkinson (1963): Linear regression on environmental mean
#'   \item Shukla (1972): Stability variance
#'   \item Wricke (1962): Ecovalence
#'   \item Coefficient of Variation: CV-based ranking
#'   \item Superiority Index: Lin & Binns (1988)
#' }
#'
#' @param data Data frame with genotype/treatment, environment, replication, and trait
#' @param genotype Name of genotype/treatment column
#' @param environment Name of environment/location/year column
#' @param replication Name of replication column
#' @param trait Name of trait/response variable
#' @param method Method: "eberhart", "ammi", "finlay", "shukla", "wricke", "cv", "superiority", or "all"
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing stability parameters and rankings
#'
#' @examples
#' data <- expand.grid(
#'   variety = paste0("V", 1:10),
#'   location = paste0("L", 1:5),
#'   rep = 1:3
#' )
#' data$yield <- rnorm(nrow(data), 1200, 200)
#' 
#' stability_analysis(data, genotype = "variety", environment = "location",
#'                    replication = "rep", trait = "yield", method = "all")
#'
#' @references
#' Eberhart, S.A. and Russell, W.A. (1966). Crop Science, 6: 36-40.
#' 
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
stability_analysis <- function(data, genotype, environment, replication, 
                                trait, method = "all",
                            verbose = TRUE) {
  
  # Convert to factors
  data[[genotype]] <- as.factor(data[[genotype]])
  data[[environment]] <- as.factor(data[[environment]])
  data[[replication]] <- as.factor(data[[replication]])
  
  g <- nlevels(data[[genotype]])
  e <- nlevels(data[[environment]])
  r <- nlevels(data[[replication]])
  
  # Calculate means
  ge_means <- aggregate(data[[trait]], 
                         by = list(data[[genotype]], data[[environment]]), 
                         FUN = mean, na.rm = TRUE)
  names(ge_means) <- c("Genotype", "Environment", "Mean")
  
  g_means <- aggregate(ge_means$Mean, by = list(ge_means$Genotype), FUN = mean)
  names(g_means) <- c("Genotype", "Mean")
  
  e_means <- aggregate(ge_means$Mean, by = list(ge_means$Environment), FUN = mean)
  names(e_means) <- c("Environment", "Mean")
  
  grand_mean <- mean(ge_means$Mean)
  e_means$Index <- e_means$Mean - grand_mean
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                    STABILITY ANALYSIS FOR AGRONOMIC TRAITS                         \n")
    cat("\n")
    cat(" Number of Genotypes/Treatments:", sprintf("%-51d", g), "\n")
    cat(" Number of Environments        :", sprintf("%-51d", e), "\n")
    cat(" Replications/Environment      :", sprintf("%-51d", r), "\n")
    cat(" Grand Mean                    :", sprintf("%-51.2f", grand_mean), "\n")
    cat("\n")
  }
  
  results <- list()
  results$grand_mean <- grand_mean
  results$genotype_means <- g_means
  results$environment_means <- e_means
  
  # Create GE matrix
  ge_matrix <- reshape(ge_means, idvar = "Genotype", timevar = "Environment", 
                        direction = "wide")
  rownames(ge_matrix) <- ge_matrix$Genotype
  ge_matrix <- ge_matrix[, -1]
  names(ge_matrix) <- gsub("Mean.", "", names(ge_matrix))
  ge_matrix <- as.matrix(ge_matrix)
  
  # EBERHART AND RUSSELL
  if (method %in% c("eberhart", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("              EBERHART & RUSSELL (1966) STABILITY ANALYSIS                         \n")
      cat("\n")
    }
    
    stability_er <- data.frame(Genotype = rownames(ge_matrix))
    stability_er$Mean <- rowMeans(ge_matrix)
    
    env_index <- colMeans(ge_matrix) - grand_mean
    bi <- numeric(g)
    s2di <- numeric(g)
    r_squared <- numeric(g)
    
    for (i in 1:g) {
      y <- ge_matrix[i, ]
      reg <- lm(y ~ env_index)
      bi[i] <- coef(reg)[2]
      predicted <- predict(reg)
      s2di[i] <- sum((y - predicted)^2) / (e - 2)
      r_squared[i] <- summary(reg)$r.squared
    }
    
    stability_er$bi <- round(bi, 3)
    stability_er$S2di <- round(s2di, 3)
    stability_er$R2 <- round(r_squared, 3)
    stability_er <- stability_er[order(-stability_er$Mean), ]
    
    if (verbose) {
      print(stability_er, row.names = FALSE)
      cat("\n bi  1.0, low Sdi: Wide adaptation | bi > 1.0: Favorable env | bi < 1.0: Poor env\n")
    }
    
    results$eberhart <- stability_er
  }
  
  # FINLAY & WILKINSON
  if (method %in% c("finlay", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("                  FINLAY & WILKINSON (1963) STABILITY ANALYSIS                     \n")
      cat("\n")
    }
    
    stability_fw <- data.frame(Genotype = rownames(ge_matrix))
    stability_fw$Mean <- rowMeans(ge_matrix)
    
    env_means <- colMeans(ge_matrix)
    bi_fw <- numeric(g)
    for (i in 1:g) {
      reg <- lm(ge_matrix[i, ] ~ env_means)
      bi_fw[i] <- coef(reg)[2]
    }
    
    stability_fw$bi <- round(bi_fw, 3)
    stability_fw <- stability_fw[order(-stability_fw$Mean), ]
    if (verbose) {
      print(stability_fw, row.names = FALSE)
    }
    
    results$finlay <- stability_fw
  }
  
  # SHUKLA'S STABILITY VARIANCE
  if (method %in% c("shukla", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("                    SHUKLA'S (1972) STABILITY VARIANCE (i)                       \n")
      cat("\n")
    }
    
    stability_shukla <- data.frame(Genotype = rownames(ge_matrix))
    stability_shukla$Mean <- rowMeans(ge_matrix)
    
    sigma2_i <- numeric(g)
    for (i in 1:g) {
      interaction <- ge_matrix[i, ] - rowMeans(ge_matrix)[i] - colMeans(ge_matrix) + grand_mean
      sigma2_i[i] <- sum(interaction^2) / (e - 1)
    }
    
    stability_shukla$Sigma2_i <- round(sigma2_i, 2)
    stability_shukla$Rank <- rank(sigma2_i)
    stability_shukla <- stability_shukla[order(stability_shukla$Sigma2_i), ]
    if (verbose) {
      print(stability_shukla, row.names = FALSE)
      cat("\n Lower i indicates higher stability\n")
    }
    
    results$shukla <- stability_shukla
  }
  
  # WRICKE'S ECOVALENCE
  if (method %in% c("wricke", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("                       WRICKE'S (1962) ECOVALENCE (Wi)                             \n")
      cat("\n")
    }
    
    stability_wricke <- data.frame(Genotype = rownames(ge_matrix))
    stability_wricke$Mean <- rowMeans(ge_matrix)
    
    Wi <- numeric(g)
    for (i in 1:g) {
      interaction <- ge_matrix[i, ] - rowMeans(ge_matrix)[i] - colMeans(ge_matrix) + grand_mean
      Wi[i] <- sum(interaction^2)
    }
    
    stability_wricke$Wi <- round(Wi, 2)
    stability_wricke$Wi_percent <- round(Wi / sum(Wi) * 100, 2)
    stability_wricke$Rank <- rank(Wi)
    stability_wricke <- stability_wricke[order(stability_wricke$Wi), ]
    if (verbose) {
      print(stability_wricke, row.names = FALSE)
      cat("\n Lower Wi indicates higher stability\n")
    }
    
    results$wricke <- stability_wricke
  }
  
  # AMMI ANALYSIS
  if (method %in% c("ammi", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("            AMMI (Additive Main Effects and Multiplicative Interaction)            \n")
      cat("\n")
    }
    
    interaction_matrix <- ge_matrix
    for (i in 1:g) {
      for (j in 1:e) {
        interaction_matrix[i, j] <- ge_matrix[i, j] - 
          rowMeans(ge_matrix)[i] - colMeans(ge_matrix)[j] + grand_mean
      }
    }
    
    svd_result <- svd(interaction_matrix)
    eigenvalues <- svd_result$d^2
    var_explained <- eigenvalues / sum(eigenvalues) * 100
    cum_var <- cumsum(var_explained)
    n_pc <- min(g-1, e-1)
    
    if (verbose) {
      cat("\n--- AMMI Components ---\n\n")
    }
    ammi_components <- data.frame(
      IPCA = paste0("IPCA", 1:n_pc),
      Eigenvalue = round(eigenvalues[1:n_pc], 3),
      Variance_Percent = round(var_explained[1:n_pc], 2),
      Cumulative_Percent = round(cum_var[1:n_pc], 2)
    )
    if (verbose) {
      print(ammi_components, row.names = FALSE)
    }
    
    # AMMI Stability Value (ASV)
    pc1_scores <- svd_result$u[, 1] * svd_result$d[1]
    pc2_scores <- svd_result$u[, 2] * svd_result$d[2]
    weight <- eigenvalues[1] / eigenvalues[2]
    asv <- sqrt(weight * pc1_scores^2 + pc2_scores^2)
    
    mean_rank <- rank(-rowMeans(ge_matrix))
    asv_rank <- rank(asv)
    ysi <- mean_rank + asv_rank
    
    ammi_stability <- data.frame(
      Genotype = rownames(ge_matrix),
      Mean = round(rowMeans(ge_matrix), 2),
      Mean_Rank = mean_rank,
      IPCA1 = round(pc1_scores, 3),
      IPCA2 = round(pc2_scores, 3),
      ASV = round(asv, 3),
      ASV_Rank = asv_rank,
      YSI = ysi
    )
    ammi_stability <- ammi_stability[order(ammi_stability$YSI), ]
    
    if (verbose) {
      cat("\n--- AMMI Stability Values ---\n\n")
      print(ammi_stability, row.names = FALSE)
      cat("\n ASV: Lower = More Stable | YSI: Lower = Better\n")
    }
    
    results$ammi <- list(components = ammi_components, stability = ammi_stability)
  }
  
  # CV STABILITY
  if (method %in% c("cv", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("                    COEFFICIENT OF VARIATION (CV) STABILITY                        \n")
      cat("\n")
    }
    
    stability_cv <- data.frame(Genotype = rownames(ge_matrix))
    stability_cv$Mean <- rowMeans(ge_matrix)
    stability_cv$SD <- apply(ge_matrix, 1, sd)
    stability_cv$CV <- round((stability_cv$SD / stability_cv$Mean) * 100, 2)
    stability_cv$CV_Rank <- rank(stability_cv$CV)
    stability_cv <- stability_cv[order(stability_cv$CV), ]
    if (verbose) {
      print(stability_cv, row.names = FALSE)
      cat("\n Lower CV indicates higher stability\n")
    }
    
    results$cv_stability <- stability_cv
  }
  
  # SUPERIORITY INDEX
  if (method %in% c("superiority", "all")) {
    if (verbose) {
      cat("\n\n")
      cat("                  CULTIVAR SUPERIORITY INDEX (Lin & Binns, 1988)                   \n")
      cat("\n")
    }
    
    stability_sup <- data.frame(Genotype = rownames(ge_matrix))
    stability_sup$Mean <- rowMeans(ge_matrix)
    
    max_env <- apply(ge_matrix, 2, max)
    Pi <- numeric(g)
    for (i in 1:g) {
      Pi[i] <- sum((ge_matrix[i, ] - max_env)^2) / (2 * e)
    }
    
    stability_sup$Pi <- round(Pi, 2)
    stability_sup$Pi_Rank <- rank(Pi)
    stability_sup <- stability_sup[order(stability_sup$Pi), ]
    if (verbose) {
      print(stability_sup, row.names = FALSE)
      cat("\n Lower Pi indicates superior performance\n")
    }
    
    results$superiority <- stability_sup
  }
  
  # INTEGRATED RANKING (all 7 methods)
  if (method == "all") {
    if (verbose) {
      cat("\n\n")
      cat("                        INTEGRATED STABILITY RANKING                                \n")
      cat("\n")
    }

    integrated <- data.frame(Genotype = rownames(ge_matrix))
    integrated$Mean <- round(rowMeans(ge_matrix), 2)

    # 1. Eberhart-Russell: rank by S2di (lower = more stable)
    env_index <- colMeans(ge_matrix) - grand_mean
    s2di_temp <- numeric(g)
    for (i in 1:g) {
      y <- ge_matrix[i, ]
      reg <- lm(y ~ env_index)
      s2di_temp[i] <- sum((y - predict(reg))^2) / max(e - 2, 1)
    }
    integrated$ER <- rank(s2di_temp)

    # 2. AMMI: rank by AMMI Stability Value (lower ASV = more stable)
    interaction_mat <- ge_matrix
    for (i in 1:g) {
      for (j in 1:e) {
        interaction_mat[i, j] <- ge_matrix[i, j] -
          rowMeans(ge_matrix)[i] - colMeans(ge_matrix)[j] + grand_mean
      }
    }
    svd_res <- svd(interaction_mat)
    if (length(svd_res$d) >= 2) {
      pc1 <- svd_res$u[, 1] * svd_res$d[1]
      pc2 <- svd_res$u[, 2] * svd_res$d[2]
      wt <- (svd_res$d[1]^2) / (svd_res$d[2]^2)
      asv_temp <- sqrt(wt * pc1^2 + pc2^2)
    } else {
      asv_temp <- abs(svd_res$u[, 1] * svd_res$d[1])
    }
    integrated$AMMI <- rank(asv_temp)

    # 3. Finlay-Wilkinson: rank by |bi - 1| (closer to 1 = more stable)
    env_means_fw <- colMeans(ge_matrix)
    bi_fw_temp <- numeric(g)
    for (i in 1:g) {
      reg <- lm(ge_matrix[i, ] ~ env_means_fw)
      bi_fw_temp[i] <- coef(reg)[2]
    }
    integrated$FW <- rank(abs(bi_fw_temp - 1))

    # 4. Shukla: rank by stability variance (lower = more stable)
    sigma2_temp <- numeric(g)
    for (i in 1:g) {
      interaction <- ge_matrix[i, ] - rowMeans(ge_matrix)[i] -
        colMeans(ge_matrix) + grand_mean
      sigma2_temp[i] <- sum(interaction^2) / (e - 1)
    }
    integrated$Shukla <- rank(sigma2_temp)

    # 5. Wricke: rank by ecovalence Wi (lower = more stable)
    Wi_temp <- numeric(g)
    for (i in 1:g) {
      interaction <- ge_matrix[i, ] - rowMeans(ge_matrix)[i] -
        colMeans(ge_matrix) + grand_mean
      Wi_temp[i] <- sum(interaction^2)
    }
    integrated$Wricke <- rank(Wi_temp)

    # 6. CV: rank by coefficient of variation (lower = more stable)
    cv_temp <- apply(ge_matrix, 1, sd) / rowMeans(ge_matrix) * 100
    integrated$CV <- rank(cv_temp)

    # 7. Lin-Binns: rank by superiority index Pi (lower = more superior)
    max_env <- apply(ge_matrix, 2, max)
    Pi_temp <- numeric(g)
    for (i in 1:g) {
      Pi_temp[i] <- sum((ge_matrix[i, ] - max_env)^2) / (2 * e)
    }
    integrated$LB <- rank(Pi_temp)

    # Compute integrated rank as mean of all 7 method ranks
    rank_cols <- c("ER", "AMMI", "FW", "Shukla", "Wricke", "CV", "LB")
    integrated$Rank <- rank(rowMeans(integrated[, rank_cols]))
    integrated <- integrated[order(integrated$Rank), ]

    if (verbose) {
      print(integrated, row.names = FALSE)
      cat("\n ER=Eberhart-Russell AMMI=AMMI FW=Finlay-Wilkinson\n")
      cat(" Shukla=Stability var CV=CV method Wricke=Ecovalence LB=Lin-Binns\n")
    }

    results$integrated <- integrated
  }

  results$ge_matrix <- ge_matrix

  class(results) <- c("aridagri_stability", "list")
  return(invisible(results))
}


#' Growing Degree Days (GDD) and Thermal Indices
#'
#' @description
#' Calculates Growing Degree Days, Helio-thermal Units, Photo-thermal Units,
#' and Heat Use Efficiency from temperature data.
#'
#' @param tmax Vector of daily maximum temperatures (C)
#' @param tmin Vector of daily minimum temperatures (C)
#' @param base_temp Base temperature (C)
#' @param sunshine_hours Vector of daily sunshine hours (optional)
#' @param day_length Vector of day length in hours (optional)
#' @param crop_yield Crop yield for HUE calculation (optional)
#' @param biomass Biomass for HUE calculation (optional)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List with thermal indices
#'
#' @examples
#' tmax <- runif(90, 30, 42)
#' tmin <- runif(90, 18, 28)
#' thermal_indices(tmax, tmin, base_temp = 10)
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
thermal_indices <- function(tmax, tmin, base_temp = 10, sunshine_hours = NULL,
                            day_length = NULL, crop_yield = NULL, biomass = NULL,
                            verbose = TRUE) {
  
  n_days <- length(tmax)
  tmean <- (tmax + tmin) / 2
  gdd_daily <- pmax(0, tmean - base_temp)
  gdd_cumulative <- cumsum(gdd_daily)
  gdd_total <- sum(gdd_daily)
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                    THERMAL INDICES ANALYSIS                                        \n")
    cat("\n")
    cat(sprintf(" Number of Days         : %10d                                               \n", n_days))
    cat(sprintf(" Base Temperature       : %10.1f C                                          \n", base_temp))
    cat(sprintf(" Mean Tmax              : %10.1f C                                          \n", mean(tmax)))
    cat(sprintf(" Mean Tmin              : %10.1f C                                          \n", mean(tmin)))
    cat("\n")
  }
  
  results <- data.frame(Index = character(), Value = numeric(), Unit = character())
  
  results <- rbind(results, data.frame(Index = "Growing Degree Days (GDD)", 
                                        Value = round(gdd_total, 1), Unit = "C-days"))
  results <- rbind(results, data.frame(Index = "Mean Daily GDD", 
                                        Value = round(mean(gdd_daily), 2), Unit = "C-days/day"))
  
  if (!is.null(sunshine_hours)) {
    htu_total <- sum(gdd_daily * sunshine_hours)
    results <- rbind(results, data.frame(Index = "Helio-thermal Units (HTU)", 
                                          Value = round(htu_total, 1), Unit = "C-days-hours"))
  }
  
  if (!is.null(day_length)) {
    ptu_total <- sum(gdd_daily * day_length)
    results <- rbind(results, data.frame(Index = "Photo-thermal Units (PTU)", 
                                          Value = round(ptu_total, 1), Unit = "C-days-hours"))
  }
  
  if (!is.null(crop_yield)) {
    hue <- crop_yield / gdd_total
    results <- rbind(results, data.frame(Index = "Heat Use Efficiency (Grain)", 
                                          Value = round(hue, 3), Unit = "kg/ha/C-day"))
  }
  
  if (!is.null(biomass)) {
    hue_b <- biomass / gdd_total
    results <- rbind(results, data.frame(Index = "Heat Use Efficiency (Biomass)", 
                                          Value = round(hue_b, 3), Unit = "kg/ha/C-day"))
  }
  
  if (verbose) {
    cat("\n")
    print(results, row.names = FALSE)
  }
  
  output <- list(summary = results, gdd_total = gdd_total, 
                  daily = data.frame(Day = 1:n_days, Tmax = tmax, Tmin = tmin, 
                                      Tmean = tmean, GDD = gdd_daily, Cum_GDD = gdd_cumulative))
  class(output) <- "aridagri_thermal"
  return(invisible(output))
}


#' Crop Growth Analysis (CGR, RGR, NAR, LAI)
#'
#' @description
#' Calculates crop growth parameters from sequential harvest data.
#'
#' @param dry_weight Vector of dry matter at different stages
#' @param leaf_area Vector of leaf area at different stages
#' @param days Vector of days after sowing
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with growth parameters
#'
#' @examples
#' dry_weight <- c(0.5, 2.1, 8.5, 25, 45, 62, 75)
#' leaf_area <- c(15, 85, 350, 800, 950, 850, 600)
#' days <- c(15, 30, 45, 60, 75, 90, 105)
#' crop_growth_analysis(dry_weight, leaf_area, days)
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
crop_growth_analysis <- function(dry_weight, leaf_area, days,
                            verbose = TRUE) {
  
  n <- length(dry_weight)
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                         CROP GROWTH ANALYSIS                                       \n")
    cat("\n")
  }
  
  cgr <- rgr <- nar <- numeric(n - 1)
  lai <- leaf_area / 10000
  
  for (i in 1:(n-1)) {
    dt <- days[i+1] - days[i]
    dw <- dry_weight[i+1] - dry_weight[i]
    cgr[i] <- dw / dt
    rgr[i] <- (log(dry_weight[i+1]) - log(dry_weight[i])) / dt
    mean_la <- (leaf_area[i] + leaf_area[i+1]) / 2
    if (leaf_area[i+1] != leaf_area[i]) {
      nar[i] <- dw * (log(leaf_area[i+1]) - log(leaf_area[i])) / (dt * (leaf_area[i+1] - leaf_area[i]))
    } else {
      nar[i] <- dw / (dt * mean_la)
    }
  }
  
  results <- data.frame(
    Period = paste0(days[-n], "-", days[-1], " DAS"),
    CGR = round(cgr, 4),
    RGR = round(rgr, 5),
    NAR = round(nar * 10000, 4)
  )
  
  if (verbose) {
    print(results, row.names = FALSE)
    cat("\nCGR = Crop Growth Rate | RGR = Relative Growth Rate | NAR = Net Assimilation Rate\n")
  }
  
  class(results) <- c("aridagri_growth", "data.frame")
  return(invisible(results))
}


#' Harvest Index Calculation
#'
#' @description
#' Calculates Harvest Index and related partitioning indices.
#'
#' @param grain_yield Grain/economic yield
#' @param straw_yield Straw/stover yield
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with harvest indices
#'
#' @examples
#' harvest_index(grain_yield = c(1200, 1350, 1100), straw_yield = c(2400, 2500, 2300))
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
harvest_index <- function(grain_yield, straw_yield,
                            verbose = TRUE) {
  
  total_biomass <- grain_yield + straw_yield
  hi <- grain_yield / total_biomass * 100
  gsr <- grain_yield / straw_yield
  
  results <- data.frame(
    Grain_Yield = grain_yield,
    Straw_Yield = straw_yield,
    Total_Biomass = total_biomass,
    HI_percent = round(hi, 2),
    Grain_Straw_Ratio = round(gsr, 3)
  )
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                         HARVEST INDEX ANALYSIS                                     \n")
    cat("\n")

    if (length(grain_yield) > 1) {
      cat(sprintf("\nMean HI: %.2f%% | Mean Grain:Straw Ratio: %.3f\n\n", mean(hi), mean(gsr)))
    }
    print(results, row.names = FALSE)
  }
  
  class(results) <- c("aridagri_hi", "data.frame")
  return(invisible(results))
}


#' Yield Gap Analysis
#'
#' @description
#' Calculates yield gaps comparing actual with potential yields.
#'
#' @param actual_yield Actual yield (kg/ha)
#' @param potential_yield Potential yield (kg/ha)
#' @param attainable_yield Attainable yield (kg/ha, optional)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with yield gap analysis
#'
#' @examples
#' yield_gap_analysis(actual_yield = c(800, 950, 720), potential_yield = 1500)
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
yield_gap_analysis <- function(actual_yield, potential_yield, attainable_yield = NULL,
                            verbose = TRUE) {
  
  gap <- potential_yield - actual_yield
  gap_pct <- (gap / potential_yield) * 100
  efficiency <- (actual_yield / potential_yield) * 100
  
  results <- data.frame(
    Actual_Yield = actual_yield,
    Potential_Yield = potential_yield,
    Yield_Gap = round(gap, 1),
    Gap_Percent = round(gap_pct, 1),
    Yield_Efficiency = round(efficiency, 1)
  )
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                           YIELD GAP ANALYSIS                                       \n")
    cat("\n")
    cat(sprintf(" Mean Yield Gap          : %10.1f kg/ha (%.1f%%)                                \n", mean(gap), mean(gap_pct)))
    cat(sprintf(" Mean Yield Efficiency   : %10.1f %%                                           \n", mean(efficiency)))
    cat("\n\n")

    print(results, row.names = FALSE)
  }
  
  class(results) <- c("aridagri_yield_gap", "data.frame")
  return(invisible(results))
}


#' Economic Efficiency Indices (B:C Ratio)
#'
#' @description
#' Calculates B:C ratio and economic efficiency metrics.
#'
#' @param gross_return Gross returns (Rs/ha)
#' @param total_cost Total cost (Rs/ha)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return Data frame with economic indices
#'
#' @examples
#' economic_indices(gross_return = c(75000, 82000), total_cost = c(35000, 38000))
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
economic_indices <- function(gross_return, total_cost,
                            verbose = TRUE) {
  
  net_return <- gross_return - total_cost
  bc_ratio <- gross_return / total_cost
  return_per_rupee <- net_return / total_cost
  
  results <- data.frame(
    Gross_Return = gross_return,
    Total_Cost = total_cost,
    Net_Return = round(net_return, 0),
    BC_Ratio = round(bc_ratio, 2),
    Return_per_Rupee = round(return_per_rupee, 2)
  )
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                         ECONOMIC EFFICIENCY ANALYSIS                               \n")
    cat("\n")
    cat(sprintf(" Mean Net Return         : Rs. %10.0f /ha                                      \n", mean(net_return)))
    cat(sprintf(" Mean B:C Ratio          : %14.2f                                              \n", mean(bc_ratio)))
    cat("\n\n")

    print(results, row.names = FALSE)
  }
  
  class(results) <- c("aridagri_economics", "data.frame")
  return(invisible(results))
}

test_that("thermal_indices returns correct structure", {
  result <- thermal_indices(tmax = c(32,34,35), tmin = c(18,20,22), base_temp = 10, verbose = FALSE)
  expect_s3_class(result, "aridagri_thermal")
  expect_true("gdd_total" %in% names(result))
})

test_that("GDD calculation is correct", {
  result <- thermal_indices(tmax = c(30), tmin = c(20), base_temp = 10, verbose = FALSE)
  expect_equal(result$gdd_total, 15)
})

test_that("harvest_index returns correct structure", {
  result <- harvest_index(grain_yield = 2500, straw_yield = 4000, verbose = FALSE)
  expect_s3_class(result, "data.frame")
  expect_true("HI_percent" %in% names(result))
})

test_that("yield_gap_analysis returns correct structure", {
  result <- yield_gap_analysis(actual_yield = 2000, potential_yield = 4000, verbose = FALSE)
  expect_s3_class(result, "data.frame")
})

test_that("economic_indices returns correct BC ratio", {
  result <- economic_indices(gross_return = 80000, total_cost = 40000, verbose = FALSE)
  expect_s3_class(result, "data.frame")
  expect_true("BC_Ratio" %in% names(result))
  expect_equal(result$BC_Ratio, 2.0)
})

test_that("stability_analysis method=all produces all results", {
  set.seed(42)
  data <- expand.grid(g = paste0("G",1:5), e = paste0("E",1:4), r = 1:3)
  data$y <- rnorm(nrow(data), 1500, 250)
  result <- stability_analysis(data, "g", "e", "r", "y", method="all", verbose=FALSE)
  expect_type(result, "list")
  expect_true("eberhart" %in% names(result))
  expect_true("finlay" %in% names(result))
  expect_true("shukla" %in% names(result))
  expect_true("wricke" %in% names(result))
  expect_true("ammi" %in% names(result))
  expect_true("cv_stability" %in% names(result))
  expect_true("superiority" %in% names(result))
  expect_true("integrated" %in% names(result))
})

test_that("integrated ranking has all 7 method columns", {
  set.seed(42)
  data <- expand.grid(g = paste0("G",1:5), e = paste0("E",1:4), r = 1:3)
  data$y <- rnorm(nrow(data), 1500, 250)
  result <- stability_analysis(data, "g", "e", "r", "y", method="all", verbose=FALSE)
  expected_cols <- c("Genotype","Mean","ER","AMMI","FW","Shukla","Wricke","CV","LB","Rank")
  expect_true(all(expected_cols %in% names(result$integrated)))
  expect_equal(ncol(result$integrated), 10)
})

test_that("Shukla and Wricke produce identical rankings", {
  set.seed(42)
  data <- expand.grid(g = paste0("G",1:6), e = paste0("E",1:5), r = 1:3)
  data$y <- rnorm(nrow(data), 1500, 250)
  result <- stability_analysis(data, "g", "e", "r", "y", method="all", verbose=FALSE)
  expect_equal(result$integrated$Shukla, result$integrated$Wricke)
})

test_that("single method skips integrated ranking", {
  set.seed(42)
  data <- expand.grid(g = paste0("G",1:5), e = paste0("E",1:4), r = 1:3)
  data$y <- rnorm(nrow(data), 1500, 250)
  result <- stability_analysis(data, "g", "e", "r", "y", method="eberhart", verbose=FALSE)
  expect_true("eberhart" %in% names(result))
  expect_false("integrated" %in% names(result))
})

test_that("stability_analysis works with many genotypes", {
  set.seed(99)
  data <- expand.grid(g = paste0("G",1:15), e = paste0("E",1:6), r = 1:3)
  data$y <- rnorm(nrow(data), 2000, 300)
  result <- stability_analysis(data, "g", "e", "r", "y", method="all", verbose=FALSE)
  expect_equal(nrow(result$integrated), 15)
  expect_equal(max(result$integrated$Rank), 15)
})

#' Package imports
#'
#' @importFrom stats aggregate anova aov as.formula bartlett.test coef cor cor.test df.residual fitted lm na.omit pchisq pf prcomp predict qt qtukey reshape residuals sd shapiro.test
#' @importFrom utils head
#' @name imports
NULL

#' ============================================================================
#' ADDITIONAL EXPERIMENTAL DESIGN FUNCTIONS
#' Package: aridagri
#' Authors: Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
#' ============================================================================

#' Strip Plot Design ANOVA
#'
#' @description
#' Performs ANOVA for Strip Plot (Strip-Split) Design where two factors
#' are applied in horizontal and vertical strips.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param horizontal_factor Factor applied in horizontal strips (A)
#' @param vertical_factor Factor applied in vertical strips (B)
#' @param replication Name of replication factor
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results
#'
#' @examples
#' data <- expand.grid(
#'   rep = 1:4,
#'   irrigation = c("I1", "I2", "I3"),
#'   tillage = c("CT", "MT", "ZT")
#' )
#' data$yield <- rnorm(nrow(data), 1200, 150)
#' anova_strip(data, response = "yield", horizontal_factor = "irrigation",
#'             vertical_factor = "tillage", replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
anova_strip <- function(data, response, horizontal_factor, vertical_factor,
                         replication, alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[horizontal_factor]] <- as.factor(data[[horizontal_factor]])
  data[[vertical_factor]] <- as.factor(data[[vertical_factor]])
  data[[replication]] <- as.factor(data[[replication]])
  
  # Get dimensions
  a <- nlevels(data[[horizontal_factor]])
  b <- nlevels(data[[vertical_factor]])
  r <- nlevels(data[[replication]])
  N <- nrow(data)
  
  # Check for balanced design
  if (N != a * b * r) {
    warning("Unbalanced design detected: expected ", a * b * r, " observations but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }
  
  # Grand mean
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  
  # Fit model for strip plot
  formula_strip <- as.formula(paste(response, "~", replication, "+",
                                     horizontal_factor, "+", replication, ":", horizontal_factor, "+",
                                     vertical_factor, "+", replication, ":", vertical_factor, "+",
                                     horizontal_factor, ":", vertical_factor))
  
  model <- aov(formula_strip, data = data)
  anova_table <- anova(model)
  
  # Print header
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                      STRIP PLOT DESIGN - ANOVA                               \n")
    cat("\n")
    cat(" Response Variable     :", sprintf("%-52s", response), "\n")
    cat(" Horizontal Factor (A) :", sprintf("%-52s", horizontal_factor), "\n")
    cat(" Vertical Factor (B)   :", sprintf("%-52s", vertical_factor), "\n")
    cat(" Replications          :", sprintf("%-52d", r), "\n")
    cat(" Levels of A           :", sprintf("%-52d", a), "\n")
    cat(" Levels of B           :", sprintf("%-52d", b), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                                  \n")
    cat("\n")

    print(anova_table)
  }
  
  # Statistics
  n_rows <- nrow(anova_table)
  MS_error <- anova_table$`Mean Sq`[n_rows]
  cv <- sqrt(MS_error) / grand_mean * 100
  
  if (verbose) {
    cat(sprintf("\nGrand Mean: %.2f\n", grand_mean))
    cat(sprintf("CV: %.2f%%\n", cv))
  }
  
  # Factor means
  mean_a <- aggregate(data[[response]], by = list(data[[horizontal_factor]]), FUN = mean)
  names(mean_a) <- c("Level", "Mean")
  
  mean_b <- aggregate(data[[response]], by = list(data[[vertical_factor]]), FUN = mean)
  names(mean_b) <- c("Level", "Mean")
  
  if (verbose) {
    cat("\n--- Horizontal Factor Means ---\n")
    print(mean_a, row.names = FALSE)
    cat("\n--- Vertical Factor Means ---\n")
    print(mean_b, row.names = FALSE)
  }
  
  # Return
  result <- list(
    design = "Strip Plot Design",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    horizontal_means = mean_a,
    vertical_means = mean_b
  )
  
  class(result) <- c("aridagri_strip", "list")
  return(invisible(result))
}


#' Three-Factor Factorial ANOVA
#'
#' @description
#' Performs three-factor factorial ANOVA with all interactions.
#'
#' @param data Data frame containing the data
#' @param response Name of the response variable
#' @param factor1 Name of first factor (A)
#' @param factor2 Name of second factor (B)
#' @param factor3 Name of third factor (C)
#' @param replication Name of replication factor (optional)
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results
#'
#' @examples
#' data <- expand.grid(rep = 1:3, A = c("A1", "A2"), B = c("B1", "B2"), C = c("C1", "C2"))
#' data$yield <- rnorm(nrow(data), 1200, 150)
#' anova_factorial_3way(data, "yield", "A", "B", "C", "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
anova_factorial_3way <- function(data, response, factor1, factor2, factor3,
                                  replication = NULL, alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[factor1]] <- as.factor(data[[factor1]])
  data[[factor2]] <- as.factor(data[[factor2]])
  data[[factor3]] <- as.factor(data[[factor3]])
  
  # Get dimensions
  a <- nlevels(data[[factor1]])
  b <- nlevels(data[[factor2]])
  c <- nlevels(data[[factor3]])
  N <- nrow(data)
  
  # Build formula
  if (!is.null(replication)) {
    data[[replication]] <- as.factor(data[[replication]])
    r <- nlevels(data[[replication]])
    formula_fact <- as.formula(paste(response, "~", replication, "+", 
                                      factor1, "*", factor2, "*", factor3))
  } else {
    r <- N / (a * b * c)
    formula_fact <- as.formula(paste(response, "~", factor1, "*", factor2, "*", factor3))
  }
  
  # Check for balanced design
  expected_n <- a * b * c * r
  if (abs(N - expected_n) > 0.5) {
    warning("Unbalanced design detected: expected ", expected_n, " observations but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }
  
  # Fit model
  model <- aov(formula_fact, data = data)
  anova_table <- anova(model)
  
  # Print header
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("              THREE-FACTOR FACTORIAL ANOVA                            \n")
    cat("\n")
    cat(" Response Variable    :", sprintf("%-46s", response), "\n")
    cat(" Factor A             :", sprintf("%-46s", factor1), "\n")
    cat(" Factor B             :", sprintf("%-46s", factor2), "\n")
    cat(" Factor C             :", sprintf("%-46s", factor3), "\n")
    cat(sprintf(" Levels: A=%d, B=%d, C=%d", a, b, c))
    cat(sprintf("%36s\n", ""))
    cat(" Total Observations   :", sprintf("%-46d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")

    print(anova_table)
  }
  
  # Grand mean and CV
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  ms_error <- anova_table$`Mean Sq`[nrow(anova_table)]
  cv <- sqrt(ms_error) / grand_mean * 100
  
  if (verbose) {
    cat(sprintf("\nGrand Mean: %.2f\n", grand_mean))
    cat(sprintf("CV: %.2f%%\n", cv))
  }
  
  # Return
  result <- list(
    design = "Three-Factor Factorial",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv
  )
  
  class(result) <- c("aridagri_factorial3", "list")
  return(invisible(result))
}


#' Augmented Block Design ANOVA
#'
#' @description
#' Performs ANOVA for Augmented Randomized Block Design where checks are
#' replicated and test entries appear once.
#'
#' @param data Data frame containing the data
#' @param response Name of response variable
#' @param genotype Name of genotype/entry column
#' @param block Name of block column
#' @param check_names Vector of check variety names
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results and adjusted means
#'
#' @examples
#' data <- data.frame(
#'   block = rep(1:5, each = 7),
#'   genotype = c(rep(c("C1","C2","C3"), 5), paste0("T", 1:20)),
#'   yield = rnorm(35, 1200, 150)
#' )
#' # Note: This is simplified example
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
anova_augmented <- function(data, response, genotype, block, check_names,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[genotype]] <- as.factor(data[[genotype]])
  data[[block]] <- as.factor(data[[block]])
  
  # Identify checks and test entries
  data$Entry_Type <- ifelse(data[[genotype]] %in% check_names, "Check", "Test")
  
  # Get dimensions
  n_checks <- length(check_names)
  n_blocks <- nlevels(data[[block]])
  n_test <- nlevels(data[[genotype]]) - n_checks
  N <- nrow(data)
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("              AUGMENTED RANDOMIZED BLOCK DESIGN - ANOVA               \n")
    cat("\n")
    cat(" Response Variable   :", sprintf("%-47s", response), "\n")
    cat(" Number of Blocks    :", sprintf("%-47d", n_blocks), "\n")
    cat(" Number of Checks    :", sprintf("%-47d", n_checks), "\n")
    cat(" Number of Test Entries:", sprintf("%-44d", n_test), "\n")
    cat(" Total Observations  :", sprintf("%-47d", N), "\n")
    cat("\n")
  }
  
  # Separate check data
  check_data <- data[data$Entry_Type == "Check", ]
  
  # Analyze checks (RBD)
  formula_checks <- as.formula(paste(response, "~", block, "+", genotype))
  model_checks <- aov(formula_checks, data = check_data)
  anova_checks <- anova(model_checks)
  
  if (verbose) {
    cat("\n\n")
    cat("                  ANOVA FOR CHECK VARIETIES (RBD)                     \n")
    cat("\n")
    print(anova_checks)
  }
  
  # Error mean square
  mse_checks <- anova_checks$`Mean Sq`[nrow(anova_checks)]
  df_error <- anova_checks$Df[nrow(anova_checks)]
  
  # Block effects
  block_means <- tapply(check_data[[response]], check_data[[block]], mean)
  grand_mean_checks <- mean(check_data[[response]])
  block_effects <- block_means - grand_mean_checks
  
  # Calculate CD
  cd_check <- qt(0.975, df_error) * sqrt(2 * mse_checks / n_blocks)
  
  if (verbose) {
    cat("\n\n")
    cat("                        SUMMARY                                       \n")
    cat("\n")
    cat(sprintf(" Error MSE                          : %10.2f                      \n", mse_checks))
    cat(sprintf(" CD for comparing two checks        : %10.3f                      \n", cd_check))
    cat("\n")
  }
  
  # Return
  result <- list(
    design = "Augmented Block Design",
    anova_checks = anova_checks,
    mse = mse_checks,
    df_error = df_error,
    block_effects = block_effects,
    cd_check = cd_check
  )
  
  class(result) <- c("aridagri_augmented", "list")
  return(invisible(result))
}


#' Alpha Lattice Design ANOVA
#'
#' @description
#' Performs ANOVA for Alpha Lattice (Resolvable Incomplete Block) Design.
#'
#' @param data Data frame containing the data
#' @param response Name of response variable
#' @param genotype Name of genotype column
#' @param replication Name of replication column
#' @param block Name of incomplete block column (nested within replication)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results
#'
#' @examples
#' data <- expand.grid(rep = 1:2, block = 1:5, entry = 1:4)
#' data$genotype <- paste0("G", 1:nrow(data))
#' data$yield <- rnorm(nrow(data), 1200, 150)
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
anova_alpha_lattice <- function(data, response, genotype, replication, block,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[genotype]] <- as.factor(data[[genotype]])
  data[[replication]] <- as.factor(data[[replication]])
  data[[block]] <- as.factor(data[[block]])
  
  # Get dimensions
  g <- nlevels(data[[genotype]])
  r <- nlevels(data[[replication]])
  N <- nrow(data)
  
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                  ALPHA LATTICE DESIGN - ANOVA                        \n")
    cat("\n")
    cat(" Response Variable   :", sprintf("%-47s", response), "\n")
    cat(" Number of Genotypes :", sprintf("%-47d", g), "\n")
    cat(" Number of Replications:", sprintf("%-44d", r), "\n")
    cat(" Total Observations  :", sprintf("%-47d", N), "\n")
    cat("\n")
  }
  
  # Fit model: Block nested within replication
  formula_alpha <- as.formula(paste(response, "~", replication, "+",
                                     replication, ":", block, "+", genotype))
  
  model <- aov(formula_alpha, data = data)
  anova_table <- anova(model)
  
  if (verbose) {
    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")
    print(anova_table)
  }
  
  # Statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  mse <- anova_table$`Mean Sq`[nrow(anova_table)]
  cv <- sqrt(mse) / grand_mean * 100
  
  if (verbose) {
    cat(sprintf("\nGrand Mean: %.2f\n", grand_mean))
    cat(sprintf("CV: %.2f%%\n", cv))
  }
  
  # Genotype means
  g_means <- aggregate(data[[response]], by = list(data[[genotype]]), FUN = mean)
  names(g_means) <- c("Genotype", "Mean")
  g_means <- g_means[order(-g_means$Mean), ]
  
  if (verbose) {
    cat("\n--- Genotype Means (Top 10) ---\n")
    print(head(g_means, 10), row.names = FALSE)
  }
  
  # Return
  result <- list(
    design = "Alpha Lattice Design",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    genotype_means = g_means
  )
  
  class(result) <- c("aridagri_alpha", "list")
  return(invisible(result))
}


#' Pooled Split-Split Plot Design ANOVA
#'
#' @description
#' Performs pooled ANOVA for SSPD experiments across multiple environments.
#'
#' @param data Data frame containing combined data
#' @param response Name of the response variable
#' @param main_plot Name of main plot factor
#' @param sub_plot Name of sub-plot factor
#' @param sub_sub_plot Name of sub-sub-plot factor
#' @param environment Name of environment factor
#' @param replication Name of replication factor
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing pooled ANOVA results
#'
#' @examples
#' data <- expand.grid(env = c("E1","E2"), rep = 1:3, A = c("A1","A2"), 
#'                     B = c("B1","B2"), C = c("C1","C2"))
#' data$yield <- rnorm(nrow(data), 1200, 150)
#' anova_sspd_pooled(data, "yield", "A", "B", "C", "env", "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
anova_sspd_pooled <- function(data, response, main_plot, sub_plot, sub_sub_plot,
                               environment, replication,
                            verbose = TRUE) {

  data[[main_plot]]    <- as.factor(data[[main_plot]])
  data[[sub_plot]]     <- as.factor(data[[sub_plot]])
  data[[sub_sub_plot]] <- as.factor(data[[sub_sub_plot]])
  data[[environment]]  <- as.factor(data[[environment]])
  data[[replication]]  <- as.factor(data[[replication]])

  a <- nlevels(data[[main_plot]]); b <- nlevels(data[[sub_plot]])
  cc <- nlevels(data[[sub_sub_plot]])
  e <- nlevels(data[[environment]]); r <- nlevels(data[[replication]])
  N <- nrow(data)
  if (N != a * b * cc * e * r) {
    warning("Unbalanced design detected: expected ", a * b * cc * e * r,
            " observations but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }

  core <- .aridagri_spd_pooled_anova(data, response, environment, replication,
                                     split_factors = c(main_plot, sub_plot, sub_sub_plot))

  main_means <- aggregate(data[[response]], by = list(data[[main_plot]]), FUN = mean)
  names(main_means) <- c("Level", "Mean")
  sub_means <- aggregate(data[[response]], by = list(data[[sub_plot]]), FUN = mean)
  names(sub_means) <- c("Level", "Mean")
  subsub_means <- aggregate(data[[response]], by = list(data[[sub_sub_plot]]), FUN = mean)
  names(subsub_means) <- c("Level", "Mean")
  env_means <- aggregate(data[[response]], by = list(data[[environment]]), FUN = mean)
  names(env_means) <- c("Level", "Mean")

  if (verbose) {
    .aridagri_print_spd_pooled(core, "POOLED SPLIT-SPLIT PLOT DESIGN ANALYSIS")
    cat("\n--- Factor Means ---\n")
    cat("\nMain Plot Means:\n");    print(main_means,   row.names = FALSE)
    cat("\nSub-Plot Means:\n");     print(sub_means,    row.names = FALSE)
    cat("\nSub-Sub-Plot Means:\n"); print(subsub_means, row.names = FALSE)
    cat("\nEnvironment Means:\n");  print(env_means,    row.names = FALSE)
  }

  result <- list(
    design       = "Pooled Split-Split Plot Design",
    anova_table  = core$anova_table,
    model        = core$model,
    grand_mean   = core$grand_mean,
    cv           = core$cv,
    main_means   = main_means,
    sub_means    = sub_means,
    subsub_means = subsub_means,
    env_means    = env_means
  )
  class(result) <- c("aridagri_sspd_pooled", "list")
  return(invisible(result))
}

test_that("anova_augmented produces correct output", {
  set.seed(666)
  checks <- expand.grid(genotype = c("C1","C2","C3"), block = 1:4)
  tests <- data.frame(genotype = paste0("G",1:12), block = rep(1:4, each=3))
  data <- rbind(checks, tests)
  data$yield <- rnorm(nrow(data), 1000, 120)
  result <- anova_augmented(data, response="yield", genotype="genotype", block="block", check_names=c("C1","C2","C3"), verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_alpha_lattice produces correct output", {
  set.seed(777)
  data <- expand.grid(genotype = paste0("G",1:9), rep = 1:3)
  data$block <- rep(rep(1:3, each=3), 3)
  data$yield <- rnorm(nrow(data), 800, 80)
  result <- anova_alpha_lattice(data, response="yield", genotype="genotype", replication="rep", block="block", verbose=FALSE)
  expect_type(result, "list")
})

# Internal helpers for split-plot / split-split-plot ANOVA.
# Not exported. These compute the correct error strata by pooling the
# replication-by-treatment interactions to the proper nesting level, so that
# every treatment source is tested against the error term at the level of its
# most-nested factor (main-plot factors against the main-plot error, sub-plot
# factors against the sub-plot error, and so on).

.aridagri_spd_anova <- function(data, response, replication, groups, alpha = 0.05) {
  facs <- unlist(groups, use.names = FALSE)
  nlev <- length(groups)
  lab_names <- names(groups)
  fac_level <- integer(0)
  for (g in seq_along(groups)) for (f in groups[[g]]) fac_level[f] <- g

  data[[replication]] <- as.factor(data[[replication]])
  for (f in facs) data[[f]] <- as.factor(data[[f]])
  y <- data[[response]]
  N <- length(y)

  form <- stats::as.formula(paste0("`", response, "` ~ `", replication, "` * ",
                                   paste0("`", facs, "`", collapse = " * ")))
  full <- suppressWarnings(stats::aov(form, data = data))
  ## The full rep*factor crossing is saturated (one observation per cell), so
  ## anova() would warn that the full-model residual is an essentially perfect
  ## fit. That residual is not used here: each effect is tested against a pooled
  ## error stratum built below. Suppress the spurious warning.
  af <- suppressWarnings(stats::anova(full))
  rn <- trimws(gsub("`", "", rownames(af)))
  SS <- af[["Sum Sq"]]; DF <- af[["Df"]]
  terms <- strsplit(rn, ":")
  elevel <- function(tf) {
    nz <- setdiff(tf, c(replication, "Residuals"))
    if (!length(nz)) NA_integer_ else max(fac_level[nz])
  }
  has_rep  <- vapply(terms, function(tf) replication %in% tf, logical(1))
  is_resid <- rn == "Residuals"

  # pool replication-interactions and the residual into per-level error terms
  errSS <- numeric(nlev); errDF <- numeric(nlev)
  for (i in seq_along(rn)) {
    if (is_resid[i]) {
      errSS[nlev] <- errSS[nlev] + SS[i]; errDF[nlev] <- errDF[nlev] + DF[i]
    } else if (has_rep[i]) {
      L <- elevel(terms[[i]]); errSS[L] <- errSS[L] + SS[i]; errDF[L] <- errDF[L] + DF[i]
    }
  }
  errMS <- errSS / errDF
  errMS[!is.finite(errMS)] <- NA

  trt_idx <- which(!has_rep & !is_resid & rn != replication)
  trt_lvl <- vapply(trt_idx, function(i) elevel(terms[[i]]), integer(1))
  grand <- mean(y)

  rows <- list()
  add <- function(s, df, ss, ms, Fv, p)
    rows[[length(rows) + 1L]] <<- data.frame(
      .s = s, Df = df, `Sum Sq` = ss, `Mean Sq` = ms,
      `F value` = Fv, `Pr(>F)` = p, check.names = FALSE, stringsAsFactors = FALSE)
  ri <- which(rn == replication)
  add(replication, DF[ri], SS[ri], SS[ri] / DF[ri], NA, NA)
  for (L in seq_len(nlev)) {
    for (i in trt_idx[trt_lvl == L]) {
      ms <- SS[i] / DF[i]; Fv <- ms / errMS[L]
      add(rn[i], DF[i], SS[i], ms, Fv, stats::pf(Fv, DF[i], errDF[L], lower.tail = FALSE))
    }
    add(paste0("Error(", letters[L], ")"), errDF[L], errSS[L], errMS[L], NA, NA)
  }
  tab <- do.call(rbind, rows)
  rownames(tab) <- make.unique(tab$.s); tab$.s <- NULL

  factor_means <- lapply(facs, function(f) {
    m <- stats::aggregate(y, by = list(data[[f]]), FUN = mean)
    names(m) <- c("Level", "Mean"); m
  })
  names(factor_means) <- facs
  cv <- stats::setNames(sqrt(errMS) / grand * 100, lab_names)
  comp <- do.call(rbind, lapply(facs, function(f) {
    L <- fac_level[f]; k <- nlevels(data[[f]]); n_f <- N / k
    se <- sqrt(2 * errMS[L] / n_f); cd <- stats::qt(1 - alpha / 2, errDF[L]) * se
    data.frame(Factor = f, Levels = k, SE_diff = se, CD = cd,
               Error = paste0("Error(", letters[L], ")"), stringsAsFactors = FALSE)
  }))

  list(anova_table = tab, grand_mean = grand,
       error_ms = stats::setNames(errMS, lab_names),
       error_df = stats::setNames(errDF, lab_names),
       cv = cv, factor_means = factor_means, comparisons = comp, model = full)
}

.aridagri_print_spd <- function(core, design) {
  cat("\n", design, "\n", sep = "")
  cat(strrep("=", nchar(design)), "\n\n", sep = "")
  cat("Analysis of Variance\n")
  print(round(core$anova_table, 4))
  cat("\nGrand mean:", round(core$grand_mean, 3), "\n\n")
  cat("Error terms\n")
  nms <- names(core$error_ms)
  for (k in seq_along(nms))
    cat(sprintf("  Error(%s) [%s]: df = %d, MS = %.4f, CV = %.2f%%\n",
                letters[k], nms[k], core$error_df[[k]], core$error_ms[[k]], core$cv[[k]]))
  cat("\nStandard error of difference and critical difference (alpha = 0.05)\n")
  cmp <- core$comparisons[, c("Factor", "SE_diff", "CD")]
  cmp$SE_diff <- round(cmp$SE_diff, 3)
  cmp$CD <- round(cmp$CD, 3)
  print(cmp, row.names = FALSE)
  invisible(NULL)
}


## ---------------------------------------------------------------------------
## Internal helper for the combined ("pooled") split-plot family over
## environments. Environments are treated as fixed and replications as random
## within environment. Each treatment source is tested against the error term
## at the level of its deepest split factor; the environment main effect is
## tested against the environment/replication error (E:Rep). Balanced data are
## assumed (the calling functions warn otherwise). Sums of squares are taken
## from a single flat aov() model (which, for balanced data, are identical to
## the corresponding aov() Error()-strata sums of squares) and each effect is
## then assigned its correct denominator. Not exported.
.aridagri_spd_pooled_anova <- function(data, response, environment, replication,
                                       split_factors, alpha = 0.05) {
  E <- environment; R <- replication; FS <- split_factors; K <- length(FS)
  data[[E]] <- as.factor(data[[E]]); data[[R]] <- as.factor(data[[R]])
  for (f in FS) data[[f]] <- as.factor(data[[f]])

  trt  <- paste(c(E, FS), collapse = " * ")
  errs <- paste(E, R, sep = ":")
  if (K >= 2) for (k in 1:(K - 1))
    errs <- c(errs, paste(c(E, R, FS[1:k]), collapse = ":"))
  form  <- stats::as.formula(paste(response, "~", trt, "+", paste(errs, collapse = " + ")))
  model <- stats::aov(form, data = data)
  at    <- suppressWarnings(stats::anova(model))

  terms <- rownames(at)
  MS <- stats::setNames(at$`Mean Sq`, terms)
  DF <- stats::setNames(at$Df,        terms)
  SS <- stats::setNames(at$`Sum Sq`,  terms)

  keyof    <- function(v) paste(sort(v), collapse = "|")
  term_key <- vapply(terms, function(tm) keyof(strsplit(tm, ":", fixed = TRUE)[[1]]), character(1))
  find_term <- function(fset) { hit <- terms[term_key == keyof(fset)]; if (length(hit) == 1) hit else NA_character_ }

  level_error_name <- function(maxlev) {
    if (maxlev == 0) return(find_term(c(E, R)))
    if (maxlev == K) return("Residuals")
    find_term(c(E, R, FS[1:maxlev]))
  }
  split_level <- function(tm) {
    parts <- strsplit(tm, ":", fixed = TRUE)[[1]]
    lv <- 0L; for (i in seq_along(FS)) if (FS[i] %in% parts) lv <- i
    lv
  }
  err_rows <- c(vapply(0:(K - 1), function(k)
                  if (k == 0) find_term(c(E, R)) else find_term(c(E, R, FS[1:k])),
                  character(1)), "Residuals")

  Fv <- stats::setNames(rep(NA_real_, length(terms)), terms); Pv <- Fv
  for (tm in terms) {
    if (tm %in% err_rows) next
    den <- level_error_name(split_level(tm))
    Fv[tm] <- MS[[tm]] / MS[[den]]
    Pv[tm] <- stats::pf(Fv[tm], DF[[tm]], DF[[den]], lower.tail = FALSE)
  }

  out <- data.frame(Df = DF, `Sum Sq` = SS, `Mean Sq` = MS,
                    `F value` = Fv, `Pr(>F)` = Pv,
                    check.names = FALSE, row.names = terms)

  gm  <- mean(data[[response]], na.rm = TRUE)
  err_labels <- c("a (main-plot)", "b (sub-plot)", "c (sub-sub-plot)", "d", "e")
  cvs <- c(); cvs["Environment (E:Rep)"] <- sqrt(MS[[find_term(c(E, R))]]) / gm * 100
  for (k in 1:K) {
    den <- if (k == K) "Residuals" else find_term(c(E, R, FS[1:k]))
    cvs[paste0("Error ", err_labels[k])] <- sqrt(MS[[den]]) / gm * 100
  }

  list(anova_table = out, model = model, grand_mean = gm, cv = cvs,
       error_terms = err_rows, split_factors = FS, environment = E)
}

## Internal printer for the pooled split-plot family. Not exported.
.aridagri_print_spd_pooled <- function(core, design) {
  cat("\n", design, "\n", sep = "")
  cat(rep("=", 70), "\n", sep = "")
  cat("Environments fixed; replications random within environment.\n")
  cat("Each source tested against the error at the level of its deepest\n")
  cat("split factor; environment tested against E:Rep.\n\n")
  at <- core$anova_table
  print(round_df_signif(at))
  cat(sprintf("\nGrand Mean : %.4f\n", core$grand_mean))
  cat("\nCoefficient of variation by stratum (%):\n")
  cvp <- data.frame(Stratum = names(core$cv), CV = round(as.numeric(core$cv), 2))
  print(cvp, row.names = FALSE)
  invisible(NULL)
}

## small formatter shared by the pooled printer
round_df_signif <- function(df) {
  d <- df
  for (col in c("Sum Sq", "Mean Sq", "F value")) if (col %in% names(d)) d[[col]] <- round(d[[col]], 3)
  if ("Pr(>F)" %in% names(d)) d[["Pr(>F)"]] <- signif(d[["Pr(>F)"]], 4)
  d
}

#' ============================================================================
#' SPLIT PLOT DESIGN ANALYSIS FUNCTIONS (All Variations)
#' Package: aridagri
#' Author: Lalit Kumar Rolaniya
#' ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
#' ============================================================================

# Internal: assemble a correct split-plot ANOVA table from an aov() model that
# was fitted with Error() strata. Each effect keeps the F-test computed against
# its own stratum residual (so main-plot factors are tested against the pooled
# main-plot error, not the sub-plot residual). Residual rows are relabelled, in
# stratum order, as Replication, Error(a), Error(b), Error(c), ... The named
# list of error mean squares and df is attached as attribute "error_terms".
# Not exported.
.spd_error_table <- function(fit) {
  s <- summary(fit)
  rows <- list()
  err  <- list()
  for (k in seq_along(s)) {
    tab <- s[[k]][[1]]
    rn  <- rownames(tab)
    res_label <- if (k == 1L) "Replication" else paste0("Error(", letters[k - 1L], ")")
    cn  <- colnames(tab)
    for (i in seq_len(nrow(tab))) {
      lab <- trimws(rn[i])
      if (lab == "Residuals") lab <- res_label
      rows[[length(rows) + 1L]] <- data.frame(
        Df        = tab[i, "Df"],
        `Sum Sq`  = tab[i, "Sum Sq"],
        `Mean Sq` = tab[i, "Mean Sq"],
        `F value` = if ("F value" %in% cn) tab[i, "F value"] else NA_real_,
        `Pr(>F)`  = if ("Pr(>F)"  %in% cn) tab[i, "Pr(>F)"]  else NA_real_,
        row.names = lab, check.names = FALSE, stringsAsFactors = FALSE)
    }
    ri <- which(trimws(rn) == "Residuals")
    if (length(ri)) err[[res_label]] <- list(ms = tab[ri, "Mean Sq"], df = tab[ri, "Df"])
  }
  out <- do.call(rbind, rows)
  attr(out, "error_terms") <- err
  out
}

#' Split Plot Design ANOVA (Standard)
#'
#' @description
#' Performs complete ANOVA for Split Plot Design with proper error terms for 
#' main plot and sub-plot factors. Includes all standard post-hoc comparisons.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param main_plot Name of main plot factor
#' @param sub_plot Name of sub-plot factor
#' @param replication Name of replication/block factor
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table, means, and post-hoc results
#'
#' @examples
#' data <- data.frame(
#'   rep = rep(1:3, each = 12),
#'   irrigation = rep(rep(c("I1", "I2", "I3"), each = 4), 3),
#'   variety = rep(c("V1", "V2", "V3", "V4"), 9),
#'   yield = rnorm(36, 1200, 150)
#' )
#' anova_spd(data, response = "yield", main_plot = "irrigation", 
#'           sub_plot = "variety", replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_spd <- function(data, response, main_plot, sub_plot, replication, 
                       posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[main_plot]] <- as.factor(data[[main_plot]])
  data[[sub_plot]] <- as.factor(data[[sub_plot]])
  data[[replication]] <- as.factor(data[[replication]])
  
  # Get dimensions
  a <- nlevels(data[[main_plot]])      # Main plot levels
  b <- nlevels(data[[sub_plot]])       # Sub-plot levels
  r <- nlevels(data[[replication]])    # Replications
  N <- nrow(data)
  
  # Check for balanced design
  if (N != a * b * r) {
    warning("Unbalanced design detected: expected ", a * b * r, " observations (", 
            a, " main-plot x ", b, " sub-plot x ", r, " reps) but found ", N, ". ",
            "Results assume balanced data and may be unreliable. ",
            "Consider using lme4::lmer() for unbalanced data.", call. = FALSE)
  }
  
  # Fit full model
  formula_spd <- as.formula(paste(response, "~", replication, "+", 
                                   main_plot, "+", replication, ":", main_plot, "+",
                                   sub_plot, "+", main_plot, ":", sub_plot))
  
  model <- aov(formula_spd, data = data)
  anova_full <- anova(model)

  # Extract sums of squares BY TERM NAME. R sorts main-effect terms before
  # interaction terms in the ANOVA table regardless of formula order, so fixed
  # positions [3]/[4] would swap Error(a) (rep:main) with the Sub-plot effect.
  rn  <- rownames(anova_full)
  SSq <- anova_full$`Sum Sq`
  SS_rep         <- SSq[which(rn == replication)]
  SS_main        <- SSq[which(rn == main_plot)]
  SS_error_a     <- SSq[which(rn %in% c(paste0(replication, ":", main_plot),
                                        paste0(main_plot, ":", replication)))]
  SS_sub         <- SSq[which(rn == sub_plot)]
  SS_interaction <- SSq[which(rn %in% c(paste0(main_plot, ":", sub_plot),
                                        paste0(sub_plot, ":", main_plot)))]
  SS_error_b     <- SSq[length(SSq)]        # Residuals is always the last row
  SS_total <- sum(SSq)
  
  # Degrees of freedom
  df_rep <- r - 1
  df_main <- a - 1
  df_error_a <- (r - 1) * (a - 1)
  df_sub <- b - 1
  df_interaction <- (a - 1) * (b - 1)
  df_error_b <- a * (r - 1) * (b - 1)
  df_total <- N - 1
  
  # Mean squares
  MS_rep <- SS_rep / df_rep
  MS_main <- SS_main / df_main
  MS_error_a <- SS_error_a / df_error_a
  MS_sub <- SS_sub / df_sub
  MS_interaction <- SS_interaction / df_interaction
  MS_error_b <- SS_error_b / df_error_b
  
  # F-values and p-values
  F_main <- MS_main / MS_error_a
  F_sub <- MS_sub / MS_error_b
  F_interaction <- MS_interaction / MS_error_b
  
  p_main <- 1 - pf(F_main, df_main, df_error_a)
  p_sub <- 1 - pf(F_sub, df_sub, df_error_b)
  p_interaction <- 1 - pf(F_interaction, df_interaction, df_error_b)
  
  # Statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  cv_a <- sqrt(MS_error_a) / grand_mean * 100
  cv_b <- sqrt(MS_error_b) / grand_mean * 100
  
  # SE and CD calculations
  # For Main Plot
  se_main <- sqrt(MS_error_a / (b * r))
  cd_main_5 <- qt(0.975, df_error_a) * sqrt(2 * MS_error_a / (b * r))
  
  # For Sub Plot
  se_sub <- sqrt(MS_error_b / (a * r))
  cd_sub_5 <- qt(0.975, df_error_b) * sqrt(2 * MS_error_b / (a * r))
  
  # For comparing sub-plot means at same main plot level
  se_sub_same <- sqrt(MS_error_b / r)
  cd_sub_same_5 <- qt(0.975, df_error_b) * sqrt(2 * MS_error_b / r)
  
  # For comparing main plot means at same or different sub-plot levels
  # Using pooled error
  MS_pooled <- ((df_error_a * MS_error_a) + (a - 1) * (b - 1) * MS_error_b) / 
               (df_error_a + (a - 1) * (b - 1))
  se_main_sub <- sqrt((MS_error_a + (b - 1) * MS_error_b) / (b * r))
  
  # Print ANOVA table
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                    SPLIT PLOT DESIGN - ANOVA                                 \n")
    cat("\n")
    cat(" Response Variable  :", sprintf("%-55s", response), "\n")
    cat(" Main Plot Factor (A):", sprintf("%-53s", main_plot), "\n")
    cat(" Sub-Plot Factor (B) :", sprintf("%-53s", sub_plot), "\n")
    cat(" Replications        :", sprintf("%-53d", r), "\n")
    cat(" Main Plot Levels    :", sprintf("%-53d", a), "\n")
    cat(" Sub-Plot Levels     :", sprintf("%-53d", b), "\n")
    cat(" Total Observations  :", sprintf("%-53d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                           ANALYSIS OF VARIANCE                               \n")
    cat("\n")
    cat(" Source                   df        SS           MS         F     Pr>F   \n")
    cat("\n")
    cat(sprintf(" Replication            %6d  %11.2f  %11.2f                 \n",
                df_rep, SS_rep, MS_rep))
    cat(sprintf(" Main Plot (A)          %6d  %11.2f  %11.2f  %5.2f  %7.4f \n",
                df_main, SS_main, MS_main, F_main, p_main))
    cat(sprintf(" Error (a)              %6d  %11.2f  %11.2f                 \n",
                df_error_a, SS_error_a, MS_error_a))
    cat("\n")
    cat(sprintf(" Sub-Plot (B)           %6d  %11.2f  %11.2f  %5.2f  %7.4f \n",
                df_sub, SS_sub, MS_sub, F_sub, p_sub))
    cat(sprintf(" A  B                  %6d  %11.2f  %11.2f  %5.2f  %7.4f \n",
                df_interaction, SS_interaction, MS_interaction, F_interaction, p_interaction))
    cat(sprintf(" Error (b)              %6d  %11.2f  %11.2f                 \n",
                df_error_b, SS_error_b, MS_error_b))
    cat("\n")
    cat(sprintf(" Total                  %6d  %11.2f                              \n",
                df_total, SS_total))
    cat("\n")
  }
  
  # Significance symbols
  sig_main <- get_sig_symbol(p_main)
  sig_sub <- get_sig_symbol(p_sub)
  sig_int <- get_sig_symbol(p_interaction)
  
  if (verbose) {
    cat("\n\n")
    cat("                           SUMMARY STATISTICS                                 \n")
    cat("\n")
    cat(sprintf(" Grand Mean                    : %10.2f                                   \n", grand_mean))
    cat(sprintf(" CV (a) for Main Plot          : %10.2f%%                                  \n", cv_a))
    cat(sprintf(" CV (b) for Sub-Plot           : %10.2f%%                                  \n", cv_b))
    cat("\n")
    cat(sprintf(" Main Plot (A)                 : %10s                                   \n", sig_main))
    cat(sprintf(" Sub-Plot (B)                  : %10s                                   \n", sig_sub))
    cat(sprintf(" Interaction (AB)             : %10s                                   \n", sig_int))
    cat("\n")

    cat("\n\n")
    cat("                    SE AND CD VALUES FOR COMPARISONS                          \n")
    cat("\n")
    cat(" For MAIN PLOT factor (A):                                                    \n")
    cat(sprintf("   SE(m) = %8.3f    SE(d) = %8.3f    CD(5%%) = %8.3f                 \n",
                se_main, se_main * sqrt(2), cd_main_5))
    cat("\n")
    cat(" For SUB-PLOT factor (B):                                                     \n")
    cat(sprintf("   SE(m) = %8.3f    SE(d) = %8.3f    CD(5%%) = %8.3f                 \n",
                se_sub, se_sub * sqrt(2), cd_sub_5))
    cat("\n")
    cat(" For B at same level of A:                                                    \n")
    cat(sprintf("   SE(m) = %8.3f    SE(d) = %8.3f    CD(5%%) = %8.3f                 \n",
                se_sub_same, se_sub_same * sqrt(2), cd_sub_same_5))
    cat("\n")
    cat(" For A at same or different level of B:                                       \n")
    cat(sprintf("   SE(m) = %8.3f    SE(d) = %8.3f                                     \n",
                se_main_sub, se_main_sub * sqrt(2)))
    cat("\n")
  }
  
  # Factor means
  main_means <- aggregate(data[[response]], by = list(data[[main_plot]]), 
                           FUN = mean, na.rm = TRUE)
  names(main_means) <- c("Level", "Mean")
  
  sub_means <- aggregate(data[[response]], by = list(data[[sub_plot]]), 
                          FUN = mean, na.rm = TRUE)
  names(sub_means) <- c("Level", "Mean")
  
  interaction_means <- aggregate(data[[response]], 
                                  by = list(data[[main_plot]], data[[sub_plot]]), 
                                  FUN = mean, na.rm = TRUE)
  names(interaction_means) <- c("MainPlot", "SubPlot", "Mean")
  
  if (verbose) {
    cat("\n\n")
    cat("                           MAIN PLOT MEANS                                    \n")
    cat("\n")
    for (i in 1:nrow(main_means)) {
      cat(sprintf(" %-31s  %42.2f \n", 
                  as.character(main_means$Level[i]), main_means$Mean[i]))
    }
    cat("\n")

    cat("\n\n")
    cat("                           SUB-PLOT MEANS                                     \n")
    cat("\n")
    for (i in 1:nrow(sub_means)) {
      cat(sprintf(" %-31s  %42.2f \n", 
                  as.character(sub_means$Level[i]), sub_means$Mean[i]))
    }
    cat("\n")
  }
  
  # Interaction table
  int_table <- reshape(interaction_means, idvar = "MainPlot", 
                        timevar = "SubPlot", direction = "wide")
  
  if (verbose) {
    cat("\n\n")
    cat("                        INTERACTION MEANS (A  B)                             \n")
    cat("\n")
    print(int_table, row.names = FALSE)
  }
  
  # Return results
  result <- list(
    design = "Split Plot Design",
    anova_table = anova_full,
    model = model,
    grand_mean = grand_mean,
    cv_a = cv_a,
    cv_b = cv_b,
    ms_error_a = MS_error_a,
    ms_error_b = MS_error_b,
    df_error_a = df_error_a,
    df_error_b = df_error_b,
    se_main = se_main,
    cd_main = cd_main_5,
    se_sub = se_sub,
    cd_sub = cd_sub_5,
    se_sub_same = se_sub_same,
    cd_sub_same = cd_sub_same_5,
    main_means = main_means,
    sub_means = sub_means,
    interaction_means = interaction_means,
    significance = list(main = sig_main, sub = sig_sub, interaction = sig_int)
  )
  
  class(result) <- c("aridagri_spd", "list")
  return(invisible(result))
}


#' Split Plot Design with AB in Main Plot
#'
#' @description
#' Performs ANOVA for Split Plot Design where main plot contains factorial 
#' combination of two factors (AB) and sub-plot contains factor C.
#' Common in irrigation  variety as main plot and nitrogen as sub-plot.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param main_factor1 First factor in main plot (A)
#' @param main_factor2 Second factor in main plot (B)
#' @param sub_plot Sub-plot factor (C)
#' @param replication Name of replication factor
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table and means
#'
#' @details
#' Design structure:
#' \itemize{
#'   \item Main Plot: A  B factorial
#'   \item Sub-Plot: C
#'   \item Error (a): For testing A, B, and AB
#'   \item Error (b): For testing C and all interactions with C
#' }
#'
#' @examples
#' data <- data.frame(
#'   rep = rep(1:3, each = 24),
#'   irrigation = rep(rep(c("I1", "I2"), each = 12), 3),
#'   variety = rep(rep(c("V1", "V2", "V3"), each = 4), 6),
#'   nitrogen = rep(c("N0", "N1", "N2", "N3"), 18),
#'   yield = rnorm(72, 1200, 150)
#' )
#' anova_spd_ab_main(data, response = "yield", 
#'                   main_factor1 = "irrigation", main_factor2 = "variety",
#'                   sub_plot = "nitrogen", replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_spd_ab_main <- function(data, response, main_factor1, main_factor2, 
                               sub_plot, replication, posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  groups <- list(main = c(main_factor1, main_factor2), sub = sub_plot)
  core <- .aridagri_spd_anova(data, response, replication, groups, alpha)
  if (verbose)
    .aridagri_print_spd(core, "Split-Plot Design: A x B factorial in main plots")
  result <- list(
    design       = "Split-Plot (A x B main, sub)",
    anova_table  = core$anova_table,
    grand_mean   = core$grand_mean,
    cv           = core$cv,
    ms_error_a   = core$error_ms[["main"]], df_error_a = core$error_df[["main"]],
    ms_error_b   = core$error_ms[["sub"]],  df_error_b = core$error_df[["sub"]],
    factor_means = core$factor_means,
    comparisons  = core$comparisons,
    model        = core$model
  )
  class(result) <- c("aridagri_spd_ab", "list")
  return(invisible(result))
}


#' Split Plot Design with C in Main Plot, AB in Sub-Plot
#'
#' @description
#' Performs ANOVA for Split Plot Design where main plot contains single factor C 
#' and sub-plot contains factorial combination of AB.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param main_plot Main plot factor (C)
#' @param sub_factor1 First factor in sub-plot (A)
#' @param sub_factor2 Second factor in sub-plot (B)
#' @param replication Name of replication factor
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table and means
#'
#' @examples
#' data <- data.frame(
#'   rep = rep(1:3, each = 24),
#'   irrigation = rep(rep(c("I1", "I2", "I3"), each = 8), 3),
#'   variety = rep(rep(c("V1", "V2"), each = 4), 9),
#'   nitrogen = rep(c("N1", "N2", "N3", "N4"), 18),
#'   yield = rnorm(72, 1200, 150)
#' )
#' anova_spd_c_main_ab_sub(data, response = "yield",
#'                          main_plot = "irrigation",
#'                          sub_factor1 = "variety", sub_factor2 = "nitrogen",
#'                          replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_spd_c_main_ab_sub <- function(data, response, main_plot, sub_factor1, 
                                     sub_factor2, replication, posthoc = "lsd", 
                                     alpha = 0.05,
                            verbose = TRUE) {
  groups <- list(main = main_plot, sub = c(sub_factor1, sub_factor2))
  core <- .aridagri_spd_anova(data, response, replication, groups, alpha)
  if (verbose)
    .aridagri_print_spd(core, "Split-Plot Design: C in main plots, A x B factorial in sub plots")
  result <- list(
    design       = "Split-Plot (C main, A x B sub)",
    anova_table  = core$anova_table,
    grand_mean   = core$grand_mean,
    cv           = core$cv,
    ms_error_a   = core$error_ms[["main"]], df_error_a = core$error_df[["main"]],
    ms_error_b   = core$error_ms[["sub"]],  df_error_b = core$error_df[["sub"]],
    factor_means = core$factor_means,
    comparisons  = core$comparisons,
    model        = core$model
  )
  class(result) <- c("aridagri_spd_cab", "list")
  return(invisible(result))
}


#' Split Plot Design with (AB) Main and (CD) Sub
#'
#' @description
#' Performs ANOVA for Split Plot Design where main plot contains factorial 
#' combination of AB and sub-plot contains factorial combination of CD.
#' Complex design for multi-factor experiments.
#'
#' @param data Data frame containing the experimental data
#' @param response Name of the response variable
#' @param main_factor1 First factor in main plot (A)
#' @param main_factor2 Second factor in main plot (B)
#' @param sub_factor1 First factor in sub-plot (C)
#' @param sub_factor2 Second factor in sub-plot (D)
#' @param replication Name of replication factor
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA table and means
#'
#' @examples
#' # Example: Irrigation  Tillage (main), Variety  Nitrogen (sub)
#' data <- expand.grid(
#'   rep = 1:3,
#'   irrigation = c("I1", "I2"),
#'   tillage = c("CT", "ZT"),
#'   variety = c("V1", "V2"),
#'   nitrogen = c("N1", "N2", "N3")
#' )
#' data$yield <- rnorm(nrow(data), 1200, 150)
#' 
#' anova_spd_ab_cd(data, response = "yield",
#'                 main_factor1 = "irrigation", main_factor2 = "tillage",
#'                 sub_factor1 = "variety", sub_factor2 = "nitrogen",
#'                 replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_spd_ab_cd <- function(data, response, main_factor1, main_factor2,
                             sub_factor1, sub_factor2, replication,
                            verbose = TRUE) {
  groups <- list(main = c(main_factor1, main_factor2),
                 sub  = c(sub_factor1, sub_factor2))
  core <- .aridagri_spd_anova(data, response, replication, groups, 0.05)
  if (verbose)
    .aridagri_print_spd(core, "Split-Plot Design: A x B factorial main, C x D factorial sub")
  result <- list(
    design       = "Split-Plot (A x B main, C x D sub)",
    anova_table  = core$anova_table,
    grand_mean   = core$grand_mean,
    cv           = core$cv,
    ms_error_a   = core$error_ms[["main"]], df_error_a = core$error_df[["main"]],
    ms_error_b   = core$error_ms[["sub"]],  df_error_b = core$error_df[["sub"]],
    factor_means = core$factor_means,
    comparisons  = core$comparisons,
    model        = core$model
  )
  class(result) <- c("aridagri_spd_abcd", "list")
  return(invisible(result))
}


#' Pooled Split Plot Design Analysis
#'
#' @description
#' Performs pooled analysis of Split Plot Design experiments conducted 
#' across multiple environments/years/locations.
#'
#' @param data Data frame containing combined data
#' @param response Name of the response variable
#' @param main_plot Name of main plot factor
#' @param sub_plot Name of sub-plot factor
#' @param environment Name of environment factor
#' @param replication Name of replication factor (nested within environment)
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing pooled ANOVA and component analyses
#'
#' @examples
#' data <- expand.grid(
#'   year = c("Y1", "Y2", "Y3"),
#'   rep = 1:3,
#'   irrigation = c("I1", "I2", "I3"),
#'   variety = c("V1", "V2", "V3", "V4")
#' )
#' data$yield <- rnorm(nrow(data), 1200, 180)
#' 
#' anova_spd_pooled(data, response = "yield", main_plot = "irrigation",
#'                  sub_plot = "variety", environment = "year", replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_spd_pooled <- function(data, response, main_plot, sub_plot, 
                              environment, replication,
                            verbose = TRUE) {

  data[[main_plot]]   <- as.factor(data[[main_plot]])
  data[[sub_plot]]    <- as.factor(data[[sub_plot]])
  data[[environment]] <- as.factor(data[[environment]])
  data[[replication]] <- as.factor(data[[replication]])

  a <- nlevels(data[[main_plot]]); b <- nlevels(data[[sub_plot]])
  e <- nlevels(data[[environment]]); r <- nlevels(data[[replication]])
  N <- nrow(data)
  if (N != a * b * e * r) {
    warning("Unbalanced design detected: expected ", a * b * e * r,
            " observations but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }

  core <- .aridagri_spd_pooled_anova(data, response, environment, replication,
                                     split_factors = c(main_plot, sub_plot))

  main_means <- aggregate(data[[response]], by = list(data[[main_plot]]), FUN = mean)
  names(main_means) <- c("Level", "Mean")
  sub_means <- aggregate(data[[response]], by = list(data[[sub_plot]]), FUN = mean)
  names(sub_means) <- c("Level", "Mean")
  env_means <- aggregate(data[[response]], by = list(data[[environment]]), FUN = mean)
  names(env_means) <- c("Level", "Mean")

  if (verbose) {
    .aridagri_print_spd_pooled(core, "POOLED SPLIT PLOT DESIGN ANALYSIS")
    cat("\n--- Factor Means ---\n")
    cat("\nMain Plot Means:\n"); print(main_means, row.names = FALSE)
    cat("\nSub-Plot Means:\n");  print(sub_means,  row.names = FALSE)
    cat("\nEnvironment Means:\n"); print(env_means, row.names = FALSE)
  }

  result <- list(
    design      = "Pooled Split Plot Design",
    anova_table = core$anova_table,
    model       = core$model,
    grand_mean  = core$grand_mean,
    cv          = core$cv,
    main_means  = main_means,
    sub_means   = sub_means,
    env_means   = env_means
  )
  class(result) <- c("aridagri_spd_pooled", "list")
  return(invisible(result))
}

test_that("anova_spd produces correct output", {
  set.seed(789)
  data <- expand.grid(irrigation = c("I1","I2","I3"), variety = c("V1","V2","V3","V4"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 1500, 200)
  result <- anova_spd(data, response="yield", main_plot="irrigation", sub_plot="variety", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
  expect_true("cv_a" %in% names(result))
  expect_true("cv_b" %in% names(result))
})

test_that("anova_spd returns correct error df", {
  set.seed(1)
  data <- expand.grid(main = c("A","B"), sub = c("X","Y","Z"), rep = 1:4)
  data$yield <- rnorm(nrow(data), 100, 15)
  result <- anova_spd(data, response="yield", main_plot="main", sub_plot="sub", replication="rep", verbose=FALSE)
  expect_equal(result$df_error_a, 3)
})

test_that("anova_spd_ab_main produces correct output", {
  set.seed(111)
  data <- expand.grid(f1 = c("A","B"), f2 = c("X","Y"), sub = c("S1","S2","S3"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 200, 25)
  result <- anova_spd_ab_main(data, response="yield", main_factor1="f1", main_factor2="f2", sub_plot="sub", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

test_that("anova_spd_c_main_ab_sub produces correct output", {
  set.seed(222)
  data <- expand.grid(main = c("M1","M2","M3"), sf1 = c("A","B"), sf2 = c("X","Y"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 300, 30)
  result <- anova_spd_c_main_ab_sub(data, response="yield", main_plot="main", sub_factor1="sf1", sub_factor2="sf2", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

test_that("anova_spd_ab_cd produces correct output", {
  set.seed(333)
  data <- expand.grid(mf1 = c("A","B"), mf2 = c("X","Y"), sf1 = c("P","Q"), sf2 = c("R","S"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 400, 40)
  result <- anova_spd_ab_cd(data, response="yield", main_factor1="mf1", main_factor2="mf2", sub_factor1="sf1", sub_factor2="sf2", replication="rep", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_spd_pooled produces correct output", {
  set.seed(444)
  data <- expand.grid(main = c("A","B"), sub = c("X","Y","Z"), env = paste0("E",1:3), rep = 1:2)
  data$yield <- rnorm(nrow(data), 500, 50)
  result <- anova_spd_pooled(data, response="yield", main_plot="main", sub_plot="sub", environment="env", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

test_that("anova_sspd produces correct output", {
  set.seed(101)
  data <- expand.grid(main = c("M1","M2"), sub = c("S1","S2","S3"), subsub = c("SS1","SS2"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 200, 30)
  result <- anova_sspd(data, response="yield", main_plot="main", sub_plot="sub", sub_sub_plot="subsub", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

test_that("anova_sspd_pooled produces correct output", {
  set.seed(555)
  data <- expand.grid(main = c("A","B"), sub = c("X","Y"), subsub = c("P","Q"), env = paste0("E",1:2), rep = 1:2)
  data$yield <- rnorm(nrow(data), 600, 60)
  result <- anova_sspd_pooled(data, response="yield", main_plot="main", sub_plot="sub", sub_sub_plot="subsub", environment="env", replication="rep", verbose=FALSE)
  expect_type(result, "list")
})

test_that("anova_strip produces correct output", {
  set.seed(202)
  data <- expand.grid(tillage = c("ZT","CT","MT"), nitrogen = c("N0","N60","N120"), rep = 1:3)
  data$yield <- rnorm(nrow(data), 2000, 250)
  result <- anova_strip(data, response="yield", horizontal_factor="tillage", vertical_factor="nitrogen", replication="rep", verbose=FALSE)
  expect_type(result, "list")
  expect_true("anova_table" %in% names(result))
})

#' ============================================================================
#' POST-HOC TESTS AND HELPER FUNCTIONS
#' Package: aridagri
#' Author: Lalit Kumar Rolaniya
#' ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
#' ============================================================================

#' Get Significance Symbol
#'
#' @description
#' Returns significance symbol based on p-value.
#'
#' @param p_value P-value from statistical test
#' @return Character string with significance symbol
#' @keywords internal
get_sig_symbol <- function(p_value) {
  if (is.na(p_value)) return("")
  if (p_value < 0.001) return("***")
  if (p_value < 0.01) return("**")
  if (p_value < 0.05) return("*")
  return("NS")
}


#' Perform Post-Hoc Tests
#'
#' @description
#' Performs multiple comparison tests after ANOVA.
#'
#' @param model ANOVA model object
#' @param data Data frame
#' @param response Response variable name
#' @param treatment Treatment factor name
#' @param mse Mean square error
#' @param df_error Error degrees of freedom
#' @param posthoc Test type: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing post-hoc test results
#' @export
perform_posthoc <- function(model, data, response, treatment, mse = NULL,
                            df_error = NULL, posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {

  # Derive the error mean square and error degrees of freedom from the fitted
  # model when they are not supplied. This lets perform_posthoc() be used
  # standalone (e.g. Bonferroni, or LSD/Tukey when 'agricolae' is absent)
  # without the caller having to compute them by hand.
  if (is.null(mse) || is.null(df_error)) {
    aov_tab <- tryCatch(anova(model), error = function(e) NULL)
    if (!is.null(aov_tab) && "Mean Sq" %in% colnames(aov_tab)) {
      resid_row <- nrow(aov_tab)
      if (is.null(df_error)) df_error <- aov_tab[resid_row, "Df"]
      if (is.null(mse))      mse      <- aov_tab[resid_row, "Mean Sq"]
    } else {
      df_e <- tryCatch(df.residual(model), error = function(e) NULL)
      if (is.null(df_e)) {
        stop("Could not derive 'mse' and 'df_error' from 'model'; ",
             "please supply them explicitly.")
      }
      if (is.null(df_error)) df_error <- df_e
      if (is.null(mse))      mse      <- sum(residuals(model)^2) / df_e
    }
  }

  # Get number of replications per treatment
  n_per_trt <- table(data[[treatment]])
  r <- mean(n_per_trt)  # Assumes balanced design
  
  results <- list()
  
  if (verbose) {
    cat("\n\n")
    cat("                        POST-HOC COMPARISONS                          \n")
    cat("\n")
  }
  
  # LSD Test
  if (posthoc %in% c("lsd", "all")) {
    if (verbose) cat("\n--- Fisher's Least Significant Difference (LSD) Test ---\n")

    if (requireNamespace("agricolae", quietly = TRUE)) {
      lsd_result <- agricolae::LSD.test(model, treatment, alpha = alpha)

      if (verbose) {
        cat(sprintf("\nLSD value ( = %.2f): %.3f\n", alpha, lsd_result$statistics$LSD))
        cat("\nTreatment Groups:\n")

        # Create formatted output
        groups_df <- lsd_result$groups
        groups_df$treatment <- rownames(groups_df)
        groups_df <- groups_df[order(-groups_df[, 1]), ]

        cat("\n")
        print(groups_df[, c("treatment", names(groups_df)[1], "groups")], row.names = FALSE)
      }

      results$lsd <- lsd_result
    } else {
      # Manual LSD calculation
      t_value <- qt(1 - alpha/2, df_error)
      lsd_value <- t_value * sqrt(2 * mse / r)

      if (verbose) {
        cat(sprintf("\nLSD value ( = %.2f): %.3f\n", alpha, lsd_value))
        cat("(Install 'agricolae' package for grouping letters)\n")
      }

      results$lsd <- list(value = lsd_value, t_value = t_value)
    }
  }
  
  # Duncan's Multiple Range Test
  if (posthoc %in% c("duncan", "all")) {
    if (verbose) cat("\n--- Duncan's Multiple Range Test (DMRT) ---\n")

    if (requireNamespace("agricolae", quietly = TRUE)) {
      duncan_result <- agricolae::duncan.test(model, treatment, alpha = alpha)

      if (verbose) {
        cat("\nTreatment Groups:\n")
        groups_df <- duncan_result$groups
        groups_df$treatment <- rownames(groups_df)
        groups_df <- groups_df[order(-groups_df[, 1]), ]

        cat("\n")
        print(groups_df[, c("treatment", names(groups_df)[1], "groups")], row.names = FALSE)
      }

      results$duncan <- duncan_result
    } else {
      if (verbose) cat("(Install 'agricolae' package for Duncan's test)\n")
    }
  }
  
  # Tukey's HSD Test
  if (posthoc %in% c("tukey", "all")) {
    if (verbose) cat("\n--- Tukey's Honest Significant Difference (HSD) Test ---\n")

    if (requireNamespace("agricolae", quietly = TRUE)) {
      hsd_result <- agricolae::HSD.test(model, treatment, alpha = alpha)

      if (verbose) {
        cat(sprintf("\nHSD value ( = %.2f): %.3f\n", alpha, hsd_result$statistics$HSD))
        cat("\nTreatment Groups:\n")

        groups_df <- hsd_result$groups
        groups_df$treatment <- rownames(groups_df)
        groups_df <- groups_df[order(-groups_df[, 1]), ]

        cat("\n")
        print(groups_df[, c("treatment", names(groups_df)[1], "groups")], row.names = FALSE)
      }

      results$tukey <- hsd_result
    } else {
      # Manual Tukey calculation
      q_value <- qtukey(1 - alpha, nlevels(data[[treatment]]), df_error)
      hsd_value <- q_value * sqrt(mse / r)

      if (verbose) {
        cat(sprintf("\nHSD value ( = %.2f): %.3f\n", alpha, hsd_value))
        cat("(Install 'agricolae' package for grouping letters)\n")
      }

      results$tukey <- list(value = hsd_value, q_value = q_value)
    }
  }
  
  # Student-Newman-Keuls (SNK) Test
  if (posthoc %in% c("snk", "all")) {
    if (verbose) cat("\n--- Student-Newman-Keuls (SNK) Test ---\n")

    if (requireNamespace("agricolae", quietly = TRUE)) {
      snk_result <- agricolae::SNK.test(model, treatment, alpha = alpha)

      if (verbose) {
        cat("\nTreatment Groups:\n")
        groups_df <- snk_result$groups
        groups_df$treatment <- rownames(groups_df)
        groups_df <- groups_df[order(-groups_df[, 1]), ]

        cat("\n")
        print(groups_df[, c("treatment", names(groups_df)[1], "groups")], row.names = FALSE)
      }

      results$snk <- snk_result
    } else {
      if (verbose) cat("(Install 'agricolae' package for SNK test)\n")
    }
  }
  
  # Scheffe's Test
  if (posthoc %in% c("scheffe", "all")) {
    if (verbose) cat("\n--- Scheff's Test ---\n")

    if (requireNamespace("agricolae", quietly = TRUE)) {
      scheffe_result <- agricolae::scheffe.test(model, treatment, alpha = alpha)

      if (verbose) {
        cat("\nTreatment Groups:\n")
        groups_df <- scheffe_result$groups
        groups_df$treatment <- rownames(groups_df)
        groups_df <- groups_df[order(-groups_df[, 1]), ]

        cat("\n")
        print(groups_df[, c("treatment", names(groups_df)[1], "groups")], row.names = FALSE)
      }

      results$scheffe <- scheffe_result
    } else {
      if (verbose) cat("(Install 'agricolae' package for Scheff test)\n")
    }
  }
  
  # Dunnett's Test (comparison with control)
  if (posthoc %in% c("dunnett", "all")) {
    if (verbose) cat("\n--- Dunnett's Test (vs Control) ---\n")

    if (requireNamespace("multcomp", quietly = TRUE)) {
      # Build the mcp() argument dynamically so the actual treatment factor
      # name (whatever the user passed) is used, not the literal 'treatment'.
      # First factor level is treated as the control.
      mcp_arg <- stats::setNames(list("Dunnett"), treatment)
      dunnett_result <- multcomp::glht(model,
                                       linfct = do.call(multcomp::mcp, mcp_arg))
      dunnett_summary <- summary(dunnett_result)
      if (verbose) print(dunnett_summary)
      results$dunnett <- dunnett_result
    } else {
      if (verbose) cat("(Install 'multcomp' package for Dunnett's test)\n")
    }
  }
  
  # Bonferroni Correction
  if (posthoc %in% c("bonferroni", "all")) {
    if (verbose) {
      cat("\n--- Bonferroni-Corrected Comparisons ---\n")
    }
    
    t <- nlevels(data[[treatment]])
    n_comparisons <- t * (t - 1) / 2
    alpha_bonf <- alpha / n_comparisons
    t_bonf <- qt(1 - alpha_bonf/2, df_error)
    bonf_value <- t_bonf * sqrt(2 * mse / r)
    
    if (verbose) {
      cat(sprintf("Number of pairwise comparisons: %d\n", n_comparisons))
      cat(sprintf("Bonferroni-corrected : %.5f\n", alpha_bonf))
      cat(sprintf("Critical t-value: %.3f\n", t_bonf))
      cat(sprintf("Bonferroni CD: %.3f\n", bonf_value))
    }
    
    results$bonferroni <- list(
      n_comparisons = n_comparisons,
      alpha_corrected = alpha_bonf,
      t_value = t_bonf,
      cd_value = bonf_value
    )
  }
  
  return(invisible(results))
}


#' Check ANOVA Assumptions
#'
#' @description
#' Tests assumptions of ANOVA: normality and homogeneity of variances.
#'
#' @param model ANOVA model object
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing diagnostic test results
#' @export
check_assumptions <- function(model,
                            verbose = TRUE) {
  
  residuals <- residuals(model)
  fitted_values <- fitted(model)
  
  if (verbose) {
    cat("\n\n")
    cat("                    ANOVA ASSUMPTION CHECKS                           \n")
    cat("\n")
  }
  
  results <- list()
  
  # 1. Normality Test (Shapiro-Wilk)
  if (verbose) cat("\n--- Normality of Residuals (Shapiro-Wilk Test) ---\n")

  if (length(residuals) <= 5000) {
    shapiro_test <- shapiro.test(residuals)

    if (verbose) {
      cat(sprintf("W statistic: %.4f\n", shapiro_test$statistic))
      cat(sprintf("P-value: %.4f\n", shapiro_test$p.value))

      if (shapiro_test$p.value >= 0.05) {
        cat("Conclusion: Residuals are normally distributed (p >= 0.05)\n")
      } else {
        cat("WARNING: Residuals may not be normally distributed (p < 0.05)\n")
        cat("Consider data transformation (log, sqrt, arcsin)\n")
      }
    }

    results$shapiro <- shapiro_test
  } else {
    if (verbose) {
      cat("Sample size too large for Shapiro-Wilk test (n > 5000)\n")
      cat("Using Anderson-Darling test instead\n")
    }
  }
  
  # 2. Homogeneity of Variances (Levene's Test)
  if (verbose) {
    cat("\n--- Homogeneity of Variances (Bartlett's Test) ---\n")
  }
  
  model_data <- model$model
  response_name <- names(model_data)[1]
  
  # Find factor columns
  factor_cols <- sapply(model_data, is.factor)
  if (sum(factor_cols) > 0) {
    treatment_col <- names(model_data)[factor_cols][1]
    
    bartlett_test <- bartlett.test(model_data[[response_name]] ~ model_data[[treatment_col]])
    
    if (verbose) {
      cat(sprintf("Bartlett's K-squared: %.4f\n", bartlett_test$statistic))
      cat(sprintf("df: %d\n", bartlett_test$parameter))
      cat(sprintf("P-value: %.4f\n", bartlett_test$p.value))

      if (bartlett_test$p.value >= 0.05) {
        cat("Conclusion: Variances are homogeneous (p >= 0.05)\n")
      } else {
        cat("WARNING: Variances may be heterogeneous (p < 0.05)\n")
        cat("Consider Welch's ANOVA or data transformation\n")
      }
    }
    
    results$bartlett <- bartlett_test
  }
  
  # 3. Independence check (Durbin-Watson)
  if (verbose) {
    cat("\n--- Independence of Residuals (Durbin-Watson) ---\n")
  }
  
  # Calculate Durbin-Watson statistic manually
  n <- length(residuals)
  dw_num <- sum((residuals[-1] - residuals[-n])^2)
  dw_den <- sum(residuals^2)
  dw_stat <- dw_num / dw_den
  
  if (verbose) {
    cat(sprintf("Durbin-Watson statistic: %.4f\n", dw_stat))

    if (dw_stat > 1.5 && dw_stat < 2.5) {
      cat("Conclusion: No significant autocorrelation detected\n")
    } else {
      cat("NOTE: Possible autocorrelation in residuals\n")
    }
  }
  
  results$durbin_watson <- dw_stat
  
  # 4. Outlier detection
  if (verbose) {
    cat("\n--- Outlier Detection ---\n")
  }
  
  # Standardized residuals
  std_residuals <- scale(residuals)
  outliers <- which(abs(std_residuals) > 3)
  
  if (length(outliers) > 0) {
    if (verbose) {
      cat(sprintf("Potential outliers (|standardized residual| > 3): %d observations\n", 
                  length(outliers)))
      cat("Observation indices:", paste(outliers, collapse = ", "), "\n")
    }
  } else {
    if (verbose) {
      cat("No extreme outliers detected (all |standardized residuals| <= 3)\n")
    }
  }
  
  results$outliers <- outliers
  
  return(invisible(results))
}


#' Factorial ANOVA (Two-Factor)
#'
#' @description
#' Performs two-factor factorial ANOVA with interaction analysis.
#'
#' @param data Data frame containing the data
#' @param response Name of the response variable
#' @param factor1 Name of first factor (A)
#' @param factor2 Name of second factor (B)
#' @param replication Name of replication factor (optional)
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results
#'
#' @examples
#' data <- expand.grid(
#'   rep = 1:4,
#'   nitrogen = c("N0", "N40", "N80", "N120"),
#'   phosphorus = c("P0", "P30", "P60")
#' )
#' data$yield <- rnorm(nrow(data), 1200, 150)
#' 
#' anova_factorial(data, response = "yield", 
#'                 factor1 = "nitrogen", factor2 = "phosphorus",
#'                 replication = "rep")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_factorial <- function(data, response, factor1, factor2, 
                            replication = NULL, posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[factor1]] <- as.factor(data[[factor1]])
  data[[factor2]] <- as.factor(data[[factor2]])
  
  # Get dimensions
  a <- nlevels(data[[factor1]])
  b <- nlevels(data[[factor2]])
  N <- nrow(data)
  
  # Build formula
  if (!is.null(replication)) {
    data[[replication]] <- as.factor(data[[replication]])
    r <- nlevels(data[[replication]])
    formula_fact <- as.formula(paste(response, "~", replication, "+", 
                                      factor1, "*", factor2))
  } else {
    r <- N / (a * b)
    formula_fact <- as.formula(paste(response, "~", factor1, "*", factor2))
  }
  
  # Check for balanced design
  expected_n <- a * b * r
  if (abs(N - expected_n) > 0.5) {
    warning("Unbalanced design detected: expected ", expected_n, " observations but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }
  
  # Fit model
  model <- aov(formula_fact, data = data)
  anova_table <- anova(model)
  
  # Print header
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                TWO-FACTOR FACTORIAL ANOVA                            \n")
    cat("\n")
    cat(" Response Variable    :", sprintf("%-46s", response), "\n")
    cat(" Factor A             :", sprintf("%-46s", factor1), "\n")
    cat(" Factor B             :", sprintf("%-46s", factor2), "\n")
    cat(" Levels of A          :", sprintf("%-46d", a), "\n")
    cat(" Levels of B          :", sprintf("%-46d", b), "\n")
    cat(" Replications         :", sprintf("%-46.0f", r), "\n")
    cat(" Total Observations   :", sprintf("%-46d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")

    print(anova_table)
  }
  
  # Statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  ms_error <- anova_table$`Mean Sq`[nrow(anova_table)]
  df_error <- anova_table$Df[nrow(anova_table)]
  cv <- sqrt(ms_error) / grand_mean * 100
  
  # SE and CD
  se_a <- sqrt(ms_error / (b * r))
  cd_a <- qt(0.975, df_error) * sqrt(2 * ms_error / (b * r))
  
  se_b <- sqrt(ms_error / (a * r))
  cd_b <- qt(0.975, df_error) * sqrt(2 * ms_error / (a * r))
  
  se_ab <- sqrt(ms_error / r)
  cd_ab <- qt(0.975, df_error) * sqrt(2 * ms_error / r)
  
  if (verbose) {
    cat("\n\n")
    cat("                        SUMMARY STATISTICS                            \n")
    cat("\n")
    cat(sprintf(" Grand Mean          : %10.2f                                     \n", grand_mean))
    cat(sprintf(" CV (%%)              : %10.2f                                     \n", cv))
    cat("\n")
    cat(sprintf(" Factor A: SE(m)=%7.3f  SE(d)=%7.3f  CD(5%%)=%7.3f           \n",
                se_a, se_a*sqrt(2), cd_a))
    cat(sprintf(" Factor B: SE(m)=%7.3f  SE(d)=%7.3f  CD(5%%)=%7.3f           \n",
                se_b, se_b*sqrt(2), cd_b))
    cat(sprintf(" A  B   : SE(m)=%7.3f  SE(d)=%7.3f  CD(5%%)=%7.3f           \n",
                se_ab, se_ab*sqrt(2), cd_ab))
    cat("\n")
  }
  
  # Means
  mean_a <- aggregate(data[[response]], by = list(data[[factor1]]), FUN = mean)
  names(mean_a) <- c("Level", "Mean")
  
  mean_b <- aggregate(data[[response]], by = list(data[[factor2]]), FUN = mean)
  names(mean_b) <- c("Level", "Mean")
  
  mean_ab <- aggregate(data[[response]], 
                        by = list(data[[factor1]], data[[factor2]]), 
                        FUN = mean)
  names(mean_ab) <- c("A", "B", "Mean")
  
  if (verbose) {
    cat("\n--- Factor A Means ---\n")
    print(mean_a, row.names = FALSE)

    cat("\n--- Factor B Means ---\n")
    print(mean_b, row.names = FALSE)

    cat("\n--- A  B Interaction Means ---\n")
  }
  ab_table <- reshape(mean_ab, idvar = "A", timevar = "B", direction = "wide")
  if (verbose) {
    print(ab_table, row.names = FALSE)
  }
  
  # Return
  result <- list(
    design = "Two-Factor Factorial",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    se_a = se_a, cd_a = cd_a,
    se_b = se_b, cd_b = cd_b,
    se_ab = se_ab, cd_ab = cd_ab,
    mean_a = mean_a,
    mean_b = mean_b,
    mean_ab = ab_table
  )
  
  class(result) <- c("aridagri_factorial", "list")
  return(invisible(result))
}

# NOTE: anova_factorial_3way is defined in design_additional.R


#' Latin Square Design ANOVA
#'
#' @description
#' Performs ANOVA for Latin Square Design with row and column blocking.
#'
#' @param data Data frame containing the data
#' @param response Name of the response variable
#' @param treatment Name of treatment factor
#' @param row Name of row factor
#' @param column Name of column factor
#' @param posthoc Post-hoc test method
#' @param alpha Significance level
#'
#' @param verbose Logical. If TRUE (default), prints formatted output to console.
#'
#' @return List containing ANOVA results
#'
#' @examples
#' # 5x5 Latin Square
#' data <- data.frame(
#'   row = rep(1:5, each = 5),
#'   col = rep(1:5, 5),
#'   treatment = c("A","B","C","D","E", "B","C","D","E","A",
#'                 "C","D","E","A","B", "D","E","A","B","C",
#'                 "E","A","B","C","D"),
#'   yield = rnorm(25, 1200, 100)
#' )
#' anova_latin(data, response = "yield", treatment = "treatment",
#'             row = "row", column = "col")
#'
#' @export
#' @author Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
anova_latin <- function(data, response, treatment, row, column,
                         posthoc = "lsd", alpha = 0.05,
                            verbose = TRUE) {
  
  # Convert to factors
  data[[treatment]] <- as.factor(data[[treatment]])
  data[[row]] <- as.factor(data[[row]])
  data[[column]] <- as.factor(data[[column]])
  
  # Get dimensions
  t <- nlevels(data[[treatment]])
  N <- nrow(data)
  
  # Check for balanced design (Latin square requires t x t observations)
  if (N != t * t) {
    warning("Unbalanced design detected: Latin square expects ", t * t, " observations (", 
            t, " x ", t, ") but found ", N, ". ",
            "Results assume balanced data and may be unreliable.", call. = FALSE)
  }
  
  # Fit model
  formula_lsd <- as.formula(paste(response, "~", row, "+", column, "+", treatment))
  model <- aov(formula_lsd, data = data)
  anova_table <- anova(model)
  
  # Print header
  if (verbose) {
    cat("\n")
    cat("\n")
    cat("                   LATIN SQUARE DESIGN - ANOVA                        \n")
    cat("\n")
    cat(" Response Variable    :", sprintf("%-46s", response), "\n")
    cat(" Treatment Factor     :", sprintf("%-46s", treatment), "\n")
    cat(" Row Factor           :", sprintf("%-46s", row), "\n")
    cat(" Column Factor        :", sprintf("%-46s", column), "\n")
    cat(" Size (t  t)         :", sprintf("%-46s", paste0(t, "  ", t)), "\n")
    cat(" Total Observations   :", sprintf("%-46d", N), "\n")
    cat("\n")

    cat("\n\n")
    cat("                        ANALYSIS OF VARIANCE                          \n")
    cat("\n")

    print(anova_table)
  }
  
  # Statistics
  grand_mean <- mean(data[[response]], na.rm = TRUE)
  ms_error <- anova_table$`Mean Sq`[4]
  df_error <- anova_table$Df[4]
  cv <- sqrt(ms_error) / grand_mean * 100
  
  se_mean <- sqrt(ms_error / t)
  cd_5 <- qt(0.975, df_error) * sqrt(2 * ms_error / t)
  
  if (verbose) {
    cat("\n\n")
    cat("                        SUMMARY STATISTICS                            \n")
    cat("\n")
    cat(sprintf(" Grand Mean          : %10.2f                                     \n", grand_mean))
    cat(sprintf(" CV (%%)              : %10.2f                                     \n", cv))
    cat(sprintf(" SE(m)               : %10.3f                                     \n", se_mean))
    cat(sprintf(" SE(d)               : %10.3f                                     \n", se_mean * sqrt(2)))
    cat(sprintf(" CD (5%%)             : %10.3f                                     \n", cd_5))
    cat("\n")
  }
  
  # Treatment means
  trt_means <- aggregate(data[[response]], by = list(data[[treatment]]), FUN = mean)
  names(trt_means) <- c("Treatment", "Mean")
  trt_means <- trt_means[order(-trt_means$Mean), ]
  
  if (verbose) {
    cat("\n--- Treatment Means ---\n")
    print(trt_means, row.names = FALSE)
  }
  
  # Post-hoc
  posthoc_results <- perform_posthoc(model, data, response, treatment,
                                      ms_error, df_error, posthoc, alpha,
                                      verbose = verbose)
  
  # Return
  result <- list(
    design = "Latin Square Design",
    anova_table = anova_table,
    model = model,
    grand_mean = grand_mean,
    cv = cv,
    se_mean = se_mean,
    cd = cd_5,
    treatment_means = trt_means,
    posthoc = posthoc_results
  )
  
  class(result) <- c("aridagri_latin", "list")
  return(invisible(result))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{thermal_indices}
\alias{thermal_indices}
\title{Growing Degree Days (GDD) and Thermal Indices}
\usage{
thermal_indices(
  tmax,
  tmin,
  base_temp = 10,
  sunshine_hours = NULL,
  day_length = NULL,
  crop_yield = NULL,
  biomass = NULL,
  verbose = TRUE
)
}
\arguments{
\item{tmax}{Vector of daily maximum temperatures (C)}

\item{tmin}{Vector of daily minimum temperatures (C)}

\item{base_temp}{Base temperature (C)}

\item{sunshine_hours}{Vector of daily sunshine hours (optional)}

\item{day_length}{Vector of day length in hours (optional)}

\item{crop_yield}{Crop yield for HUE calculation (optional)}

\item{biomass}{Biomass for HUE calculation (optional)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List with thermal indices
}
\description{
Calculates Growing Degree Days, Helio-thermal Units, Photo-thermal Units,
and Heat Use Efficiency from temperature data.
}
\examples{
tmax <- runif(90, 30, 42)
tmin <- runif(90, 18, 28)
thermal_indices(tmax, tmin, base_temp = 10)

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_additional.R
\name{anova_strip}
\alias{anova_strip}
\title{============================================================================
ADDITIONAL EXPERIMENTAL DESIGN FUNCTIONS
Package: aridagri
Authors: Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
============================================================================
Strip Plot Design ANOVA}
\usage{
anova_strip(
  data,
  response,
  horizontal_factor,
  vertical_factor,
  replication,
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{horizontal_factor}{Factor applied in horizontal strips (A)}

\item{vertical_factor}{Factor applied in vertical strips (B)}

\item{replication}{Name of replication factor}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results
}
\description{
Performs ANOVA for Strip Plot (Strip-Split) Design where two factors
are applied in horizontal and vertical strips.
}
\examples{
data <- expand.grid(
  rep = 1:4,
  irrigation = c("I1", "I2", "I3"),
  tillage = c("CT", "MT", "ZT")
)
data$yield <- rnorm(nrow(data), 1200, 150)
anova_strip(data, response = "yield", horizontal_factor = "irrigation",
            vertical_factor = "tillage", replication = "rep")

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{crop_growth_analysis}
\alias{crop_growth_analysis}
\title{Crop Growth Analysis (CGR, RGR, NAR, LAI)}
\usage{
crop_growth_analysis(dry_weight, leaf_area, days, verbose = TRUE)
}
\arguments{
\item{dry_weight}{Vector of dry matter at different stages}

\item{leaf_area}{Vector of leaf area at different stages}

\item{days}{Vector of days after sowing}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with growth parameters
}
\description{
Calculates crop growth parameters from sequential harvest data.
}
\examples{
dry_weight <- c(0.5, 2.1, 8.5, 25, 45, 62, 75)
leaf_area <- c(15, 85, 350, 800, 950, 850, 600)
days <- c(15, 30, 45, 60, 75, 90, 105)
crop_growth_analysis(dry_weight, leaf_area, days)

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posthoc_helpers.R
\name{anova_factorial}
\alias{anova_factorial}
\title{Factorial ANOVA (Two-Factor)}
\usage{
anova_factorial(
  data,
  response,
  factor1,
  factor2,
  replication = NULL,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the data}

\item{response}{Name of the response variable}

\item{factor1}{Name of first factor (A)}

\item{factor2}{Name of second factor (B)}

\item{replication}{Name of replication factor (optional)}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results
}
\description{
Performs two-factor factorial ANOVA with interaction analysis.
}
\examples{
data <- expand.grid(
  rep = 1:4,
  nitrogen = c("N0", "N40", "N80", "N120"),
  phosphorus = c("P0", "P30", "P60")
)
data$yield <- rnorm(nrow(data), 1200, 150)

anova_factorial(data, response = "yield", 
                factor1 = "nitrogen", factor2 = "phosphorus",
                replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistical_functions.R
\name{anova_sspd}
\alias{anova_sspd}
\title{Split-Split Plot Design ANOVA}
\usage{
anova_sspd(
  data,
  response,
  main_plot,
  sub_plot,
  sub_sub_plot,
  replication,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable (as string)}

\item{main_plot}{Name of main plot factor}

\item{sub_plot}{Name of sub-plot factor}

\item{sub_sub_plot}{Name of sub-sub-plot factor}

\item{replication}{Name of replication/block factor}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table, means, and significance tests
}
\description{
Performs complete ANOVA for Split-Split Plot Design with proper error terms
for main plot, sub-plot, and sub-sub-plot factors. Generates publication-ready
ANOVA table with significance levels.
}
\examples{
# Example with sample data
data <- expand.grid(rep=1:3, A=c('A1','A2'), B=c('B1','B2'), C=c('C1','C2'))
data$yield <- rnorm(24, 1200, 150)
anova_sspd(data, response='yield', main_plot='A', sub_plot='B',
           sub_sub_plot='C', replication='rep')

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_split_plot.R
\name{anova_spd_c_main_ab_sub}
\alias{anova_spd_c_main_ab_sub}
\title{Split Plot Design with C in Main Plot, AB in Sub-Plot}
\usage{
anova_spd_c_main_ab_sub(
  data,
  response,
  main_plot,
  sub_factor1,
  sub_factor2,
  replication,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{main_plot}{Main plot factor (C)}

\item{sub_factor1}{First factor in sub-plot (A)}

\item{sub_factor2}{Second factor in sub-plot (B)}

\item{replication}{Name of replication factor}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table and means
}
\description{
Performs ANOVA for Split Plot Design where main plot contains single factor C 
and sub-plot contains factorial combination of AB.
}
\examples{
data <- data.frame(
  rep = rep(1:3, each = 24),
  irrigation = rep(rep(c("I1", "I2", "I3"), each = 8), 3),
  variety = rep(rep(c("V1", "V2"), each = 4), 9),
  nitrogen = rep(c("N1", "N2", "N3", "N4"), 18),
  yield = rnorm(72, 1200, 150)
)
anova_spd_c_main_ab_sub(data, response = "yield",
                         main_plot = "irrigation",
                         sub_factor1 = "variety", sub_factor2 = "nitrogen",
                         replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aridagri-package.R
\docType{package}
\name{aridagri-package}
\alias{aridagri-package}
\alias{aridagri}
\title{aridagri: Comprehensive Statistical Tools for Agricultural Research}
\description{
The aridagri package provides comprehensive statistical and analytical tools 
for agricultural research. It includes complete ANOVA functions for all 
experimental designs, multiple post-hoc tests, stability analysis methods,
thermal indices, crop growth analysis, and advanced statistical methods.
}
\section{Experimental Design ANOVA Functions}{

\itemize{
  \item \code{\link{anova_crd}}: Completely Randomized Design
  \item \code{\link{anova_rbd}}: Randomized Block Design
  \item \code{\link{anova_rbd_pooled}}: Pooled RBD (Multi-Environment)
  \item \code{\link{anova_latin}}: Latin Square Design
  \item \code{\link{anova_factorial}}: Two-Factor Factorial
  \item \code{\link{anova_factorial_3way}}: Three-Factor Factorial
  \item \code{\link{anova_spd}}: Split Plot Design
  \item \code{\link{anova_sspd}}: Split-Split Plot Design
  \item \code{\link{anova_strip}}: Strip Plot Design
  \item \code{\link{anova_augmented}}: Augmented Block Design
  \item \code{\link{anova_alpha_lattice}}: Alpha Lattice Design
}
}

\section{Post-Hoc Tests}{

\itemize{
  \item \code{\link{perform_posthoc}}: Multiple comparison tests (LSD, Duncan, Tukey, SNK, Scheffe, Bonferroni, Dunnett)
  \item \code{\link{check_assumptions}}: ANOVA assumption checking
}
}

\section{Agronomic Analysis Functions}{

\itemize{
  \item \code{\link{stability_analysis}}: Multi-method stability analysis (Eberhart-Russell, AMMI, Finlay-Wilkinson, Shukla, Wricke, CV, Superiority)
  \item \code{\link{thermal_indices}}: GDD, HTU, PTU, Heat Use Efficiency
  \item \code{\link{crop_growth_analysis}}: CGR, RGR, NAR, LAI
  \item \code{\link{harvest_index}}: Harvest index and partitioning
  \item \code{\link{yield_gap_analysis}}: Yield gap calculations
  \item \code{\link{economic_indices}}: B:C ratio, net returns
}
}

\section{Statistical Functions}{

\itemize{
  \item \code{\link{correlation_analysis}}: Correlation matrix with significance
  \item \code{\link{pca_analysis}}: Principal component analysis
  \item \code{\link{path_analysis}}: Path coefficient analysis
  \item \code{\link{sem_analysis}}: Structural equation modeling
}
}

\section{Nutrient Analysis Functions}{

\itemize{
  \item \code{\link{nue_calculate}}: Nutrient use efficiency calculations
  \item \code{\link{nutrient_response}}: Response curve analysis
  \item \code{\link{economic_analysis}}: Economic viability assessment
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/lalitrolaniya/aridagri}
  \item Report bugs at \url{https://github.com/lalitrolaniya/aridagri/issues}
}

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_additional.R
\name{anova_augmented}
\alias{anova_augmented}
\title{Augmented Block Design ANOVA}
\usage{
anova_augmented(data, response, genotype, block, check_names, verbose = TRUE)
}
\arguments{
\item{data}{Data frame containing the data}

\item{response}{Name of response variable}

\item{genotype}{Name of genotype/entry column}

\item{block}{Name of block column}

\item{check_names}{Vector of check variety names}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results and adjusted means
}
\description{
Performs ANOVA for Augmented Randomized Block Design where checks are
replicated and test entries appear once.
}
\examples{
data <- data.frame(
  block = rep(1:5, each = 7),
  genotype = c(rep(c("C1","C2","C3"), 5), paste0("T", 1:20)),
  yield = rnorm(35, 1200, 150)
)
# Note: This is simplified example

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistical_functions.R
\name{path_analysis}
\alias{path_analysis}
\title{Path Coefficient Analysis}
\usage{
path_analysis(data, dependent, independent, digits = 4, verbose = TRUE)
}
\arguments{
\item{data}{Data frame with numeric variables}

\item{dependent}{Name of dependent variable (e.g., "yield")}

\item{independent}{Character vector of independent variable names}

\item{digits}{Number of decimal places}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List with direct effects, indirect effects, and correlation breakdown
}
\description{
Performs path analysis to determine direct and indirect effects of
independent variables on a dependent variable. Essential for 
understanding yield contributing factors.
}
\examples{
data <- data.frame(
  yield = c(1200, 1350, 1100, 1450, 1280, 1380, 1220, 1400),
  pods = c(45, 52, 42, 58, 48, 54, 46, 56),
  seeds = c(8.2, 9.1, 7.8, 9.5, 8.5, 9.0, 8.3, 9.3),
  weight = c(32, 35, 30, 38, 33, 36, 31, 37)
)
path_analysis(data, dependent = "yield", 
              independent = c("pods", "seeds", "weight"))

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_basic.R
\name{anova_crd}
\alias{anova_crd}
\title{============================================================================
COMPREHENSIVE EXPERIMENTAL DESIGN ANALYSIS FUNCTIONS
Package: aridagri
Author: Lalit Kumar Rolaniya
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
============================================================================
Completely Randomized Design (CRD) ANOVA}
\usage{
anova_crd(
  data,
  response,
  treatment,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable (as string)}

\item{treatment}{Name of treatment factor}

\item{posthoc}{Post-hoc test: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"}

\item{alpha}{Significance level (default 0.05)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table, means, post-hoc results, and diagnostics
}
\description{
Performs complete ANOVA for Completely Randomized Design with post-hoc tests,
assumptions checking, and publication-ready output.
}
\examples{
data <- data.frame(
  treatment = rep(c("T1", "T2", "T3", "T4"), each = 5),
  yield = c(rnorm(5, 1200, 50), rnorm(5, 1350, 60), 
            rnorm(5, 1100, 55), rnorm(5, 1450, 65))
)
anova_crd(data, response = "yield", treatment = "treatment", posthoc = "all")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistical_functions.R
\name{pca_analysis}
\alias{pca_analysis}
\title{Principal Component Analysis}
\usage{
pca_analysis(data, scale = TRUE, ncp = 5, plot = TRUE, verbose = TRUE)
}
\arguments{
\item{data}{Data frame with numeric variables}

\item{scale}{Logical, whether to scale variables (default TRUE)}

\item{ncp}{Number of components to retain (default 5)}

\item{plot}{Logical, whether to generate plots}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
PCA results with eigenvalues, loadings, and scores
}
\description{
Performs PCA with visualization suitable for agricultural research data.
Includes scree plot, biplot, and variable contributions.
}
\examples{
data <- data.frame(
  yield = rnorm(30, 1200, 200),
  wue = rnorm(30, 4.5, 0.5),
  protein = rnorm(30, 22, 2),
  biomass = rnorm(30, 3500, 500)
)
pca_analysis(data)

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_split_plot.R
\name{anova_spd_ab_cd}
\alias{anova_spd_ab_cd}
\title{Split Plot Design with (AB) Main and (CD) Sub}
\usage{
anova_spd_ab_cd(
  data,
  response,
  main_factor1,
  main_factor2,
  sub_factor1,
  sub_factor2,
  replication,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{main_factor1}{First factor in main plot (A)}

\item{main_factor2}{Second factor in main plot (B)}

\item{sub_factor1}{First factor in sub-plot (C)}

\item{sub_factor2}{Second factor in sub-plot (D)}

\item{replication}{Name of replication factor}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table and means
}
\description{
Performs ANOVA for Split Plot Design where main plot contains factorial 
combination of AB and sub-plot contains factorial combination of CD.
Complex design for multi-factor experiments.
}
\examples{
# Example: Irrigation  Tillage (main), Variety  Nitrogen (sub)
data <- expand.grid(
  rep = 1:3,
  irrigation = c("I1", "I2"),
  tillage = c("CT", "ZT"),
  variety = c("V1", "V2"),
  nitrogen = c("N1", "N2", "N3")
)
data$yield <- rnorm(nrow(data), 1200, 150)

anova_spd_ab_cd(data, response = "yield",
                main_factor1 = "irrigation", main_factor2 = "tillage",
                sub_factor1 = "variety", sub_factor2 = "nitrogen",
                replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_basic.R
\name{anova_rbd}
\alias{anova_rbd}
\title{Randomized Block Design (RBD) ANOVA}
\usage{
anova_rbd(
  data,
  response,
  treatment,
  block,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{treatment}{Name of treatment factor}

\item{block}{Name of block/replication factor}

\item{posthoc}{Post-hoc test: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"}

\item{alpha}{Significance level (default 0.05)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table, means, post-hoc results
}
\description{
Performs complete ANOVA for Randomized Block Design (RCBD) with post-hoc tests,
assumptions checking, and publication-ready output.
}
\examples{
data <- data.frame(
  rep = rep(1:4, each = 5),
  treatment = rep(c("T1", "T2", "T3", "T4", "T5"), 4),
  yield = c(rnorm(5, 1200, 50), rnorm(5, 1250, 55),
            rnorm(5, 1180, 45), rnorm(5, 1270, 60))
)
anova_rbd(data, response = "yield", treatment = "treatment", 
          block = "rep", posthoc = "duncan")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posthoc_helpers.R
\name{check_assumptions}
\alias{check_assumptions}
\title{Check ANOVA Assumptions}
\usage{
check_assumptions(model, verbose = TRUE)
}
\arguments{
\item{model}{ANOVA model object}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing diagnostic test results
}
\description{
Tests assumptions of ANOVA: normality and homogeneity of variances.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{stability_analysis}
\alias{stability_analysis}
\title{============================================================================
ADVANCED AGRONOMIC STATISTICAL METHODS
Package: aridagri
Authors: Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
ICAR-Indian Institute of Groundnut Research, Regional Research Station, Bikaner
============================================================================
Stability Analysis for Agronomic Traits (Multiple Methods)}
\usage{
stability_analysis(
  data,
  genotype,
  environment,
  replication,
  trait,
  method = "all",
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame with genotype/treatment, environment, replication, and trait}

\item{genotype}{Name of genotype/treatment column}

\item{environment}{Name of environment/location/year column}

\item{replication}{Name of replication column}

\item{trait}{Name of trait/response variable}

\item{method}{Method: "eberhart", "ammi", "finlay", "shukla", "wricke", "cv", "superiority", or "all"}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing stability parameters and rankings
}
\description{
Performs comprehensive stability analysis using multiple established methods for 
evaluating genotype/treatment performance across environments.

Methods included:
\itemize{
  \item Eberhart & Russell (1966): Regression approach
  \item AMMI: Additive Main effects and Multiplicative Interaction
  \item Finlay & Wilkinson (1963): Linear regression on environmental mean
  \item Shukla (1972): Stability variance
  \item Wricke (1962): Ecovalence
  \item Coefficient of Variation: CV-based ranking
  \item Superiority Index: Lin & Binns (1988)
}
}
\examples{
data <- expand.grid(
  variety = paste0("V", 1:10),
  location = paste0("L", 1:5),
  rep = 1:3
)
data$yield <- rnorm(nrow(data), 1200, 200)

stability_analysis(data, genotype = "variety", environment = "location",
                   replication = "rep", trait = "yield", method = "all")

}
\references{
Eberhart, S.A. and Russell, W.A. (1966). Crop Science, 6: 36-40.
}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aridagri-package.R
\name{print.aridagri_pca}
\alias{print.aridagri_pca}
\title{Print Method for PCA Results}
\usage{
\method{print}{aridagri_pca}(x, ...)
}
\arguments{
\item{x}{An object of class 'aridagri_pca' from \code{\link{pca_analysis}}}

\item{...}{Additional arguments (currently unused)}
}
\value{
No return value, called for side effects. Prints the number of
  components retained by Kaiser criterion and cumulative variance explained
  to the console. The input object is returned invisibly.
}
\description{
Prints a formatted summary of Principal Component Analysis (PCA) results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_additional.R
\name{anova_sspd_pooled}
\alias{anova_sspd_pooled}
\title{Pooled Split-Split Plot Design ANOVA}
\usage{
anova_sspd_pooled(
  data,
  response,
  main_plot,
  sub_plot,
  sub_sub_plot,
  environment,
  replication,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing combined data}

\item{response}{Name of the response variable}

\item{main_plot}{Name of main plot factor}

\item{sub_plot}{Name of sub-plot factor}

\item{sub_sub_plot}{Name of sub-sub-plot factor}

\item{environment}{Name of environment factor}

\item{replication}{Name of replication factor}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing pooled ANOVA results
}
\description{
Performs pooled ANOVA for SSPD experiments across multiple environments.
}
\examples{
data <- expand.grid(env = c("E1","E2"), rep = 1:3, A = c("A1","A2"), 
                    B = c("B1","B2"), C = c("C1","C2"))
data$yield <- rnorm(nrow(data), 1200, 150)
anova_sspd_pooled(data, "yield", "A", "B", "C", "env", "rep")

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posthoc_helpers.R
\name{anova_latin}
\alias{anova_latin}
\title{Latin Square Design ANOVA}
\usage{
anova_latin(
  data,
  response,
  treatment,
  row,
  column,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the data}

\item{response}{Name of the response variable}

\item{treatment}{Name of treatment factor}

\item{row}{Name of row factor}

\item{column}{Name of column factor}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results
}
\description{
Performs ANOVA for Latin Square Design with row and column blocking.
}
\examples{
# 5x5 Latin Square
data <- data.frame(
  row = rep(1:5, each = 5),
  col = rep(1:5, 5),
  treatment = c("A","B","C","D","E", "B","C","D","E","A",
                "C","D","E","A","B", "D","E","A","B","C",
                "E","A","B","C","D"),
  yield = rnorm(25, 1200, 100)
)
anova_latin(data, response = "yield", treatment = "treatment",
            row = "row", column = "col")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_basic.R
\name{anova_rbd_pooled}
\alias{anova_rbd_pooled}
\title{Pooled Analysis of RBD Experiments (Multi-Environment/Year)}
\usage{
anova_rbd_pooled(
  data,
  response,
  treatment,
  environment,
  block,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing combined data from all environments}

\item{response}{Name of the response variable}

\item{treatment}{Name of treatment factor}

\item{environment}{Name of environment/year/location factor}

\item{block}{Name of block factor (nested within environment)}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing pooled ANOVA, individual ANOVAs, and interaction analysis
}
\description{
Performs pooled ANOVA for RBD experiments conducted across multiple 
environments, years, or locations. Tests homogeneity of error variances
using Bartlett's test before pooling.
}
\examples{
# Data from 3 years
data <- data.frame(
  year = rep(c("Y1", "Y2", "Y3"), each = 20),
  rep = rep(rep(1:4, each = 5), 3),
  treatment = rep(c("T1", "T2", "T3", "T4", "T5"), 12),
  yield = rnorm(60, 1200, 150)
)
anova_rbd_pooled(data, response = "yield", treatment = "treatment",
                 environment = "year", block = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{harvest_index}
\alias{harvest_index}
\title{Harvest Index Calculation}
\usage{
harvest_index(grain_yield, straw_yield, verbose = TRUE)
}
\arguments{
\item{grain_yield}{Grain/economic yield}

\item{straw_yield}{Straw/stover yield}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with harvest indices
}
\description{
Calculates Harvest Index and related partitioning indices.
}
\examples{
harvest_index(grain_yield = c(1200, 1350, 1100), straw_yield = c(2400, 2500, 2300))

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posthoc_helpers.R
\name{perform_posthoc}
\alias{perform_posthoc}
\title{Perform Post-Hoc Tests}
\usage{
perform_posthoc(
  model,
  data,
  response,
  treatment,
  mse,
  df_error,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{model}{ANOVA model object}

\item{data}{Data frame}

\item{response}{Response variable name}

\item{treatment}{Treatment factor name}

\item{mse}{Mean square error}

\item{df_error}{Error degrees of freedom}

\item{posthoc}{Test type: "lsd", "duncan", "tukey", "snk", "scheffe", or "all"}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing post-hoc test results
}
\description{
Performs multiple comparison tests after ANOVA.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistical_functions.R
\name{correlation_analysis}
\alias{correlation_analysis}
\title{Correlation Analysis with Significance}
\usage{
correlation_analysis(
  data,
  method = "pearson",
  plot = TRUE,
  digits = 3,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame with numeric variables}

\item{method}{Correlation method: "pearson", "spearman", or "kendall"}

\item{plot}{Logical, whether to generate correlation plot}

\item{digits}{Number of decimal places}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List with correlation matrix and significance matrix
}
\description{
Computes correlation matrix with significance levels and generates
publication-ready correlation table and plot.
}
\examples{
data <- data.frame(
  yield = c(1200, 1350, 1100, 1450, 1280),
  wue = c(4.2, 4.8, 3.9, 5.1, 4.5),
  protein = c(22.1, 23.5, 21.8, 24.2, 22.9)
)
correlation_analysis(data)

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_additional.R
\name{anova_factorial_3way}
\alias{anova_factorial_3way}
\title{Three-Factor Factorial ANOVA}
\usage{
anova_factorial_3way(
  data,
  response,
  factor1,
  factor2,
  factor3,
  replication = NULL,
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the data}

\item{response}{Name of the response variable}

\item{factor1}{Name of first factor (A)}

\item{factor2}{Name of second factor (B)}

\item{factor3}{Name of third factor (C)}

\item{replication}{Name of replication factor (optional)}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results
}
\description{
Performs three-factor factorial ANOVA with all interactions.
}
\examples{
data <- expand.grid(rep = 1:3, A = c("A1", "A2"), B = c("B1", "B2"), C = c("C1", "C2"))
data$yield <- rnorm(nrow(data), 1200, 150)
anova_factorial_3way(data, "yield", "A", "B", "C", "rep")

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nutrient_functions.R
\name{economic_analysis}
\alias{economic_analysis}
\title{Economic Analysis for Arid Agriculture}
\usage{
economic_analysis(
  yield,
  price,
  cost_fixed,
  cost_variable,
  byproduct_yield = 0,
  byproduct_price = 0,
  verbose = TRUE
)
}
\arguments{
\item{yield}{Crop yield (kg/ha)}

\item{price}{Market price (Rs/kg)}

\item{cost_fixed}{Fixed costs (Rs/ha)}

\item{cost_variable}{Variable costs (Rs/ha)}

\item{byproduct_yield}{Byproduct yield (kg/ha), optional}

\item{byproduct_price}{Byproduct price (Rs/kg), optional}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with economic analysis
}
\description{
Calculates economic indicators including Cost of Cultivation, Gross Returns,
Net Returns, B:C Ratio, and profitability indices for arid farming systems.
}
\examples{
economic_analysis(yield = 1200, price = 65, cost_fixed = 15000, cost_variable = 12000)

# With byproduct
economic_analysis(yield = 1200, price = 65, cost_fixed = 15000, 
                  cost_variable = 12000, byproduct_yield = 1800, byproduct_price = 5)

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_split_plot.R
\name{anova_spd_ab_main}
\alias{anova_spd_ab_main}
\title{Split Plot Design with AB in Main Plot}
\usage{
anova_spd_ab_main(
  data,
  response,
  main_factor1,
  main_factor2,
  sub_plot,
  replication,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{main_factor1}{First factor in main plot (A)}

\item{main_factor2}{Second factor in main plot (B)}

\item{sub_plot}{Sub-plot factor (C)}

\item{replication}{Name of replication factor}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table and means
}
\description{
Performs ANOVA for Split Plot Design where main plot contains factorial 
combination of two factors (AB) and sub-plot contains factor C.
Common in irrigation  variety as main plot and nitrogen as sub-plot.
}
\details{
Design structure:
\itemize{
  \item Main Plot: A  B factorial
  \item Sub-Plot: C
  \item Error (a): For testing A, B, and AB
  \item Error (b): For testing C and all interactions with C
}
}
\examples{
data <- data.frame(
  rep = rep(1:3, each = 24),
  irrigation = rep(rep(c("I1", "I2"), each = 12), 3),
  variety = rep(rep(c("V1", "V2", "V3"), each = 4), 6),
  nitrogen = rep(c("N0", "N1", "N2", "N3"), 18),
  yield = rnorm(72, 1200, 150)
)
anova_spd_ab_main(data, response = "yield", 
                  main_factor1 = "irrigation", main_factor2 = "variety",
                  sub_plot = "nitrogen", replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aridagri-package.R
\name{arid_plot}
\alias{arid_plot}
\title{Visualization Functions for aridagri}
\usage{
arid_plot(x, type = "bar", ...)
}
\arguments{
\item{x}{An object from aridagri analysis functions}

\item{type}{Plot type: "bar", "line", "interaction", "boxplot"}

\item{...}{Additional arguments passed to plotting functions}
}
\value{
No return value; called for its side effect of drawing a plot on the
current graphics device. The plot produced depends on the class of \code{x}:
a correlation heatmap for \code{aridagri_correlation} objects, a bar chart of
treatment means for \code{aridagri_anova} objects, and a bar chart of the
integrated stability ranking for \code{aridagri_stability} objects. Uses only
base graphics, so no additional packages are required.
}
\description{
Generate publication-quality plots for aridagri analyses.
}
\examples{
df <- data.frame(
  yield = c(1200, 1350, 1100, 1450, 1280),
  wue = c(4.2, 4.8, 3.9, 5.1, 4.5),
  protein = c(22.1, 23.5, 21.8, 24.2, 22.9)
)
result <- correlation_analysis(df, plot = FALSE)
arid_plot(result)

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_split_plot.R
\name{anova_spd_pooled}
\alias{anova_spd_pooled}
\title{Pooled Split Plot Design Analysis}
\usage{
anova_spd_pooled(
  data,
  response,
  main_plot,
  sub_plot,
  environment,
  replication,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing combined data}

\item{response}{Name of the response variable}

\item{main_plot}{Name of main plot factor}

\item{sub_plot}{Name of sub-plot factor}

\item{environment}{Name of environment factor}

\item{replication}{Name of replication factor (nested within environment)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing pooled ANOVA and component analyses
}
\description{
Performs pooled analysis of Split Plot Design experiments conducted 
across multiple environments/years/locations.
}
\examples{
data <- expand.grid(
  year = c("Y1", "Y2", "Y3"),
  rep = 1:3,
  irrigation = c("I1", "I2", "I3"),
  variety = c("V1", "V2", "V3", "V4")
)
data$yield <- rnorm(nrow(data), 1200, 180)

anova_spd_pooled(data, response = "yield", main_plot = "irrigation",
                 sub_plot = "variety", environment = "year", replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistical_functions.R
\name{sem_analysis}
\alias{sem_analysis}
\title{Structural Equation Modeling for Field Experiments}
\usage{
sem_analysis(data, model, plot = TRUE, verbose = TRUE)
}
\arguments{
\item{data}{Data frame with variables}

\item{model}{Model specification in lavaan syntax}

\item{plot}{Logical, whether to generate path diagram}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
SEM results including fit indices and parameter estimates
}
\description{
Performs SEM analysis for agricultural field experiments. Allows testing
of hypothesized causal relationships among variables.
}
\examples{
\donttest{
if (requireNamespace("lavaan", quietly = TRUE)) {
  set.seed(42)
  n <- 100
  nitrogen <- rnorm(n, 60, 10)
  phosphorus <- rnorm(n, 30, 5)
  yield <- 0.5 * nitrogen + 0.3 * phosphorus + rnorm(n, 0, 5)
  df <- data.frame(yield = yield, nitrogen = nitrogen, phosphorus = phosphorus)
  model <- 'yield ~ nitrogen + phosphorus'
  result <- sem_analysis(df, model, plot = FALSE)
}
}

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aridagri-package.R
\name{export_results}
\alias{export_results}
\title{Export Results to Publication Format}
\usage{
export_results(x, file, format = "xlsx", digits = 3, verbose = TRUE)
}
\arguments{
\item{x}{An object from aridagri analysis functions}

\item{file}{Output file path}

\item{format}{Output format: \code{"xlsx"} (default) or \code{"csv"}}

\item{digits}{Number of decimal places}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Invisibly returns the file path
}
\description{
Exports aridagri analysis results to Excel format for publication.
}
\examples{
\donttest{
df <- data.frame(
  yield = c(1200, 1350, 1100, 1450, 1280),
  wue = c(4.2, 4.8, 3.9, 5.1, 4.5)
)
result <- correlation_analysis(df, plot = FALSE)
export_results(result, tempfile(fileext = ".xlsx"))
}

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nutrient_functions.R
\name{nue_calculate}
\alias{nue_calculate}
\title{Nutrient Use Efficiency Calculations}
\usage{
nue_calculate(
  yield_fertilized,
  yield_control,
  nutrient_applied,
  nutrient_uptake_fert = NULL,
  nutrient_uptake_ctrl = NULL,
  biomass_fert = NULL,
  biomass_ctrl = NULL,
  verbose = TRUE
)
}
\arguments{
\item{yield_fertilized}{Yield with fertilizer application (kg/ha)}

\item{yield_control}{Yield in control/unfertilized plot (kg/ha)}

\item{nutrient_applied}{Amount of nutrient applied (kg/ha)}

\item{nutrient_uptake_fert}{Nutrient uptake in fertilized plot (kg/ha), optional}

\item{nutrient_uptake_ctrl}{Nutrient uptake in control plot (kg/ha), optional}

\item{biomass_fert}{Total biomass in fertilized plot (kg/ha), optional}

\item{biomass_ctrl}{Total biomass in control plot (kg/ha), optional}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with efficiency indices
}
\description{
Comprehensive nutrient use efficiency calculations including Agronomic Efficiency (AE),
Physiological Efficiency (PE), Apparent Recovery Efficiency (ARE), and Partial Factor
Productivity (PFP). Essential for INM research in arid regions.
}
\details{
Efficiency calculations:
\itemize{
  \item AE (Agronomic Efficiency) = (Yield_fert - Yield_ctrl) / Nutrient_applied
  \item PFP (Partial Factor Productivity) = Yield_fert / Nutrient_applied
  \item ARE (Apparent Recovery Efficiency) = (Uptake_fert - Uptake_ctrl) / Nutrient_applied  100
  \item PE (Physiological Efficiency) = (Yield_fert - Yield_ctrl) / (Uptake_fert - Uptake_ctrl)
}
}
\examples{
# Basic NUE calculation
nue_calculate(yield_fertilized = 1850, yield_control = 1200, nutrient_applied = 40)

# Complete NUE with uptake data
nue_calculate(
  yield_fertilized = 1850, 
  yield_control = 1200,
  nutrient_applied = 40,
  nutrient_uptake_fert = 65,
  nutrient_uptake_ctrl = 35
)

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_additional.R
\name{anova_alpha_lattice}
\alias{anova_alpha_lattice}
\title{Alpha Lattice Design ANOVA}
\usage{
anova_alpha_lattice(
  data,
  response,
  genotype,
  replication,
  block,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the data}

\item{response}{Name of response variable}

\item{genotype}{Name of genotype column}

\item{replication}{Name of replication column}

\item{block}{Name of incomplete block column (nested within replication)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA results
}
\description{
Performs ANOVA for Alpha Lattice (Resolvable Incomplete Block) Design.
}
\examples{
data <- expand.grid(rep = 1:2, block = 1:5, entry = 1:4)
data$genotype <- paste0("G", 1:nrow(data))
data$yield <- rnorm(nrow(data), 1200, 150)

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{yield_gap_analysis}
\alias{yield_gap_analysis}
\title{Yield Gap Analysis}
\usage{
yield_gap_analysis(
  actual_yield,
  potential_yield,
  attainable_yield = NULL,
  verbose = TRUE
)
}
\arguments{
\item{actual_yield}{Actual yield (kg/ha)}

\item{potential_yield}{Potential yield (kg/ha)}

\item{attainable_yield}{Attainable yield (kg/ha, optional)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with yield gap analysis
}
\description{
Calculates yield gaps comparing actual with potential yields.
}
\examples{
yield_gap_analysis(actual_yield = c(800, 950, 720), potential_yield = 1500)

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_split_plot.R
\name{anova_spd}
\alias{anova_spd}
\title{============================================================================
SPLIT PLOT DESIGN ANALYSIS FUNCTIONS (All Variations)
Package: aridagri
Author: Lalit Kumar Rolaniya
ICAR-Indian Institute of Pulses Research, Regional Centre, Bikaner
============================================================================
Split Plot Design ANOVA (Standard)}
\usage{
anova_spd(
  data,
  response,
  main_plot,
  sub_plot,
  replication,
  posthoc = "lsd",
  alpha = 0.05,
  verbose = TRUE
)
}
\arguments{
\item{data}{Data frame containing the experimental data}

\item{response}{Name of the response variable}

\item{main_plot}{Name of main plot factor}

\item{sub_plot}{Name of sub-plot factor}

\item{replication}{Name of replication/block factor}

\item{posthoc}{Post-hoc test method}

\item{alpha}{Significance level}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List containing ANOVA table, means, and post-hoc results
}
\description{
Performs complete ANOVA for Split Plot Design with proper error terms for 
main plot and sub-plot factors. Includes all standard post-hoc comparisons.
}
\examples{
data <- data.frame(
  rep = rep(1:3, each = 12),
  irrigation = rep(rep(c("I1", "I2", "I3"), each = 4), 3),
  variety = rep(c("V1", "V2", "V3", "V4"), 9),
  yield = rnorm(36, 1200, 150)
)
anova_spd(data, response = "yield", main_plot = "irrigation", 
          sub_plot = "variety", replication = "rep")

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nutrient_functions.R
\name{nutrient_response}
\alias{nutrient_response}
\title{Nutrient Response Curve Analysis}
\usage{
nutrient_response(
  dose,
  yield,
  model = "quadratic",
  price_output = 50,
  price_nutrient = 15,
  verbose = TRUE
)
}
\arguments{
\item{dose}{Numeric vector of nutrient doses (kg/ha)}

\item{yield}{Numeric vector of corresponding yields (kg/ha)}

\item{model}{Model type: "quadratic", "linear_plateau", or "mitscherlich"}

\item{price_output}{Price of output (Rs/kg)}

\item{price_nutrient}{Price of nutrient (Rs/kg)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
List with model parameters and economic optimum
}
\description{
Fits nutrient response curves using quadratic, linear-plateau, or 
Mitscherlich models to determine economic optimum dose.
}
\examples{
dose <- c(0, 20, 40, 60, 80, 100)
yield <- c(1100, 1350, 1520, 1610, 1650, 1660)
nutrient_response(dose, yield, model = "quadratic", 
                  price_output = 60, price_nutrient = 15)

}
\author{
Lalit Kumar Rolaniya, ICAR-IIPR, Bikaner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agronomic_analysis.R
\name{economic_indices}
\alias{economic_indices}
\title{Economic Efficiency Indices (B:C Ratio)}
\usage{
economic_indices(gross_return, total_cost, verbose = TRUE)
}
\arguments{
\item{gross_return}{Gross returns (Rs/ha)}

\item{total_cost}{Total cost (Rs/ha)}

\item{verbose}{Logical. If TRUE (default), prints formatted output to console.}
}
\value{
Data frame with economic indices
}
\description{
Calculates B:C ratio and economic efficiency metrics.
}
\examples{
economic_indices(gross_return = c(75000, 82000), total_cost = c(35000, 38000))

}
\author{
Lalit Kumar Rolaniya, Ram Lal Jat, Monika Punia, Raja Ram Choudhary
}
